/* 파일 설명 적는곳

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#include <unistd.h>
#include <glib/gstdio.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#include <glade/glade-xml.h>

#include "otak-lib/otak-util/otsys.h"
#include "otak-lib/otak-util/otstring.h"
#include "otak-lib/otak-util/otxml.h"
#include "otak-lib/otak-widget/otvalue.h"
#include "otak-lib/otak-widget/otgrid.h"
#include "odmvalidation.h"
#include "widget/odmdialog.h"
#include "type/odmconninfodefine.h"
#include "type/odmconninfofactory.h"
#include "odmconf.h"
#include "odmmainui.h"
#include "odmconnui.h"
#include "../config-path.h"

#define ODM_CONNUI_LST_CLM_CNT 7

#ifndef _OTAK_DBMANAGER_PATH_H_
#define INST_PREFIX "/usr"
#endif

static
const gchar *COLUMN_TITLE[ODM_CONNUI_LST_CLM_CNT] =
{
    "Name",
    "Host",
    "Port",
    "Database",
    "Username",
    "Password",
    "Encoding"
};

static
const gchar *COLUMN_XML_NAME[ODM_CONNUI_LST_CLM_CNT] =
{
    "name",
    "host",
    "port",
    "database",
    "username",
    "password",
    "encoding"
};

static gboolean init = FALSE;

static GladeXML *conn_glade_xml = NULL;

static xmlDocPtr     serv_list_doc      = NULL;
static GPtrArray    *form_value_list    = NULL;
static OtGrid       *conn_list_grid     = NULL;


static GladeXML*    odm_connui_get_glade_xml (void);
static GtkWidget*   odm_connui_get_window (void);
static GtkWidget*   odm_connui_get_entry_name (void);
static GtkWidget*   odm_connui_get_entry_host (void);
static GtkWidget*   odm_connui_get_entry_port (void);
static GtkWidget*   odm_connui_get_entry_database (void);
static GtkWidget*   odm_connui_get_entry_username (void);
static GtkWidget*   odm_connui_get_entry_password (void);
static void         odm_connui_close (void);
static void         odm_connui_reset_form_values (void);
static void         odm_connui_server_list_set_row_from_node (xmlNodePtr node,
                            GtkTreeIter *iter);
static void         odm_connui_server_list_save (void);
void                odm_connui_connect (void);



/*
 * signal
 */

void
window_conn_show_cb (GtkWidget  *widget,
                     gpointer    user_data)
{
    gtk_widget_grab_focus (odm_connui_get_entry_name ());
}

/* window delete */
gboolean
window_conn_delete_event_cb (GtkWidget  *widget,
                             GdkEvent   *event,
                             gpointer    user_data)
{
    odm_connui_close ();

    return TRUE;
}


void
button_conn_list_delete_clicked_cb (GtkButton   *button,
                                    gpointer     user_data)
{
    GtkTreeSelection    *sel    = NULL;
    GtkTreeModel        *model  = NULL;
    GtkTreeIter          iter;
    xmlNodePtr           nd     = NULL;

    sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (conn_list_grid));
    if (gtk_tree_selection_get_selected (sel, &model, &iter))
    {
        gtk_tree_model_get (model, &iter, ODM_CONNUI_LST_CLM_CNT, &nd, -1);

        ot_xml_remove_node (nd);
        gtk_list_store_remove (GTK_LIST_STORE (model), &iter);

        odm_connui_server_list_save ();
    }
}

void
button_conn_list_delete_all_clicked_cb (GtkButton   *button,
                                        gpointer     user_data)
{
    xmlNodePtr  r_nd    = xmlDocGetRootElement (serv_list_doc);
    gboolean    is_del  = FALSE;

    while (ot_xml_remove_child_node (r_nd, "server", 0))
    {
        is_del = TRUE;
    }

    if (is_del)
    {
        ot_grid_clear_datas (conn_list_grid);
        odm_connui_server_list_save ();
    }
}

void
button_conn_info_save_clicked_cb (GtkButton *button,
                                  gpointer   user_data)
{
    GtkTreeSelection    *sel    = NULL;
    GtkTreeModel        *model  = NULL;
    GtkTreeIter          iter;
    GtkTreeIter         *f_iter = NULL;
    xmlNodePtr           nd     = NULL;

    sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (conn_list_grid));
    if (gtk_tree_selection_get_selected (sel, &model, &iter))
    {
        f_iter = &iter;
        gtk_tree_model_get (model, f_iter, ODM_CONNUI_LST_CLM_CNT, &nd, -1);
    }
    /* 신규 접속 정보 */
    else
    {
        nd = ot_xml_add_child_node (xmlDocGetRootElement (serv_list_doc),
                "server", NULL);
    }

    gint i = 0;
    for (i = 0; i < ODM_CONNUI_LST_CLM_CNT; i++)
    {
        OtValue     *ot_val = NULL;
        xmlNodePtr   c_nd   = NULL;
        const gchar *val    = NULL;

        ot_val = (OtValue *) g_ptr_array_index (form_value_list, i);
        val = ot_value_get_value (ot_val);

        c_nd = ot_xml_get_child_node (nd, COLUMN_XML_NAME[i], 0, NULL);
        if (c_nd == NULL)
        {
            ot_xml_add_child_node (nd, COLUMN_XML_NAME[i], val);
        }
        else
        {
            ot_xml_set_value (c_nd, val);
        }
    }

    odm_connui_server_list_set_row_from_node (nd, f_iter);

    odm_connui_server_list_save ();
}

void
button_conn_info_new_clicked_cb (GtkButton  *button,
                                 gpointer    user_data)
{
    ot_grid_unselect_all (conn_list_grid);
    odm_connui_reset_form_values ();
}

void
button_conn_info_connect_clicked_cb (GtkButton  *button,
                                     gpointer    user_data)
{
    odm_connui_connect ();
}

void
button_conn_info_close_clicked_cb (GtkButton    *button,
                                   gpointer      user_data)
{
    odm_connui_close ();
}

static void
grid_conn_list_cursor_changed_cb (GtkTreeView   *treeview,
                                  gpointer       user_data)
{
    GtkTreeSelection    *sel    = gtk_tree_view_get_selection (treeview);
    GtkTreeModel        *model  = NULL;
    GtkTreeIter          iter;

    if (gtk_tree_selection_get_selected (sel, &model, &iter))
    {
        gint i = 0;
        for (i = 0; i < ODM_CONNUI_LST_CLM_CNT; i++)
        {
            OtValue *val        = g_ptr_array_index (form_value_list, i);
            gchar   *row_val    = NULL;

            gtk_tree_model_get (model, &iter, i, &row_val, -1);
            ot_value_set_value (val, row_val);
            row_val = ot_sys_g_free (row_val);
        }
    }
}

static void
grid_conn_list_row_activated_cb (GtkTreeView        *treeview,
                                 GtkTreePath        *path,
                                 GtkTreeViewColumn  *column,
                                 gpointer            user_data)
{
    odm_connui_connect ();
}



/*
 * private
 */

/*
 * odm_connui_get_glade_xml:
 * 
 * 화면의 위젯을 호출할 수 있는 GladeXML* 을 반환한다.
 * 
 * Return value: libglade용 GladeXML* 또는 NULL
 */
static GladeXML*
odm_connui_get_glade_xml (void)
{
    if (conn_glade_xml == NULL)
    {
        gchar *filepath = g_strdup_printf ("%s/otak-dbmanager-conncub.glade",
                odm_conf_get_glade_abs_dir ());
        if (0 != g_access ((const gchar *)filepath, R_OK))
        {
            filepath = ot_sys_g_free (filepath);
            filepath = g_strdup_printf (INST_PREFIX"/share/otak-dbmanager/otak-dbmanager-conncub.glade");
            if (0 != g_access ((const gchar *)filepath, R_OK))
            {
                filepath = ot_sys_g_free (filepath);
                filepath = g_strdup_printf ("%s/otak-dbmanager-conncub.glade",
                    odm_conf_get_glade_abs_dir ());
            }
        }

        conn_glade_xml = glade_xml_new (filepath, NULL, NULL);
        glade_xml_signal_autoconnect (conn_glade_xml);

        filepath = ot_sys_g_free (filepath);
    }

    return conn_glade_xml;
}

/*
 * odm_connui_get_window:
 * 
 * 화면의 윈도우를 반환한다.
 * 
 * Return value: 바탕격인 위젯 또는 NULL
 */
static GtkWidget*
odm_connui_get_window (void)
{
    return glade_xml_get_widget (odm_connui_get_glade_xml(), "window_conn");
}

/*
 * odm_connui_get_scrolledwindow_conn_list:
 * 
 * 화면의 connection list scrolled window 반환한다.
 * 
 * Return value: 바탕격인 위젯 또는 NULL
 */
static GtkWidget*
odm_connui_get_scrolledwindow_conn_list (void)
{
    return glade_xml_get_widget (odm_connui_get_glade_xml(),
            "scrolledwindow_conn_list");
}

/*
 * odm_connui_get_entry_name:
 * 
 * 접속정보 입력창 name entry 반환
 * 
 * Return value: 입력창 name entry 위젯 또는 NULL
 */
static GtkWidget*
odm_connui_get_entry_name (void)
{
    return glade_xml_get_widget (odm_connui_get_glade_xml(),
            "entry_conn_info_name");
}

static GtkWidget*
odm_connui_get_entry_host (void)
{
    return glade_xml_get_widget (odm_connui_get_glade_xml(),
            "entry_conn_info_host");
}

static GtkWidget*
odm_connui_get_entry_port (void)
{
    return glade_xml_get_widget (odm_connui_get_glade_xml(), "entry_conn_info_port");
}

static GtkWidget*
odm_connui_get_entry_database (void)
{
    return glade_xml_get_widget (odm_connui_get_glade_xml(), "entry_conn_info_database");
}

static GtkWidget*
odm_connui_get_entry_username (void)
{
    return glade_xml_get_widget (odm_connui_get_glade_xml(), "entry_conn_info_username");
}

static GtkWidget*
odm_connui_get_entry_password (void)
{
    return glade_xml_get_widget (odm_connui_get_glade_xml(), "entry_conn_info_password");
}

static GtkWidget*
odm_connui_get_entry_encoding (void)
{
    return glade_xml_get_widget (odm_connui_get_glade_xml(), "entry_conn_info_encoding");
}

static void
odm_connui_close (void)
{
    gtk_widget_hide (odm_connui_get_window ());
}

static void
odm_connui_reset_form_values (void)
{
    gint i = 0;
    for (i = 0; i < form_value_list->len; i++)
    {
        ot_value_set_value (
                (OtValue *) (g_ptr_array_index (form_value_list, i)), NULL);
    }
}

static void
odm_connui_conn_list_init (void)
{
    if (conn_list_grid == NULL)
    {
        GType        types[ODM_CONNUI_LST_CLM_CNT + 1];
        GtkWidget   *scrll                              = NULL;

        scrll = odm_connui_get_scrolledwindow_conn_list ();

        conn_list_grid = ot_grid_new ();
        gtk_widget_show (GTK_WIDGET (conn_list_grid));

        g_signal_connect ((gpointer) conn_list_grid, "cursor_changed",
                G_CALLBACK (grid_conn_list_cursor_changed_cb), NULL);
        g_signal_connect ((gpointer) conn_list_grid, "row_activated",
                G_CALLBACK (grid_conn_list_row_activated_cb), NULL);

        gint i = 0;
        for (i = 0; i < ODM_CONNUI_LST_CLM_CNT; i++)
        {
            types[i] = G_TYPE_STRING;
        }
        types[ODM_CONNUI_LST_CLM_CNT] = G_TYPE_POINTER;

        ot_grid_construct_columns (conn_list_grid, ODM_CONNUI_LST_CLM_CNT + 1,
                types, COLUMN_TITLE);

        ot_grid_set_resizable (conn_list_grid, TRUE);
        ot_grid_set_clickable (conn_list_grid, TRUE);

        gtk_container_add (GTK_CONTAINER (scrll),
                GTK_WIDGET (conn_list_grid));
    }
}

static void
odm_connui_server_list_init (void)
{
    if (serv_list_doc == NULL)
    {
        gchar *filepath = g_strdup_printf ("%s/server_list.xml",
                odm_conf_get_config_abs_dir ());
        if (0 != g_access ((const gchar *)filepath, R_OK))
        {
            filepath = ot_sys_g_free (filepath);
            filepath = g_strdup_printf (INST_PREFIX"/share/otak-dbmanager/server_list.xml");
            if (0 != g_access ((const gchar *)filepath, R_OK))
            {
                filepath = ot_sys_g_free (filepath);
                filepath = g_strdup_printf ("%s/server_list.xml",
                    odm_conf_get_glade_abs_dir ());
            }
        }

        serv_list_doc = ot_xml_new (filepath);

        filepath = ot_sys_g_free (filepath);
    }
}

static void
odm_connui_server_list_row_array_free (GPtrArray *array)
{
    gint i = 0;
    for (i = 0; i < ODM_CONNUI_LST_CLM_CNT; i++)
    {
        ot_sys_g_free (g_ptr_array_index (array, i));
    }
}

static GPtrArray*
odm_connui_dup_server_list_row_array_from_node (xmlNodePtr node)
{
    void
    add_value (GPtrArray *arr, xmlNodePtr node, const gchar *name)
    {
        xmlNodePtr   c_nd   = ot_xml_get_child_node (node, name, 0, NULL);
        gchar       *value  = NULL;

        if (c_nd != NULL)
        {
            value = ot_xml_dup_value (serv_list_doc, c_nd);
        }

        g_ptr_array_add (arr, value);
    }

    
    GPtrArray *arr = NULL;

    arr = g_ptr_array_new ();
    add_value (arr, node, COLUMN_XML_NAME[0]);
    add_value (arr, node, COLUMN_XML_NAME[1]);
    add_value (arr, node, COLUMN_XML_NAME[2]);
    add_value (arr, node, COLUMN_XML_NAME[3]);
    add_value (arr, node, COLUMN_XML_NAME[4]);
    add_value (arr, node, COLUMN_XML_NAME[5]);
    add_value (arr, node, COLUMN_XML_NAME[6]);
    g_ptr_array_add (arr, node);

    return arr;
}

static void
odm_connui_server_list_set_row_from_node (xmlNodePtr     node,
                                          GtkTreeIter   *iter)
{
    GPtrArray *arr = odm_connui_dup_server_list_row_array_from_node (node);

    if (iter == NULL)
    {
        ot_grid_append_row_from_array (conn_list_grid, arr);
    }
    else
    {
        ot_grid_set_row_from_array (conn_list_grid, iter, arr);
    }

    odm_connui_server_list_row_array_free (arr);
}

static void
odm_connui_server_list_refresh (void)
{
    xmlNodePtr r_nd = xmlDocGetRootElement (serv_list_doc);

    ot_grid_clear_datas (conn_list_grid);

    gint i = 0;
    while (TRUE)
    {
        xmlNodePtr   s_nd   = NULL;

        s_nd = ot_xml_get_child_node (r_nd, "server", i, NULL);
        if (s_nd == NULL)
        {
            break;
        }

        odm_connui_server_list_set_row_from_node (s_nd, NULL);

        i++;
    }
}

static void
odm_connui_server_list_save (void)
{
    gchar *filepath = g_strdup_printf ("%s/server_list.xml",
                        odm_conf_get_config_abs_dir ());

    xmlSaveFormatFile (filepath, serv_list_doc, 0);
    filepath = ot_sys_g_free (filepath);
}

static void
odm_connui_form_value_list_init (void)
{
    if (form_value_list == NULL)
    {
        // TODO : sohalee 아래 기본값 30000빼고 제거
        form_value_list = g_ptr_array_new ();

        g_ptr_array_add (form_value_list,
                ot_value_new (odm_connui_get_entry_name (),
                        ODM_CONN_INFO_FIELD_NAME_NAME, NULL,
                        OT_VALUE_VALID_REQUIRE));
        g_ptr_array_add (form_value_list,
                ot_value_new (odm_connui_get_entry_host (),
                        ODM_CONN_INFO_FIELD_NAME_HOST, NULL,
                        OT_VALUE_VALID_REQUIRE | OT_VALUE_VALID_ONEBYTE));
        g_ptr_array_add (form_value_list,
                ot_value_new (odm_connui_get_entry_port (),
                        ODM_CONN_INFO_FIELD_NAME_PORT, NULL,
                        OT_VALUE_VALID_REQUIRE | OT_VALUE_VALID_NUMERIC));
        g_ptr_array_add (form_value_list,
                ot_value_new (odm_connui_get_entry_database (),
                        ODM_CONN_INFO_FIELD_NAME_DATABASE, NULL,
                        OT_VALUE_VALID_REQUIRE | OT_VALUE_VALID_ONEBYTE));
        g_ptr_array_add (form_value_list,
                ot_value_new (odm_connui_get_entry_username (),
                        ODM_CONN_INFO_FIELD_NAME_USERNAME, NULL,
                        OT_VALUE_VALID_REQUIRE | OT_VALUE_VALID_ONEBYTE));
        g_ptr_array_add (form_value_list,
                ot_value_new (odm_connui_get_entry_password (),
                        ODM_CONN_INFO_FIELD_NAME_PASSWORD, NULL,
                        OT_VALUE_VALID_ONEBYTE));
        g_ptr_array_add (form_value_list,
                ot_value_new (odm_connui_get_entry_encoding (),
                        ODM_CONN_INFO_FIELD_NAME_ENCODING, NULL,
                        OT_VALUE_VALID_ONEBYTE));
    }
}

void
odm_connui_connect (void)
{
    gint i = 0;
    for (i = 0; i < form_value_list->len; i++)
    {
        OtValue *otvalue = (OtValue *) g_ptr_array_index (form_value_list, i);

        gchar *err_msg = odm_validation_dup_err_msg (otvalue);
    
        if (err_msg != NULL)
        {
            odm_dialog_message_modal (GTK_WINDOW (odm_connui_get_window ()),
                    err_msg, GTK_MESSAGE_ERROR);
            err_msg = ot_sys_g_free (err_msg);
            gtk_widget_grab_focus (otvalue->widget);

            return;
        }
    }

    OdmConnInfo *conn_info = odm_conn_info_factory_new_instance (
            odm_conf_get_conn_info_prov (), form_value_list);

    if (odm_mainui_add_connections (conn_info))
    {
        odm_connui_close ();
    }
}



/*
 * piblic
 */

void
odm_connui_init (void)
{
    if (!init)
    {
        odm_connui_server_list_init ();
        odm_connui_conn_list_init ();
        odm_connui_form_value_list_init ();

        init = TRUE;
    }
}

void
odm_connui_show (GtkWindow  *parent_window,
                 gboolean    modal)
{
    GtkWidget *window = odm_connui_get_window ();

    odm_connui_reset_form_values ();

    gtk_window_set_transient_for (GTK_WINDOW (window), parent_window);
    gtk_window_set_modal (GTK_WINDOW (window), modal);
    gtk_widget_show (window);

    odm_connui_server_list_refresh ();
}
