/* 파일 설명 적는곳

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#include <unistd.h>
#include <glib/gstdio.h>
#include <glib.h>
#include <glib-object.h>
#include <gdk/gdk.h>

#include "otak-lib/otak-util/otsys.h"
#include "odmconf.h"
#include "odmimagebank.h"
#include "../config-path.h"

#ifndef _OTAK_DBMANAGER_PATH_H_
#define INST_PREFIX "/usr"
#endif

static char *IMAGE_NAMES[ODM_IMG_BANK_COUNT] = {
    ODM_IMG_BNK_TABLE_22,
    ODM_IMG_BNK_INDEX_22,
    ODM_IMG_BNK_PROCEDURE_22,
    ODM_IMG_BNK_FUNCTION_22
};

static gboolean init = FALSE;

static GHashTable *image_table = NULL;

void
odm_image_bank_init (void)
{
    if (!init)
    {
        if (image_table == NULL)
        {
            gchar *image_path = NULL;

            image_table = g_hash_table_new_full (g_str_hash, g_str_equal,
                    NULL, g_object_unref);

            gint i = 0;
            for (i = 0; i < ODM_IMG_BANK_COUNT; i++)
            {
                 image_path = g_strdup_printf ("%s/%s",
                        odm_conf_get_image_abs_dir (), IMAGE_NAMES[i]);

                 if (0 != g_access ((const gchar *)image_path, R_OK))
                 {
                     image_path = ot_sys_g_free (image_path);
                     image_path = g_strdup_printf (INST_PREFIX"/share/otak-dbmanager/%s", IMAGE_NAMES[i]);
                     if (0 != g_access ((const gchar *)image_path, R_OK))
                     {
                         image_path = ot_sys_g_free (image_path);
                         image_path = g_strdup_printf ("%s/%s",
                             odm_conf_get_image_abs_dir (), IMAGE_NAMES[i]);
                     }
                }

                g_hash_table_insert (image_table, IMAGE_NAMES[i],
                        (gpointer) gdk_pixbuf_new_from_file (image_path, NULL));
                image_path = ot_sys_g_free (image_path);
            }
        }

        init = TRUE;
    }
}

GdkPixbuf*
odm_image_bank_get_pixbuf (const gchar *key)
{
    return g_hash_table_lookup (image_table, key);
}
