/* 파일 설명 적는곳

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#include <gtk/gtk.h>

#include "otak-lib/otak-widget/otnotebookpage.h"
#include "otak-lib/otak-util/otsys.h"
#include "otak-lib/otak-util/otgtk.h"
#include "odmimagebank.h"
#include "odmmainbody.h"

enum {
    SCHEMA_COL_ICON = 0,
    SCHEMA_COL_ICON_VISIBLE,
    SCHEMA_COL_NAME,
    SCHEMA_NUM_COLS
};



static GtkWidget*
odm_mainui_body_get_schema_treeview (OdmMainBody    *body,
                                     OdmSchemaType   type)
{
    GtkWidget *return_treeview = NULL;

    switch (type)
    {
        case ODM_SCHEMA_TYPE_TABLE:
            return_treeview = body->schema_table_treeview;
        break;

        case ODM_SCHEMA_TYPE_INDEX:
            return_treeview = body->schema_index_treeview;
        break;

        case ODM_SCHEMA_TYPE_PROCEDURE:
            return_treeview = body->schema_procedure_treeview;
        break;

        default:
        break;
    }

    return return_treeview;
}

static GtkWidget*
odm_mainui_body_get_active_schema_treeview (OdmMainBody *body)
{
    return odm_mainui_body_get_schema_treeview (body,
            odm_main_body_get_active_schema_type (body));
}

static void
odm_mainui_body_construct_schema_treeview (GtkTreeView *treeview)
{
    GtkListStore        *store      = NULL;
    GtkTreeViewColumn   *column     = NULL;
    GtkCellRenderer     *renderer   = NULL;

    store = gtk_list_store_new (SCHEMA_NUM_COLS,
            GDK_TYPE_PIXBUF, G_TYPE_BOOLEAN, G_TYPE_STRING);
    gtk_tree_view_set_model (treeview, GTK_TREE_MODEL (store));
    g_object_unref (store);

    column = gtk_tree_view_column_new ();

    renderer = gtk_cell_renderer_pixbuf_new ();
    gtk_tree_view_column_pack_start (column, renderer, FALSE);
    gtk_tree_view_column_set_attributes (column, renderer,
            "pixbuf", SCHEMA_COL_ICON, "visible", SCHEMA_COL_ICON_VISIBLE, NULL);

    renderer = gtk_cell_renderer_text_new ();
    gtk_tree_view_column_pack_end (column, renderer, FALSE);
    gtk_tree_view_column_set_attributes (column, renderer, "markup", SCHEMA_COL_NAME, NULL);

    gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
}

static void
odm_mainui_body_refresh_schema_treeview_elements (GtkTreeView   *treeview,
                                                  GdkPixbuf     *pixbuf,
                                                  GPtrArray     *array)
{
    GtkTreeIter      iter;
    GtkListStore    *store          = NULL;
    gboolean         pixbuf_visible = (pixbuf != NULL);

    store = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (treeview)));
    gtk_list_store_clear (store);

    gint i = 0;
    for (i = 0; i < array->len; i++)
    {
        gtk_list_store_append (store, &iter);
        gtk_list_store_set (store, &iter,
                SCHEMA_COL_ICON, pixbuf, SCHEMA_COL_ICON_VISIBLE, pixbuf_visible,
                SCHEMA_COL_NAME, g_ptr_array_index (array, i), -1);
    }
}



/*
 * public
 */

OdmMainBody*
odm_main_body_new (OtDbConnection *conn)
{
    OdmMainBody *body = g_malloc0 (sizeof (OdmMainBody));

    body->container                 = gtk_alignment_new (0.0, 0.0, 1.0, 1.0);
    body->hpaned                    = gtk_hpaned_new ();
    body->schema_table              = gtk_table_new (2, 1, FALSE);
    body->schema_notebook           = gtk_notebook_new ();
    body->schema_table_scrolled     = gtk_scrolled_window_new (NULL, NULL);
    body->schema_index_scrolled     = gtk_scrolled_window_new (NULL, NULL);
    body->schema_procedure_scrolled = gtk_scrolled_window_new (NULL, NULL);
    body->schema_table_treeview     = gtk_tree_view_new ();
    body->schema_index_treeview     = gtk_tree_view_new ();
    body->schema_procedure_treeview = gtk_tree_view_new ();
    body->content_notebook          = gtk_notebook_new ();
    body->connections_alignment     = gtk_alignment_new (0.0, 0.0, 1.0, 1.0);
    body->conn                      = conn;

    gtk_widget_show (body->schema_table);
    gtk_widget_show (body->schema_notebook);
    gtk_widget_show (body->schema_table_scrolled);
    gtk_widget_show (body->schema_index_scrolled);
    gtk_widget_show (body->schema_procedure_scrolled);
    gtk_widget_show (body->schema_table_treeview);
    gtk_widget_show (body->schema_index_treeview);
    gtk_widget_show (body->schema_procedure_treeview);
    gtk_widget_show (body->content_notebook);
    gtk_widget_show (body->connections_alignment);

    /* scrolled window */
    gtk_scrolled_window_set_policy (
            GTK_SCROLLED_WINDOW (body->schema_table_scrolled),
            GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_scrolled_window_set_policy (
            GTK_SCROLLED_WINDOW (body->schema_index_scrolled),
            GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_scrolled_window_set_policy (
            GTK_SCROLLED_WINDOW (body->schema_procedure_scrolled),
            GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

    gtk_container_add (GTK_CONTAINER (body->schema_table_scrolled),
            body->schema_table_treeview);
    gtk_container_add (GTK_CONTAINER (body->schema_index_scrolled),
            body->schema_index_treeview);
    gtk_container_add (GTK_CONTAINER (body->schema_procedure_scrolled),
            body->schema_procedure_treeview);

    /* paned기본 위치 */
    gtk_paned_set_position (GTK_PANED (body->hpaned), 250);

    /* TreeView의 컬럼정보를 구성한다. */
    odm_mainui_body_construct_schema_treeview (
            GTK_TREE_VIEW (body->schema_table_treeview));
    odm_mainui_body_construct_schema_treeview (
            GTK_TREE_VIEW (body->schema_index_treeview));
    odm_mainui_body_construct_schema_treeview (
            GTK_TREE_VIEW (body->schema_procedure_treeview));

    /* paned에 기본 위젯을 넣는다 */
    gtk_paned_add1 (GTK_PANED (body->hpaned), body->schema_table);
    gtk_paned_add2 (GTK_PANED (body->hpaned), body->content_notebook);

    /* 스키마(현재는 왼쪽) table에 서버 목록과 스키마 notebook을 넣는다 */
    gtk_table_attach (GTK_TABLE (body->schema_table), body->connections_alignment,
            0, 1, 0, 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
    gtk_table_attach (GTK_TABLE (body->schema_table), body->schema_notebook,
            0, 1, 1, 2, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);

    /* 스키마 notebook에 table, index, procedure등을 넣는다 */
    OtNotebookPage *notebook_page = NULL;

    gtk_notebook_set_scrollable (GTK_NOTEBOOK (body->schema_notebook), TRUE);

    notebook_page = ot_notebook_page_new (
            GTK_NOTEBOOK (body->schema_notebook),
            body->schema_table_scrolled, FALSE, NULL);
    ot_notebook_page_set_tab_label (notebook_page, "table");
    notebook_page = ot_notebook_page_new (
            GTK_NOTEBOOK (body->schema_notebook),
            body->schema_index_scrolled, FALSE, NULL);
    ot_notebook_page_set_tab_label (notebook_page, "index");
    notebook_page = ot_notebook_page_new (
            GTK_NOTEBOOK (body->schema_notebook),
            body->schema_procedure_scrolled, FALSE, NULL);
    ot_notebook_page_set_tab_label (notebook_page, "procedure");

    /* content notebook */
    gtk_notebook_set_scrollable (GTK_NOTEBOOK (body->content_notebook), TRUE);
    gtk_notebook_set_tab_pos (GTK_NOTEBOOK (body->content_notebook), GTK_POS_BOTTOM);

    /* 기본 임시 컨테이너에 paned를 넣는다 */
    gtk_container_add (GTK_CONTAINER (body->container), body->hpaned);

    return body;
}

OdmMainBody*
odm_main_body_free (OdmMainBody *main_body)
{
    gint    i       = 0;
    gint    count   = 0;

    count = gtk_notebook_get_n_pages (GTK_NOTEBOOK (main_body->schema_notebook));
    for (i = count - 1; i >= 0; i--)
    {
        gtk_notebook_remove_page (GTK_NOTEBOOK (main_body->schema_notebook), i);
    }
    count = gtk_notebook_get_n_pages (GTK_NOTEBOOK (main_body->content_notebook));
    for (i = count - 1; i >= 0; i--)
    {
        gtk_notebook_remove_page (GTK_NOTEBOOK (main_body->content_notebook), i);
    }

    odm_main_body_return_self_container (main_body);

    gtk_widget_destroy (main_body->connections_alignment);
    gtk_widget_destroy (main_body->schema_notebook);
    gtk_widget_destroy (main_body->schema_table);
    gtk_widget_destroy (main_body->content_notebook);
    gtk_widget_destroy (main_body->hpaned);
    ot_gtk_widget_free (main_body->container);

    g_free (main_body);

    return NULL;
}

GtkWidget*
odm_main_body_get_base_widget (OdmMainBody *main_body)
{
    return main_body->hpaned;
}

void
odm_main_body_set_schema_row_activated_cb (OdmMainBody      *main_body,
                                           OdmSchemaType     type,
                                           GCallback         cb,
                                           gpointer          user_data)
{
    GtkWidget *tv = odm_mainui_body_get_schema_treeview (main_body, type);

    g_signal_connect ((gpointer) tv, "row_activated", cb, user_data);
}

void
odm_main_body_contain_connections (OdmMainBody  *main_body,
                                   GtkWidget    *connections)
{
    ot_gtk_widget_move_container (connections,
            GTK_CONTAINER (main_body->connections_alignment));
}

void
odm_main_body_return_self_container (OdmMainBody *main_body)
{
    GtkWidget *base_widget = odm_main_body_get_base_widget (main_body);

    ot_gtk_widget_move_container (base_widget, GTK_CONTAINER (main_body->container));
}

GtkWidget*
odm_main_body_get_content_noteook (OdmMainBody *main_body)
{
    return main_body->content_notebook;
}

OdmSchemaType
odm_main_body_get_active_schema_type (OdmMainBody *main_body)
{
    OdmSchemaType return_type = ODM_SCHEMA_TYPE_NONE;

    switch (gtk_notebook_get_current_page (GTK_NOTEBOOK (main_body->schema_notebook)))
    {
        case 0:
            return_type = ODM_SCHEMA_TYPE_TABLE;
        break;

        case 1:
            return_type = ODM_SCHEMA_TYPE_INDEX;
        break;

        case 2:
            return_type = ODM_SCHEMA_TYPE_PROCEDURE;
        break;

        default:
        break;
    }

    return return_type;
}

gchar*
odm_main_body_dup_selected_schema_name (OdmMainBody *main_body)
{
    GtkWidget           *treeview   = odm_mainui_body_get_active_schema_treeview (main_body);
    GtkTreeSelection    *selection  = NULL;
    GtkTreeModel        *model      = NULL;
    GtkTreeIter          iter;
    gchar               *name       = NULL;

    if (treeview == NULL)
    {
        return NULL;
    }

    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
    if (gtk_tree_selection_get_selected (selection, &model, &iter))
    {
        gtk_tree_model_get (model, &iter, SCHEMA_COL_NAME, &name, -1);
    }

    return name;
}

gboolean
odm_main_body_refresh_schema_table (OdmMainBody *main_body)
{
    OtDbConnection  *conn   = main_body->conn;
    GdkPixbuf       *pixbuf = odm_image_bank_get_pixbuf (ODM_IMG_BNK_TABLE_22);

    GPtrArray *array = ot_db_connection_dup_table_name_list (conn, TRUE);
    if (array == NULL)
    {
        return FALSE;
    }

    odm_mainui_body_refresh_schema_treeview_elements (
            GTK_TREE_VIEW (main_body->schema_table_treeview),
            pixbuf, array);

    ot_sys_g_ptr_array_free_with_element (array, g_free);

    return TRUE;
}

gboolean
odm_main_body_refresh_schema_index (OdmMainBody *main_body)
{
    OtDbConnection  *conn   = main_body->conn;
    GdkPixbuf       *pixbuf = odm_image_bank_get_pixbuf (ODM_IMG_BNK_INDEX_22);

    GPtrArray *array = ot_db_connection_dup_index_name_list (conn);
    if (array == NULL)
    {
        return FALSE;
    }

    odm_mainui_body_refresh_schema_treeview_elements (
            GTK_TREE_VIEW (main_body->schema_index_treeview),
            pixbuf, array);

    ot_sys_g_ptr_array_free_with_element (array, g_free);

    return TRUE;
}

gboolean
odm_main_body_refresh_schema_procedure (OdmMainBody *main_body)
{
    OtDbConnection  *conn   = main_body->conn;
    GdkPixbuf       *pixbuf = odm_image_bank_get_pixbuf (ODM_IMG_BNK_PROCEDURE_22);

    GPtrArray *array = ot_db_connection_dup_procedure_name_list (conn);
    if (array == NULL)
    {
        return FALSE;
    }

    odm_mainui_body_refresh_schema_treeview_elements (
            GTK_TREE_VIEW (main_body->schema_procedure_treeview),
            pixbuf, array);

    ot_sys_g_ptr_array_free_with_element (array, g_free);

    return TRUE;
}

gboolean
odm_main_body_refresh_schema_all (OdmMainBody *main_body)
{
    if (!odm_main_body_refresh_schema_table (main_body))
    {
        return FALSE;
    }

    if (!odm_main_body_refresh_schema_index (main_body))
    {
        return FALSE;
    }

    if (!odm_main_body_refresh_schema_procedure (main_body))
    {
        return FALSE;
    }

    return TRUE;
}
