/* 파일 설명 적는곳

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#include <unistd.h>
#include <glib/gstdio.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#include <glade/glade-xml.h>

#include "otak-lib/otak-widget/otnotebookpage.h"
#include "otak-lib/otak-util/otsys.h"
#include "otak-lib/otak-util/otgtk.h"
#include "widget/odmschemabrowser.h"
#include "widget/odmsqleditor.h"
#include "odmconf.h"
#include "odmgbwdg.h"
#include "widget/odmdialog.h"
#include "odmmainbody.h"
#include "odmconnui.h"
#include "odmmainui.h"
#include "../config-path.h"

#ifndef _OTAK_DBMANAGER_PATH_H_
#define INST_PREFIX "/usr"
#endif


static gboolean init = FALSE;

static GladeXML     *main_glade_xml         = NULL;
static GtkWidget    *connections_container  = NULL;
static GtkWidget    *connections_combobox    = NULL;
static OdmMainBody  *curr_main_body         = NULL;

static GPtrArray *conn_info_list = NULL;


static GladeXML*    odm_mainui_get_glade_xml (void);
static GtkWidget*   odm_mainui_get_window (void);
static GtkWidget*   odm_mainui_get_body_alignment (void);
static void         odm_mainui_remove_connections (gint index);
static void         odm_mainui_change_body (gint index);
static void         odm_mainui_disconnect (void);
static void         odm_mainui_append_schema_browser (void);


/*
 * signal
 */

/* window delete */
gboolean
window_main_delete_event_cb (GtkWidget  *widget,
                                 GdkEvent   *event,
                                 gpointer    user_data)
{
    gboolean noquit = FALSE;
    GtkWidget *dialog = gtk_dialog_new_with_buttons ("Otak DB Manager for Cubrid Quit?",
                                                     GTK_WINDOW (odm_mainui_get_window ()),
                                                     GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                                     GTK_STOCK_OK,
                                                     GTK_RESPONSE_OK,
                                                     GTK_STOCK_CANCEL,
                                                     GTK_RESPONSE_CANCEL,
                                                     NULL);

    gint result = gtk_dialog_run (GTK_DIALOG (dialog));
    switch (result)
    {
        case GTK_RESPONSE_OK:
            gtk_main_quit ();
            break;

        default:
            noquit = TRUE;
            break;
    }

    gtk_widget_destroy (dialog);
    dialog = NULL;

    return noquit;
}

void
connections_combobox_changed_cb (GtkComboBox    *widget,
                                 gpointer        user_data)
{
    gint index = gtk_combo_box_get_active (GTK_COMBO_BOX (connections_combobox));
    if (index >= 0)
    {
        odm_mainui_change_body (index);
    }
}

void
toolbutton_main_connect_clicked_cb (GtkButton   *button,
                                    gpointer     user_data)
{
    odm_connui_show (GTK_WINDOW (odm_mainui_get_window ()), TRUE);
}

void
toolbutton_main_disconnect_clicked_cb (GtkButton    *button,
                                       gpointer      user_data)
{
    odm_mainui_disconnect ();
}

void
toolbutton_main_sql_editor_clicked_cb (GtkButton    *button,
                                       gpointer      user_data)
{
    if (curr_main_body != NULL)
    {
        OdmSqlEditor    *editor     = NULL;
        gchar           *err_msg    = NULL;

        editor = odm_sql_editor_new (curr_main_body->conn,
                    odm_conf_get_image_abs_dir (), &err_msg);
        if (editor != NULL)
        {
            GtkNotebook *content_notebook = GTK_NOTEBOOK (
                    odm_main_body_get_content_noteook (curr_main_body));

            OtNotebookPage *page = ot_notebook_page_new (content_notebook,
                    GTK_WIDGET (editor), TRUE, NULL);
            ot_notebook_page_set_tab_label (page, "SQL Editor");

            gtk_notebook_set_current_page (content_notebook,
                    gtk_notebook_get_n_pages (content_notebook) - 1);
        }
        else if (err_msg != NULL)
        {
            odm_dialog_message_modal (GTK_WINDOW (odm_mainui_get_window ()),
                    err_msg, GTK_MESSAGE_ERROR);
        }

        err_msg = ot_sys_g_free (err_msg);
    }
}

void
toolbutton_main_schema_browser_clicked_cb (GtkButton    *button,
                                           gpointer      user_data)
{
    odm_mainui_append_schema_browser ();
}

void
toolbutton_main_commit_clicked_cb (GtkButton    *button,
                                   gpointer      user_data)
{
    if (curr_main_body != NULL)
    {
        ot_db_connection_commit (curr_main_body->conn);
    }
}

void
toolbutton_main_rollback_clicked_cb (GtkButton  *button,
                                     gpointer    user_data)
{
    if (curr_main_body != NULL)
    {
        ot_db_connection_rollback (curr_main_body->conn);
    }
}

void
treeview_schema_browser_table_row_activated_cb (GtkWidget           *widget,
                                                GtkTreePath         *path,
                                                GtkTreeViewColumn   *column,
                                                gpointer             user_data)
{
    odm_mainui_append_schema_browser ();
}



/*
 * private
 */

/*
 * odm_mainui_get_glade_xml:
 * 
 * 화면의 위젯을 호출할 수 있는 GladeXML* 을 반환한다.
 * 
 * Return value: libglade용 GladeXML* 또는 %NULL
 */
static GladeXML*
odm_mainui_get_glade_xml (void)
{
    if (main_glade_xml == NULL)
    {
        gchar *filepath = g_strdup_printf ("%s/otak-dbmanager-main.glade",
                            odm_conf_get_glade_abs_dir ());
        if (0 != g_access ((const gchar *)filepath, R_OK))
        {
            filepath = ot_sys_g_free (filepath);
            filepath = g_strdup_printf (INST_PREFIX"/share/otak-dbmanager/otak-dbmanager-main.glade");
            if (0 != g_access ((const gchar *)filepath, R_OK))
            {
                filepath = ot_sys_g_free (filepath);
                filepath = g_strdup_printf ("%s/otak-dbmanager-main.glade",
                   odm_conf_get_glade_abs_dir ());
            }
        }

        main_glade_xml = glade_xml_new (filepath, NULL, NULL);
        glade_xml_signal_autoconnect (main_glade_xml);

        filepath = ot_sys_g_free (filepath);
    }

    return main_glade_xml;
}

/*
 * odm_mainui_get_window:
 * 
 * 화면의 윈도우를 반환한다.
 * 
 * Return value: 바탕격인 위젯 또는 %NULL
 */
static GtkWidget*
odm_mainui_get_window (void)
{
    return glade_xml_get_widget (odm_mainui_get_glade_xml(), "window_main");
}

static GtkWidget*
odm_mainui_get_body_alignment (void)
{
    return glade_xml_get_widget (odm_mainui_get_glade_xml(),
            "alignment_main_body");
}

static void
odm_mainui_remove_connections (gint index)
{
    OdmConnInfo     *conn_info  = ODM_CONN_INFO (g_ptr_array_index (conn_info_list, index));
    OdmMainBody     *body       = (OdmMainBody *) conn_info->ref_data;
    OtDbConnection  *conn       = body->conn;

    if (conn_info_list->len - 1 > 0)
    {
        gint change_index = 0;
        if (index > 0)
        {
            change_index = index - 1;
        }
        else if (index == 0)
        {
            change_index = index + 1;
        }
        odm_mainui_change_body (change_index);
    }
    else
    {
        ot_gtk_widget_move_container (connections_combobox,
                GTK_CONTAINER (connections_container));
        curr_main_body = NULL;
    }

    gtk_combo_box_remove_text (GTK_COMBO_BOX (connections_combobox), index);

    g_object_unref (conn_info);
    g_ptr_array_remove_index (conn_info_list, index);
    odm_main_body_free (body);

    if (!ot_db_connection_disconnect (conn))
    {
        odm_dialog_message_modal (GTK_WINDOW (odm_mainui_get_window ()),
                "커넥션 해제 실패", GTK_MESSAGE_ERROR);
    }
}

static void
odm_mainui_change_body (gint index)
{
    OdmConnInfo *conn_info      = ODM_CONN_INFO (g_ptr_array_index (conn_info_list, index));
    OdmMainBody *body           = (OdmMainBody *) (conn_info->ref_data);
    GtkWidget   *base_widget    = odm_main_body_get_base_widget (body);

    // 기존에 횔성화된것을 숨기고 임시 위젯으로 옮긴다.
    if (curr_main_body != NULL)
    {
        gtk_widget_hide (odm_main_body_get_base_widget (curr_main_body));
        odm_main_body_return_self_container (curr_main_body);
    }

    ot_gtk_widget_move_container (base_widget, GTK_CONTAINER (odm_mainui_get_body_alignment ()));
    gtk_widget_show (base_widget);

    odm_main_body_contain_connections (body, connections_combobox);

    gtk_combo_box_set_active (GTK_COMBO_BOX (connections_combobox), index);

    curr_main_body = body;
}

static void
odm_mainui_disconnect (void)
{
    gint index = gtk_combo_box_get_active (GTK_COMBO_BOX (connections_combobox));
    if (index >= 0)
    {
        odm_mainui_remove_connections (index);
    }
}

static void
odm_mainui_append_schema_browser (void)
{
    if (curr_main_body != NULL)
    {
        if (odm_main_body_get_active_schema_type (curr_main_body) == ODM_SCHEMA_TYPE_TABLE)
        {
            gchar *table_name = odm_main_body_dup_selected_schema_name (curr_main_body);
            if (table_name != NULL)
            {
                OdmSchemaBrowser    *browser    = NULL;
                OtNotebookPage      *page       = NULL;
                gchar               *err_msg    = NULL;

                browser = odm_schema_browser_new (curr_main_body->conn,
                        table_name, odm_conf_get_image_abs_dir (), &err_msg);
                if (browser != NULL)
                {
                    GtkNotebook *content_notebook = GTK_NOTEBOOK (
                            odm_main_body_get_content_noteook (curr_main_body));

                    page = ot_notebook_page_new (content_notebook,
                            GTK_WIDGET (browser), TRUE, NULL);
                    ot_notebook_page_set_tab_label (page, "Schema Browser");

                    gtk_notebook_set_current_page (content_notebook,
                            gtk_notebook_get_n_pages (content_notebook) - 1);
                }
                else if (err_msg != NULL)
                {
                    odm_dialog_message_modal (GTK_WINDOW (odm_mainui_get_window ()),
                            err_msg, GTK_MESSAGE_ERROR);
                }

                err_msg = ot_sys_g_free (err_msg);
                table_name = ot_sys_g_free (table_name);
            }
        }
    }
}



/*
 * piblic
 */

void
odm_mainui_init (void)
{
    if (!init)
    {
        if (conn_info_list == NULL)
        {
            conn_info_list = g_ptr_array_new ();
        }

        if (connections_combobox == NULL)
        {
            connections_container   = gtk_alignment_new (0.0, 0.0, 1.0, 1.0);
            connections_combobox     = gtk_combo_box_new_text ();

            g_signal_connect ((gpointer) connections_combobox, "changed",
                    G_CALLBACK (connections_combobox_changed_cb), NULL);
            gtk_widget_show (connections_combobox);
            gtk_container_add (GTK_CONTAINER (connections_container),
                    connections_combobox);
        }

        odm_gb_wdg_set_main_window (GTK_WINDOW (odm_mainui_get_window ()));

        init = TRUE;
    }
}

void
odm_mainui_show (void)
{
    gtk_widget_show (odm_mainui_get_window ());
}

gboolean
odm_mainui_add_connections (OdmConnInfo *conn_info)
{
    OtDbConnection  *conn   = NULL;
    OdmMainBody     *body   = NULL;

    conn = odm_conn_info_new_ot_db_connection (conn_info);
    if (conn == NULL)
    {
        goto label_error;
    }

    body = odm_main_body_new (conn);
    if (body == NULL)
    {
        goto label_error;
    }

    /* Schema Browser의 Table list에 row_activated signal callback 등록 */
    odm_main_body_set_schema_row_activated_cb (body, ODM_SCHEMA_TYPE_TABLE,
            G_CALLBACK (treeview_schema_browser_table_row_activated_cb),
            NULL);

    if (!ot_db_connection_connect (conn))
    {
        gchar *err_msg = ot_db_connection_dup_current_err_msg (conn);
        if (err_msg != NULL)
        {
            odm_dialog_message_modal (GTK_WINDOW (odm_mainui_get_window ()),
                    err_msg, GTK_MESSAGE_ERROR);
            err_msg = ot_sys_g_free (err_msg);
        }

        goto label_error;
    }

    odm_conn_info_set_ref_data (conn_info, body);
    g_ptr_array_add (conn_info_list, conn_info);

    gtk_combo_box_append_text (GTK_COMBO_BOX (connections_combobox),
            conn_info->name);

    odm_main_body_refresh_schema_all (body);

    odm_mainui_change_body (conn_info_list->len - 1);

    return TRUE;

label_error:
    if (conn != NULL)
    {
        g_object_unref (conn);
        conn = NULL;
    }

    if (body != NULL)
    {
        odm_main_body_free (body);
        body = NULL;
    }

    return FALSE;
}
