/* 파일 설명 적는곳

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#include <unistd.h>
#include <glib/gstdio.h>
#include <gtk/gtk.h>

#include "otak-lib/otak-util/otsys.h"
#include "otak-lib/otak-util/otxml.h"
#include "otak-lib/otak-db/otdbconnectioncub.h"
#include "type/odmconninfodefine.h"
#include "odmimagebank.h"
#include "odmmainui.h"
#include "odmconnui.h"
#include "odmconf.h"
#include "../config-path.h"

// For eclipse compilation environment
#ifndef _OTAK_DBMANAGER_PATH_H_
#define INST_PREFIX "/usr"
#endif

int
main (int argc,
      const char * argv[])
{
    gtk_set_locale ();

    g_thread_init (NULL);
    gdk_threads_init ();

    gtk_init (&argc, (gpointer) &argv);

    gchar *abs_path = ot_sys_exec_abs_path_new (argv[0]);


    // wrong absolute path configuration fallback
    gchar *test_path = g_strdup_printf ("%s/glade/otak-dbmanager-main.glade", abs_path);
    // access() test, if glade file is not found, 
    // change path to pkgdatadir by UNIX Hierarchical Directory Structure
    if (0 != g_access ((const gchar *)test_path, R_OK))
    {
        abs_path = ot_sys_g_free (abs_path);
        abs_path = g_strdup_printf (INST_PREFIX"/share/otak-dbmanager/");
        test_path = ot_sys_g_free (test_path);
        test_path = g_strdup_printf ("%s/glade/otak-dbmanager-main.glade", abs_path);
        // if it fail again, fallback to current dir's parent
        if (0 != g_access ((const gchar *) test_path, R_OK))
        {
            abs_path = ot_sys_g_free (abs_path);
            abs_path = g_strdup_printf ("%s/../", getenv("PWD"));
        }
    }
    test_path = ot_sys_g_free (test_path);

    /*
     * 초기화 순서주의
     */

    odm_conf_set_conn_info_prov (ODM_CONN_INFO_PROV_CUB);
    odm_conf_path_init (abs_path);

    odm_image_bank_init ();
    odm_mainui_init ();
    odm_connui_init ();

    abs_path = ot_sys_g_free (abs_path);

    odm_mainui_show ();

//    gchar *filename = g_strdup_printf ("%s/config/server_list.xml",
//            odm_conf_get_exec_abs_path ());
//    xmlDocPtr xml_doc = ot_xml_new (filename);
//    xmlNodePtr r_node = xmlDocGetRootElement (xml_doc);
//    xmlNodePtr s_node = ot_xml_get_child_node (r_node, "server", 0, NULL);
//    xmlNodePtr node = ot_xml_get_child_node (s_node, "host", 0, NULL);
//    if (node != NULL)
//    {
//        gchar *host = ot_xml_dup_value (xml_doc, node);
//        g_printf ("%s\n", host);
//        ot_xml_set_attribute (node, "aa", "123123");
//    }
//    xmlSaveFormatFile (filename, xml_doc, 0);
//    xml_doc = ot_xml_free (xml_doc);

    gtk_main ();

    return 0;
}
