/* 파일 설명 적는곳

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#include <stdlib.h>
#include <gtk/gtk.h>

#include "../otak-util/otsys.h"
#include "../otak-util/otstring.h"
#include "../otak-util/otgtk.h"
#include "../otak-db/otdbutil.h"
#include "../otak-db/otdbindexkey.h"
#include "otdbgrid.h"

static OtGridClass *ot_db_grid_parent_class = NULL;



/*
 * signal
 */



/*
 * private
 */
static gboolean
ot_db_grid_is_simple (OtDbGrid *grid)
{
    return grid->table != NULL;
}

static void
ot_db_grid_reset_sql (OtDbGrid *grid)
{
    g_assert (ot_db_grid_is_simple (grid));

    grid->sql = ot_sys_g_free (grid->sql);
    grid->sql = ot_db_connection_dup_simple_sql_select (
                    grid->dataset->connection, grid->table, grid->fields,
                    NULL);
}

static gboolean
ot_db_grid_dupset_data (OtDbColumnData  *column_data,
                        OtDbCType        ctype,
                        const gchar     *value)
{
    if (value == NULL)
    {
        column_data->data = NULL;
    }
    else
    {
        switch (ctype)
        {
            case OT_DB_C_TYPE_INT:
                if (ot_string_is_empty (value)
                        || !ot_string_is_numeric (value))
                {
                    return FALSE;
                }
                column_data->data = (gint *) g_malloc0 (sizeof (gint));
                *((gint *) column_data->data)  = atoi (value);
            break;
    
            case OT_DB_C_TYPE_INT64:
                if (ot_string_is_empty (value)
                        || !ot_string_is_numeric (value))
                {
                    return FALSE;
                }
                column_data->data = (gint64 *) g_malloc0 (sizeof (gint64));
                *((gint64 *) column_data->data)  = atol (value);
            break;
    
            case OT_DB_C_TYPE_FLOAT:
                if (ot_string_is_empty (value) || !ot_string_is_float (value))
                {
                    return FALSE;
                }
                column_data->data = (gfloat *) g_malloc0 (sizeof (gfloat));
                *((gfloat *) column_data->data)  = atof (value);
            break;
    
            case OT_DB_C_TYPE_DOUBLE:
                if (ot_string_is_empty (value) || !ot_string_is_float (value))
                {
                    return FALSE;
                }
                column_data->data = (gdouble *) g_malloc0 (sizeof (gdouble));
                *((gdouble *) column_data->data)  = atof (value);
            break;
    
            case OT_DB_C_TYPE_STR:
            case OT_DB_C_TYPE_STR_DATE:
            case OT_DB_C_TYPE_STR_DATETIME:
            case OT_DB_C_TYPE_STR_TIME:
            case OT_DB_C_TYPE_FILENAME:
                column_data->data = g_strdup (value);
            break;
    
            case OT_DB_C_TYPE_STR_BUF:
            case OT_DB_C_TYPE_FD:
            case OT_DB_C_TYPE_OBJECT:
            case OT_DB_C_TYPE_COLLECTION:
                // TODO : sohalee 구현
            break;
    
            default:
            break;
        }
    }

    return TRUE;
}

static void
ot_db_grid_changed_data_element_free (GPtrArray *array)
{
    ot_sys_g_ptr_array_free_with_element (array, (GDestroyNotify) ot_db_column_data_free);
}

static void
ot_db_grid_changed_data_pack_clear (OtDbGrid *db_grid)
{
    if (db_grid->changed_data != NULL)
    {
        g_hash_table_destroy (db_grid->changed_data);
        db_grid->changed_data = NULL;
    }

    if (db_grid->changed_row_index != NULL)
    {
        // element의 free는 changed_data에서 하므로 생략 (changed_data의 key와 같은 주소)
        g_ptr_array_free (db_grid->changed_row_index, TRUE);
        db_grid->changed_row_index = NULL;
    }

    if (db_grid->delete_row_index != NULL)
    {
        db_grid->delete_row_index = ot_sys_g_ptr_array_free_with_element(
                                        db_grid->delete_row_index,
                                        g_free);
    }

    db_grid->last_add_row_index = 0;
}
    
static void
ot_db_grid_changed_data_pack_reset (OtDbGrid *db_grid)
{
    ot_db_grid_changed_data_pack_clear (db_grid);

    db_grid->changed_data       = g_hash_table_new_full (g_int_hash, g_int_equal,
                                    g_free, (GDestroyNotify) ot_db_grid_changed_data_element_free);
    db_grid->changed_row_index  = g_ptr_array_new ();
    db_grid->delete_row_index   = g_ptr_array_new ();
}

static OtDbColumnData*
ot_db_grid_get_changed_column_data (OtDbGrid    *grid,
                                    gint         row_index,
                                    gint         column_index,
                                    gint        *array_index)
{
    GPtrArray *data_array = NULL;

    /* 처음 수정하는 row이면 데이타 저장 array 생성하여 수정 테이블에 등록 */
    data_array = g_hash_table_lookup (grid->changed_data, &row_index);
    if (data_array != NULL)
    {
        /* 기존에 있는 컬럼데이타를 찾아 반환한다. */
        gint i = 0;
        for (i = 0; i < data_array->len; i++)
        {
            OtDbColumnData *column_data = g_ptr_array_index (data_array, i);
    
            if (column_data->column_index == column_index)
            {
                if (array_index != NULL)
                {
                    *array_index = i;
                }

                return column_data;
            }
        }
    }

    return NULL;
}

static void
ot_db_grid_append_changed_data (OtDbGrid        *grid,
                                OtDbColumnData  *column_data,
                                gint             row_index)
{
    g_assert (ot_db_grid_is_simple (grid));

    GPtrArray *data_array = NULL;

    /* 처음 수정하는 row이면 데이타 저장 array 생성하여 수정 테이블에 등록 */
    data_array = g_hash_table_lookup (grid->changed_data, &row_index);
    if (data_array == NULL)
    {
        gint *new_row_index = g_malloc0 (sizeof (gint));

        *new_row_index = row_index;

        data_array = g_ptr_array_new ();
        g_hash_table_insert (grid->changed_data, new_row_index, data_array);
        g_ptr_array_add (grid->changed_row_index, new_row_index);
    }
    else
    {
        gint             array_index    = -1;
        OtDbColumnData  *old_data       = NULL;

        old_data = ot_db_grid_get_changed_column_data (grid, row_index,
                    column_data->column_index, &array_index);

        /* 기존에 있는 컬럼데이타 삭제한다. */
        if (old_data != NULL)
        {
            g_ptr_array_remove_index_fast (data_array, array_index);
            old_data = ot_db_column_data_free (old_data);
        }
    }
    
    g_ptr_array_add (data_array, column_data);
}

static gboolean
ot_db_grid_renderer_text_start_editing_aft (GtkCellRenderer        *cell,
                                            GdkEvent               *event,
                                            GtkWidget              *widget,
                                            const gchar            *path,
                                            GdkRectangle           *background_area,
                                            GdkRectangle           *cell_area,
                                            GtkCellRendererState    flags,
                                            GtkCellEditable        *editable)
{
    g_assert (ot_db_grid_is_simple (OT_DB_GRID (widget)));

    OtGridRendererText  *renderer       = OT_GRID_RENDERER_TEXT (cell);
    OtDbGrid            *grid           = OT_DB_GRID (widget);
    GtkTreeIter          iter;
    GtkTreeModel        *model          = gtk_tree_view_get_model (GTK_TREE_VIEW (grid));
    OtDbColumnData      *column_data    = NULL;
    GPtrArray           *data_array     = NULL;
    gint                 row_index      = -1;
    gint                 column_index   = renderer->renderer_index - 1;
    gboolean             is_null        = FALSE;

    if (gtk_tree_model_get_iter_from_string (model, &iter, path))
    {
        gtk_tree_model_get (model, &iter, 0, &row_index, -1);
    }
    else
    {
        return FALSE;
    }

    column_data = ot_db_grid_get_changed_column_data (grid, row_index,
                    column_index, NULL);
    if (column_data != NULL)
    {
        is_null = column_data->data == NULL;
    }
    else
    {
        if (!ot_db_dataset_result_move_cursor (grid->dataset, row_index))
        {
            return FALSE;
        }
    
        data_array = ot_db_dataset_dup_result_data (grid->dataset, FALSE);
        if (data_array == NULL)
        {
            return FALSE;
        }

        is_null = g_ptr_array_index (data_array, column_index) == NULL;

        data_array = ot_db_dataset_free_result_data (grid->dataset,
                        data_array);
    }

    if (is_null)
    {
        gtk_entry_set_text (GTK_ENTRY (editable), "");
    }

    return TRUE;
}

static void
ot_db_grid_construct_columns (OtDbGrid  *grid)
{
    GPtrArray   *columns                    = grid->dataset->req_columns;
    gint         columns_len                = columns->len + 1;
    OtDbColumn  *column                     = NULL;
    const gchar *column_titles[columns_len];
    GType       *types                      = NULL;
    gint         i                          = 0;

    types = g_malloc0 (sizeof (GType) * (columns->len + 1));

    /*
     * 컬럼의 첫번째는 append index
     */

    column_titles[0]    = NULL;
    types[0]            = G_TYPE_INT;
    for (i = 0; i < columns->len; i++)
    {
        column = g_ptr_array_index (columns, i);

        column_titles[i + 1]    = column->name;
        types[i + 1]            = G_TYPE_STRING;
//        types[i + 1]            = ot_db_column_get_g_type (column);
        
    }

    ot_grid_construct_columns (OT_GRID (grid), columns_len, types,
            column_titles);
    g_free (types);
    types = NULL;

    GtkTreeViewColumn *treeview_column = gtk_tree_view_get_column (
                                            GTK_TREE_VIEW (grid), 0);
    gtk_tree_view_column_set_visible (treeview_column, FALSE);

    ot_grid_set_resizable (OT_GRID (grid), TRUE);
    ot_grid_set_clickable (OT_GRID (grid), TRUE);
    ot_grid_set_editable (OT_GRID (grid), grid->editable);
    ot_grid_set_start_editing_aft (OT_GRID (grid),
            ot_db_grid_renderer_text_start_editing_aft);
}

static void
ot_db_grid_destroy (GtkObject *object)
{
    OtDbGrid *db_grid = OT_DB_GRID (object);

    if (db_grid->dataset != NULL)
    {
        g_object_unref (db_grid->dataset);
        db_grid->dataset = NULL;
    }

    ot_db_grid_changed_data_pack_clear (db_grid);

    db_grid->sql    = ot_sys_g_free (db_grid->sql);
    db_grid->table  = ot_sys_g_free (db_grid->table);
    db_grid->fields = ot_sys_g_ptr_array_free_with_element (db_grid->fields,
                        g_free);
    db_grid->orders = ot_sys_g_ptr_array_free_with_element (db_grid->orders,
                        g_free);

    (* GTK_OBJECT_CLASS (ot_db_grid_parent_class)->destroy) (object);
}

static gboolean
ot_db_grid_changed (OtGrid              *grid,
                    OtGridRendererText  *renderer,
                    GtkTreeIter         *iter,
                    const gchar         *new_text)
{
    g_assert (ot_db_grid_is_simple (OT_DB_GRID (grid)));

    OtDbGrid        *db_grid        = OT_DB_GRID (grid);
    GtkTreeModel    *model          = gtk_tree_view_get_model (GTK_TREE_VIEW (grid));
    OtDbDataset     *dataset        = db_grid->dataset;
    OtDbColumn      *column         = NULL;
    OtDbColumnData  *column_data    = NULL;
    gint             column_index   = renderer->renderer_index - 1;
    gboolean         is_changed     = FALSE;

    column = g_ptr_array_index (dataset->req_columns, column_index);
    column_data = g_malloc0 (sizeof (OtDbColumnData));

    if (ot_db_grid_dupset_data (column_data, column->ctype, new_text))
    {
        column_data->column_index = column_index;
        column_data->ctype        = column->ctype;

        is_changed = TRUE;

        if (ot_db_grid_parent_class->changed != NULL)
        {
            is_changed = ot_db_grid_parent_class->changed (grid,
                            renderer, iter, new_text);
        }
    }
    else
    {
        column_data = ot_db_column_data_free (column_data);
    }

    /* 아래 블록에서 실패가 있어선 안된다 무조건 성공으로 본다 */
    if (is_changed)
    {
        gint row_index = -1;

        gtk_tree_model_get (model, iter, 0, &row_index, -1);
        ot_db_grid_append_changed_data (db_grid, column_data, row_index);
    }

    return is_changed;
}

/* class init */
static void
ot_db_grid_class_init (OtDbGridClass *klass)
{
    GtkObjectClass  *object_class = GTK_OBJECT_CLASS (klass);
    OtGridClass     *gclass       = OT_GRID_CLASS (klass);

    ot_db_grid_parent_class = g_type_class_peek_parent (klass);

    // 이곳에 연결하면 gtk_widget_destroy시 함께 실행된다.
    object_class->destroy = ot_db_grid_destroy;

    gclass->changed = ot_db_grid_changed;
}

/* instance init */
static void
ot_db_grid_init (OtDbGrid *db_grid)
{
    db_grid->dataset            = NULL;
    db_grid->changed_data       = NULL;
    db_grid->changed_row_index  = NULL;
    db_grid->delete_row_index   = NULL;
    db_grid->last_add_row_index = 0;
    db_grid->sql                = NULL;
    db_grid->table              = NULL;
    db_grid->fields             = NULL;
    db_grid->orders             = NULL;
    db_grid->editable           = TRUE;

    ot_grid_set_resizable (OT_GRID (db_grid), TRUE);
    ot_grid_set_clickable (OT_GRID (db_grid), TRUE);
}

/* get type */
GType
ot_db_grid_get_type (void)
{
    static GType widget_type = 0;

    if (G_UNLIKELY (widget_type == 0))
    {
        const GTypeInfo widget_info =
        {
            sizeof (OtDbGridClass),
            NULL,       /* base_init */
            NULL,
            (GClassInitFunc) ot_db_grid_class_init,
            NULL,       /* class_finalize */
            NULL,       /* class_data */
            sizeof (OtDbGrid),
            0,          /* n_preallocs */
            (GInstanceInitFunc) ot_db_grid_init,
            NULL,       /* value_table */
        };
        widget_type = g_type_register_static (OT_TYPE_GRID, "OtDbGrid",
                &widget_info, 0);
    }

    return widget_type;
}



/*
 * public
 */

OtDbGrid*
ot_db_grid_new (void)
{
    OtDbGrid *grid = OT_DB_GRID (g_object_new (OT_TYPE_DB_GRID, NULL));

    return grid;
}

OtDbDataset*
ot_db_grid_get_dataset (OtDbGrid *grid)
{
    return grid->dataset;
}

void
ot_db_grid_set_dataset (OtDbGrid    *grid,
                        OtDbDataset *dataset)
{
    if (grid->dataset != NULL)
    {
        g_object_unref (grid->dataset);
    }

    grid->dataset = dataset;
    g_object_ref (grid->dataset);
}

void
ot_db_grid_set_editable (OtDbGrid   *grid,
                         gboolean    editable)
{
    grid->editable = editable;
}

void
ot_db_grid_dupset_sql (OtDbGrid     *grid,
                       const gchar  *sql)
{
    // sql이 수동으로 설정된 경우는 table이 NULL이 되어 simple type을 막는다
    grid->table = ot_sys_g_free (grid->table);

    grid->sql = ot_sys_g_free (grid->sql);
    grid->sql = g_strdup (sql);
}

void
ot_db_grid_dupset_table (OtDbGrid       *grid,
                         const gchar    *table)
{
    grid->table = ot_sys_g_free (grid->table);
    grid->table = g_strdup (table);

    ot_db_grid_reset_sql (grid);
}

void
ot_db_grid_dupset_fields (OtDbGrid  *grid,
                          GPtrArray *fields)
{
    grid->fields = ot_sys_g_ptr_array_free_with_element (grid->fields,
                    g_free);
    grid->fields = g_ptr_array_new ();

    gint i = 0;
    for (i = 0; i < fields->len; i++)
    {
        gchar *field = g_strdup (g_ptr_array_index (fields, i));
        g_ptr_array_add (grid->fields, field);
    }

    ot_db_grid_reset_sql (grid);
}

void
ot_db_grid_dupset_orders (OtDbGrid  *grid,
                          GPtrArray *orders)
{
    grid->orders = ot_sys_g_ptr_array_free_with_element (grid->orders,
                    g_free);
    grid->orders = g_ptr_array_new ();

    gint i = 0;
    for (i = 0; i < orders->len; i++)
    {
        gchar *order = g_strdup (g_ptr_array_index (orders, i));
        g_ptr_array_add (grid->fields, order);
    }

    ot_db_grid_reset_sql (grid);
}

static GPtrArray*
ot_db_grid_dup_data_array_for_grid (OtDbGrid    *grid,
                                    GPtrArray   *data_array,
                                    gint        *row_index)
{
    GPtrArray   *columns            = grid->dataset->req_columns;
    GPtrArray   *data_array_grid    = g_ptr_array_new ();

    g_ptr_array_add (data_array_grid, row_index);

    gint i = 0;
    for (i = 0; i < data_array->len; i++)
    {
        OtDbColumn  *column = g_ptr_array_index (columns, i);
        gchar       *value  = ot_db_util_dup_str_value (column->ctype,
                                g_ptr_array_index (data_array, i));
        g_ptr_array_add (data_array_grid, value);
    }

    return data_array_grid;
}

gint
ot_db_grid_next_block (OtDbGrid *grid)
{
    gint    fetch_size  = grid->dataset->fetch_size;
    gint    cnt         = 0;

    while (ot_db_dataset_result_next (grid->dataset))
    {
        GPtrArray   *data_array         = NULL;
        GPtrArray   *data_array_grid    = NULL;
        gint        data_index          = grid->dataset->req_cursor_index;
        gint        *row_index          = NULL;

        data_array = ot_db_dataset_dup_result_data (grid->dataset, FALSE);
        if (data_array == NULL)
        {
            break;
        }

        // cursor_index는 따로 free하지 않음.
        // ot_sys_g_ptr_array_free_with_element에서 일괄적으로 free 
        row_index   = (gint *) g_malloc0 (sizeof (gint));
        *row_index  = data_index;

        data_array_grid = ot_db_grid_dup_data_array_for_grid (grid,
                            data_array, row_index);
        ot_grid_append_row_from_array (OT_GRID (grid), data_array_grid);

        ot_sys_g_ptr_array_free_with_element (data_array_grid, g_free);
        ot_db_dataset_free_result_data (grid->dataset, data_array);

        cnt++;

        if (fetch_size > 0)
        {
            if (data_index > 0 && ((data_index + 1) % fetch_size) == 0)
            {
                break;
            }
        }
    }

    if (ot_db_connection_is_error (grid->dataset->connection))
    {
        cnt = -1;
    }

    return cnt;
}

gboolean
ot_db_grid_refresh (OtDbGrid *grid)
{
    if (!ot_db_dataset_prepare (grid->dataset, grid->sql))
    {
        return FALSE;
    }

    if (!ot_db_dataset_execute (grid->dataset))
    {
        return FALSE;
    }

    ot_db_grid_changed_data_pack_reset (grid);
    ot_db_grid_construct_columns (grid);

    return ot_db_grid_next_block (grid) >= 0;
}

gboolean
ot_db_grid_delete (OtDbGrid *grid)
{
    g_assert (ot_db_grid_is_simple (grid));

    GtkTreeView         *treeview   = GTK_TREE_VIEW (grid);
    GtkTreeSelection    *selection  = gtk_tree_view_get_selection (treeview);
    GtkTreeModel        *model      = NULL;
    GtkTreeIter          iter;
    gint                 row_index  = -1;

    if (!gtk_tree_selection_get_selected (selection, &model, &iter))
    {
        return FALSE;
    }

    gtk_tree_model_get (model, &iter, 0, &row_index, -1);
    ot_grid_delete_row (OT_GRID (grid), &iter);


    gint *new_row_index = g_malloc0 (sizeof (gint));

    *new_row_index = row_index;
    g_ptr_array_add (grid->delete_row_index, new_row_index);
    /* changed_row_index에서 찾아 지우는건 삽질이니 changed_data에서 지우자 */
    g_hash_table_remove (grid->changed_data, new_row_index);

    return TRUE;
}

static gboolean
ot_db_grid_insert_data (OtDbGrid *grid)
{
    g_assert (ot_db_grid_is_simple (grid));

    GPtrArray   *ins_arr    = NULL;
    GPtrArray   *c_row_ids  = grid->changed_row_index;
    gboolean     result     = TRUE;

    if (c_row_ids->len > 0)
    {
        ins_arr = g_ptr_array_new ();

        gint i = 0;
        for (i = 0; i < c_row_ids->len; i++)
        {
            GPtrArray   *arr        = NULL;
            gint        *row_index  = g_ptr_array_index (c_row_ids, i);
    
            arr = g_hash_table_lookup (grid->changed_data, row_index);
    
            /*
             * 일단 이경우는 등후에 삭제까지 된 경우이다.
             * 삭제가 되면 더이상 changed_data에 들어올수가 없다.
             */
            if (arr == NULL)
            {
                continue;
            }
    
            // row_index의 실제값이 0보다 작은것은 등록이다.
            if (*row_index < 0)
            {
                g_ptr_array_add (ins_arr, arr);
            }
        }

        if (ins_arr->len > 0)
        {
        // TODO sohalee OtDbDataset에 update_result_datas와 같이 insert도 구현
//        result = ot_db_dataset_insert_result_datas (grid->dataset, ins_arr,
//                    grid->table);
        }
    }

    if (ins_arr != NULL)
    {
        g_ptr_array_free (ins_arr, TRUE);
        ins_arr = NULL;
    }

    return result;
}

static gboolean
ot_db_grid_update_data (OtDbGrid *grid)
{
    g_assert (ot_db_grid_is_simple (grid));

    gint        *upd_ids    = NULL;
    gint         upd_cnt    = 0;
    GPtrArray   *upd_arr    = NULL;
    GPtrArray   *c_row_ids  = grid->changed_row_index;
    gboolean     result     = TRUE;

    if (c_row_ids->len > 0)
    {
        upd_ids = g_malloc0 (sizeof (gint) * c_row_ids->len);
        upd_arr = g_ptr_array_new ();

        gint i = 0;
        for (i = 0; i < c_row_ids->len; i++)
        {
            GPtrArray   *arr        = NULL;
            gint        *row_index  = g_ptr_array_index (c_row_ids, i);
    
            arr = g_hash_table_lookup (grid->changed_data, row_index);
    
            /*
             * 일단 이경우는 변경후에 삭제까지 된 경우이다.
             * 삭제가 되면 더이상 changed_data에 들어올수가 없다.
             */
            if (arr == NULL)
            {
                continue;
            }
    
            if (*row_index >= 0)
            {
                g_ptr_array_add (upd_arr, arr);
                upd_ids[upd_cnt] = *row_index;
                upd_cnt++;
            }
        }

        if (upd_cnt > 0)
        {
            result = ot_db_dataset_update_result_datas (grid->dataset,
                        upd_ids, upd_arr, grid->table);
        }
    }

    upd_ids = ot_sys_g_free (upd_ids);

    if (upd_arr != NULL)
    {
        g_ptr_array_free (upd_arr, TRUE);
        upd_arr = NULL;
    }

    return result;
}

static gboolean
ot_db_grid_delete_data (OtDbGrid *grid)
{
    g_assert (ot_db_grid_is_simple (grid));

    gint        *del_ids    = NULL;
    GPtrArray   *d_row_ids  = grid->delete_row_index;
    gboolean     result     = TRUE;

    // 삭제
    if (d_row_ids->len > 0)
    {
        del_ids = g_malloc0 (sizeof (gint) * d_row_ids->len);

        gint i = 0;
        for (i = 0; i < d_row_ids->len; i++)
        {
            gint *row_index = g_ptr_array_index (d_row_ids, i);

            del_ids[i] = *row_index;
        }

        result = ot_db_dataset_delete_result_datas (grid->dataset, del_ids,
                    d_row_ids->len, grid->table);
    }

    del_ids = ot_sys_g_free (del_ids);

    return result;
}

gboolean
ot_db_grid_apply_changed (OtDbGrid   *grid,
                          gchar     **error)
{
    g_assert (ot_db_grid_is_simple (grid));

    OtDbConnection  *conn       = grid->dataset->connection;
    gboolean         result     = FALSE;
    gchar           *e_msg      = NULL;
    gchar           *db_e_msg   = NULL;

    if (ot_db_grid_insert_data (grid))
    {
        if (ot_db_grid_update_data (grid))
        {
            if (ot_db_grid_delete_data (grid))
            {
                result = TRUE;
            }
            else
            {
                e_msg = g_strdup ("삭제");
            }
        }
        else
        {
            e_msg = g_strdup ("수정");
        }
    }
    else
    {
        e_msg = g_strdup ("등록");
    }

    if (result && error != NULL && ot_db_connection_is_error (conn))
    {
        if (e_msg == NULL)
        {
            e_msg = g_strdup ("");
        }

        db_e_msg = ot_db_connection_dup_current_err_msg (conn);
        if (db_e_msg == NULL)
        {
            db_e_msg = g_strdup ("");
        }

        *error = g_strdup_printf ("%s 자료 DB 반영에 실패하였습니다.\n" \
                    "일부는 반영되었을수도 있습니다.\n\n%s", e_msg, db_e_msg);
    }

    e_msg       = ot_sys_g_free (e_msg);
    db_e_msg    = ot_sys_g_free (db_e_msg);

    ot_db_grid_changed_data_pack_reset (grid);
    ot_db_grid_refresh (grid);

    return result;
}

gboolean
ot_db_grid_cancel_changed (OtDbGrid *grid)
{
    g_assert (ot_db_grid_is_simple (grid));

    return ot_db_grid_refresh (grid);
}

gboolean
ot_db_grid_set_null_value (OtDbGrid *grid)
{
    gboolean is_set = FALSE;

    if (grid->editable)
    {
        OtGridRendererText  *rnd    = NULL;
        GtkTreeModel        *model  = gtk_tree_view_get_model (GTK_TREE_VIEW (grid));
        GtkListStore        *store  = GTK_LIST_STORE (model);
        GtkTreeSelection    *sel    = NULL;
        GtkTreeIter          iter;

        rnd = ot_grid_get_cursor_renderer (OT_GRID (grid));
        if (rnd != NULL)
        {
            sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (grid));
        
            if (gtk_tree_selection_get_selected (sel, NULL, &iter))
            {
                if (ot_db_grid_changed (OT_GRID (grid), rnd, &iter, NULL))
                {
                    gchar *val = ot_db_util_dup_str_value (OT_DB_C_TYPE_NONE,
                                    NULL);
    
                    is_set = ot_gtk_list_store_set_value (store, &iter,
                            G_TYPE_STRING, rnd->renderer_index, val, FALSE);
    
                    val = ot_sys_g_free (val);
                }
            }
        }
    }

    return is_set;
}
