#ifndef OTDBGRID_H_
#define OTDBGRID_H_

#include "../otak-widget/otgrid.h"
#include "../otak-db/otdbcolumn.h"
#include "../otak-db/otdbdataset.h"

G_BEGIN_DECLS

#define OT_TYPE_DB_GRID             (ot_db_grid_get_type ())
#define OT_DB_GRID(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), OT_TYPE_DB_GRID, OtDbGrid))
#define OT_DB_GRID_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), OT_TYPE_DB_GRID, OtDbGridClass))
#define OT_IS_DB_GRID(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), OT_TYPE_DB_GRID))
#define OT_IS_DB_GRID_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), OT_TYPE_DB_GRID))
#define OT_DB_GRID_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), OT_TYPE_DB_GRID, OtDbGridClass))


typedef struct _OtDbGrid      OtDbGrid;
typedef struct _OtDbGridClass OtDbGridClass;

struct _OtDbGrid
{
    OtGrid grid;

    OtDbDataset *dataset;
    GHashTable  *changed_data;

    /*
     * changed_row_index 이것을 사용하는 궁극적인 목적은 에러처리이다.
     * 좀 납득하기 어렵겠지만 GHashTable은 아래와 같은 문제가 있다.
     * 
     * 1. g_hash_table_foreach 를 통해 row_index없이 할수 있지만 중간에 멈추거나 에러처리를 할수없다.
     * 2. g_hash_tabel_get_keys를 통해 key를 GList*로 반환받아 처리할수 있지만 GLib 2.14이상에서만 된다.
     * 3. GHashTable내의 멤버를 직접 사용해 가져올수도 있으나 GLib API에 멤버변수에 대한 언급이 없는것으로 보아
     *      내부는 언제든 변할 여지가 있으므로 사용하지 않는다.
     * 
     * 결국 아래와 같이 직접 row_index를 저장해 두는것이 귀찮고 우울하지만 제일 안전하고 확실한 방법이다.
     */
    GPtrArray   *changed_row_index;
    GPtrArray   *delete_row_index;
    /* insert되는 row는 -1부터 거꾸로 row_index가 -1씩 된다. -1, -2와 같이 */
    gint         last_add_row_index;

    gchar       *sql;
    gchar       *table;
    GPtrArray   *fields;
    GPtrArray   *orders;

    gboolean     editable;
};

struct _OtDbGridClass {
    OtGridClass parent_class;
};

GType           ot_db_grid_get_type             (void);
OtDbGrid*       ot_db_grid_new                  (void);
OtDbDataset*    ot_db_grid_get_dataset          (OtDbGrid *grid);
void            ot_db_grid_set_dataset          (OtDbGrid       *grid,
                                                 OtDbDataset    *dataset);
void            ot_db_grid_set_editable         (OtDbGrid   *grid,
                                                 gboolean    editable);
void            ot_db_grid_dupset_sql           (OtDbGrid       *grid,
                                                 const gchar    *sql);
void            ot_db_grid_dupset_table         (OtDbGrid       *grid,
                                                 const gchar    *table);
void            ot_db_grid_dupset_fields        (OtDbGrid   *grid,
                                                 GPtrArray  *fields);
void            ot_db_grid_dupset_orders        (OtDbGrid   *grid,
                                                 GPtrArray  *orders);
gint            ot_db_grid_next_block           (OtDbGrid *grid);
gboolean        ot_db_grid_refresh              (OtDbGrid *grid);
gboolean        ot_db_grid_delete               (OtDbGrid *grid);
gboolean        ot_db_grid_apply_changed        (OtDbGrid *grid, gchar **error);
gboolean        ot_db_grid_cancel_changed       (OtDbGrid *grid);
gboolean        ot_db_grid_set_null_value       (OtDbGrid *grid);

G_END_DECLS

#endif /*OTDBGRID_H_*/
