/* 파일 설명 적는곳

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#include <glib-object.h>

#include "otdbcolumn.h"


static gpointer parent_class = NULL;



static void
ot_db_column_dispose (GObject *gobject)
{
    OtDbColumn *column = OT_DB_COLUMN (gobject);

    if (column->name != NULL)
    {
        g_free (column->name);
        column->name = NULL;
    }

    if (column->default_value != NULL)
    {
        g_free (column->default_value);
        column->default_value = NULL;
    }

    if (column->comment != NULL)
    {
        g_free (column->comment);
        column->comment = NULL;
    }

    G_OBJECT_CLASS (parent_class)->dispose (gobject);
}

static void
ot_db_column_base_class_init (OtDbColumnClass *class)
{
}

static void
ot_db_column_base_class_finalize (OtDbColumnClass *class)
{
}

static void
ot_db_column_class_init (OtDbColumnClass *class)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (class);

    parent_class = g_type_class_ref (G_TYPE_OBJECT);

    gobject_class->dispose = ot_db_column_dispose;

    class->set_type         = NULL;
    class->is_using_where   = NULL;
    class->dup_type_name    = NULL;
}

static void
ot_db_column_init (OtDbColumn       *column,
                   OtDbColumnClass  *class)
{
    column->order           = 0;
    column->name            = NULL;
    column->ctype           = OT_DB_C_TYPE_NONE;
    column->size            = 0;
    column->scale           = 0;
    column->not_null        = FALSE;
    column->default_value   = NULL;
    column->comment         = NULL;
    column->pkey_order      = 0;
    column->unique          = FALSE;
    column->index           = FALSE;
}

GType
ot_db_column_get_type (void)
{
    static GType object_type = 0;

    if (!object_type)
    {
        const GTypeInfo object_info =
        {
            sizeof (OtDbColumnClass),
            (GBaseInitFunc) ot_db_column_base_class_init,
            (GBaseFinalizeFunc) ot_db_column_base_class_finalize,
            (GClassInitFunc) ot_db_column_class_init,
            NULL,       /* class_finalize */
            NULL,       /* class_data */
            sizeof (OtDbColumn),
            16,     /* n_preallocs */
            (GInstanceInitFunc) ot_db_column_init,
            NULL,       /* value_table */
        };

        object_type = g_type_register_static (G_TYPE_INITIALLY_UNOWNED, "OtDbColumn", 
                &object_info, G_TYPE_FLAG_ABSTRACT);
    }

    return object_type;
}



/*
 * public
 */

void
ot_db_column_set_type (OtDbColumn   *column,
                       gpointer      dbtype,
                       gint          size)
{
    OtDbColumnClass *class = OT_DB_COLUMN_GET_CLASS (column);

    return class->set_type (column, dbtype, size);
}

GType
ot_db_column_get_g_type (OtDbColumn *column)
{
    GType return_type = G_TYPE_NONE;

    switch (column->ctype)
    {
        case OT_DB_C_TYPE_INT:
                return_type = G_TYPE_INT;
        break;

        case OT_DB_C_TYPE_INT64:
                return_type = G_TYPE_INT64;
        break;

        case OT_DB_C_TYPE_FLOAT:
                return_type = G_TYPE_FLOAT;
        break;

        case OT_DB_C_TYPE_DOUBLE:
                return_type = G_TYPE_DOUBLE;
        break;

        case OT_DB_C_TYPE_STR:
        case OT_DB_C_TYPE_STR_DATE:
        case OT_DB_C_TYPE_STR_DATETIME:
        case OT_DB_C_TYPE_STR_TIME:
        case OT_DB_C_TYPE_STR_BUF:
        case OT_DB_C_TYPE_FILENAME:
        case OT_DB_C_TYPE_FD:
        case OT_DB_C_TYPE_OBJECT:
        case OT_DB_C_TYPE_COLLECTION:
            return_type = G_TYPE_STRING;
        break;

        default:
        break;
    }

    return return_type;
}

GType*
ot_db_column_dup_g_types (GPtrArray *columns)
{
    GType *types = g_malloc0 (sizeof (GType) * columns->len);

    gint i = 0;
    for (i = 0; i < columns->len; i++)
    {
        OtDbColumn *column = OT_DB_COLUMN (g_ptr_array_index (columns, i));
        types[i] = ot_db_column_get_g_type (column);
    }

    return types;
}

gboolean
ot_db_column_is_using_where (OtDbColumn *column)
{
    OtDbColumnClass *class = OT_DB_COLUMN_GET_CLASS (column);

    return class->is_using_where (column);
}

gchar*
ot_db_column_dup_type_name (OtDbColumn *column)
{
    OtDbColumnClass *class = OT_DB_COLUMN_GET_CLASS (column);

    return class->dup_type_name (column);
}

OtDbColumnData*
ot_db_column_data_free (OtDbColumnData *column_data)
{
    switch (column_data->ctype)
    {
        case OT_DB_C_TYPE_FD:
        case OT_DB_C_TYPE_OBJECT:
        case OT_DB_C_TYPE_COLLECTION:
            // TODO : sohalee 구현
        break;

        default:
            if (column_data->data != NULL)
            {
                g_free (column_data->data);
                column_data->data = NULL;
            }
        break;
    }

    g_free (column_data);

    return NULL;
}
