/* 파일 설명 적는곳

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#include <string.h>
#include <stdlib.h>

#include "otdbcolumncub.h"

static OtDbColumnClass *ot_db_column_cub_parent_class = NULL;



static void
ot_db_column_cub_dispose (GObject *gobject)
{
    OtDbColumnCub *column_cub = OT_DB_COLUMN_CUB (gobject);

    if (column_cub->inheritance != NULL)
    {
        g_free (column_cub->inheritance);
        column_cub->inheritance = NULL;
    }

    G_OBJECT_CLASS (ot_db_column_cub_parent_class)->dispose (gobject);
}

static void
ot_db_column_cub_set_type (OtDbColumn   *column,
                           gpointer      dbtype,
                           gint          size)
{
    OtDbColumnCub   *column_cub = OT_DB_COLUMN_CUB (column);
    OtDbCType        ctype      = OT_DB_C_TYPE_NONE;
    T_CCI_U_TYPE     utype      = *(T_CCI_U_TYPE *) dbtype;

    column->size        = size;
    column_cub->utype   = utype;

    switch (utype)
    {
        case CCI_U_TYPE_INT:
        case CCI_U_TYPE_SHORT:
            if (size <= 10)
            {
                ctype = OT_DB_C_TYPE_INT;
            }
            else
            {
                ctype = OT_DB_C_TYPE_INT64;
            }
        break;

        case CCI_U_TYPE_FLOAT:
            ctype = OT_DB_C_TYPE_FLOAT;
        break;

        case CCI_U_TYPE_DOUBLE:
        case CCI_U_TYPE_NUMERIC:
        case CCI_U_TYPE_MONETARY:
            ctype = OT_DB_C_TYPE_DOUBLE;
        break;

        case CCI_U_TYPE_CHAR:
        case CCI_U_TYPE_STRING:
        case CCI_U_TYPE_NCHAR:
        case CCI_U_TYPE_VARNCHAR:
            ctype = OT_DB_C_TYPE_STR;
        break;

        case CCI_U_TYPE_DATE:
            ctype = OT_DB_C_TYPE_STR_DATE;
        break;

        case CCI_U_TYPE_TIMESTAMP:
            ctype = OT_DB_C_TYPE_STR_DATETIME;
        break;

        case CCI_U_TYPE_TIME:
            ctype = OT_DB_C_TYPE_STR_TIME;
        break;

        case CCI_U_TYPE_BIT:
        case CCI_U_TYPE_VARBIT:
            ctype = OT_DB_C_TYPE_STR_BUF;
        break;

        case CCI_U_TYPE_SET:
        case CCI_U_TYPE_OBJECT:
            ctype = OT_DB_C_TYPE_OBJECT;
        break;

        case CCI_U_TYPE_MULTISET:
        case CCI_U_TYPE_SEQUENCE:
            ctype = OT_DB_C_TYPE_COLLECTION;
        break;

        default:
        break;
    }

    column->ctype = ctype;
}

static gboolean
ot_db_column_cub_is_using_where (OtDbColumn *column)
{
    OtDbColumnCub   *col_cub    = OT_DB_COLUMN_CUB (column);
    gboolean         result     = FALSE;

    switch (col_cub->utype)
    {
        case CCI_U_TYPE_INT:
        case CCI_U_TYPE_SHORT:
        case CCI_U_TYPE_FLOAT:
        case CCI_U_TYPE_DOUBLE:
        case CCI_U_TYPE_NUMERIC:
        case CCI_U_TYPE_MONETARY:
        case CCI_U_TYPE_CHAR:
        case CCI_U_TYPE_STRING:
        case CCI_U_TYPE_NCHAR:
        case CCI_U_TYPE_VARNCHAR:
        case CCI_U_TYPE_DATE:
        case CCI_U_TYPE_TIMESTAMP:
        case CCI_U_TYPE_TIME:
            result = TRUE;
        break;

        default:
            result = FALSE;
        break;
    }

    return result;
}

static gchar*
ot_db_column_cub_dup_type_name (OtDbColumn *column)
{
    OtDbColumnCub   *col_cub    = OT_DB_COLUMN_CUB (column);
    gchar           *type_name  = NULL;
    switch (col_cub->utype)
    {
        case CCI_U_TYPE_INT:
            type_name = g_strdup("INTEGER");
        break;

        case CCI_U_TYPE_SHORT:
            type_name = g_strdup("SMALLINT");
        break;

        case CCI_U_TYPE_FLOAT:
            type_name = g_strdup("FLOAT");
        break;

        case CCI_U_TYPE_DOUBLE:
            type_name = g_strdup("DOUBLE");
        break;

        case CCI_U_TYPE_NUMERIC:
            type_name = g_strdup("NUMERIC");
        break;

        case CCI_U_TYPE_MONETARY:
            type_name = g_strdup("MONETARY");
        break;

        case CCI_U_TYPE_CHAR:
            type_name = g_strdup("CHAR");
        break;

        case CCI_U_TYPE_STRING:
            type_name = g_strdup("VARCHAR");
        break;

        case CCI_U_TYPE_NCHAR:
            type_name = g_strdup("NCHAR");
        break;

        case CCI_U_TYPE_VARNCHAR:
            type_name = g_strdup("NCHAR VARYING");
        break;

        case CCI_U_TYPE_DATE:
            type_name = g_strdup("DATE");
        break;

        case CCI_U_TYPE_TIMESTAMP:
            type_name = g_strdup("TIMESTAMP");
        break;

        case CCI_U_TYPE_TIME:
            type_name = g_strdup("TIME");
        break;

        case CCI_U_TYPE_BIT:
            type_name = g_strdup("BIT");
        break;

        case CCI_U_TYPE_VARBIT:
            type_name = g_strdup("BIT VARYING");
        break;

        case CCI_U_TYPE_SET:
            type_name = g_strdup("SET");
        break;

        case CCI_U_TYPE_OBJECT:
            type_name = g_strdup("OBJECT");
        break;

        case CCI_U_TYPE_MULTISET:
            type_name = g_strdup("MULTISET");
        break;

        case CCI_U_TYPE_SEQUENCE:
            type_name = g_strdup("SEQUENCE");
        break;

        default:
        break;
    }

    return type_name;
}

static void
ot_db_column_cub_class_init (OtDbColumnCubClass *class)
{
    GObjectClass    *gobject_class  = G_OBJECT_CLASS (class);
    OtDbColumnClass *cclass         = OT_DB_COLUMN_CLASS (class);

    ot_db_column_cub_parent_class = g_type_class_peek_parent (class);

    gobject_class->dispose = ot_db_column_cub_dispose;

    cclass->set_type        = ot_db_column_cub_set_type;
    cclass->is_using_where  = ot_db_column_cub_is_using_where;
    cclass->dup_type_name   = ot_db_column_cub_dup_type_name;
}

static void
ot_db_column_cub_init (OtDbColumnCub *column)
{
    column->inheritance = NULL;
    column->shared      = FALSE;
}

GType
ot_db_column_cub_get_type (void)
{
    static GType widget_type = 0;

    if (G_UNLIKELY (widget_type == 0))
    {
        const GTypeInfo widget_info =
        {
            sizeof (OtDbColumnCubClass),
            NULL,       /* base_init */
            NULL,
            (GClassInitFunc) ot_db_column_cub_class_init,
            NULL,       /* class_finalize */
            NULL,       /* class_data */
            sizeof (OtDbColumnCub),
            0,          /* n_preallocs */
            (GInstanceInitFunc) ot_db_column_cub_init,
            NULL,       /* value_table */
        };
        widget_type = g_type_register_static (OT_TYPE_DB_COLUMN, "OtDbColumnCub",
                &widget_info, 0);
    }

    return widget_type;
}



/*
 * public
 */

OtDbColumn*
ot_db_column_cub_new (void)
{
    OtDbColumnCub *column = OT_DB_COLUMN_CUB (g_object_new (OT_TYPE_DB_COLUMN_CUB, NULL));

    return OT_DB_COLUMN (column);
}
