/* 파일 설명 적는곳

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#ifndef OTDBCONNECTION_H_
#define OTDBCONNECTION_H_

#include <glib-object.h>

#include "otdbdefine.h"
#include "otdbcolumn.h"

G_BEGIN_DECLS

#define OT_DB_CONNECTION_URL_PROTO_DELIMITER    "://"
#define OT_DB_CONNECTION_URL_PROP_DELIMITER     ":"

#define OT_TYPE_DB_CONNECTION               (ot_db_connection_get_type ())
#define OT_DB_CONNECTION(object)            (G_TYPE_CHECK_INSTANCE_CAST ((object), OT_TYPE_DB_CONNECTION, OtDbConnection))
#define OT_DB_CONNECTION_CLASS(klass)       (G_TYPE_CHECK_CLASS_CAST ((klass), OT_TYPE_DB_CONNECTION, OtDbConnectionClass))
#define OT_IS_DB_CONNECTION(object)         (G_TYPE_CHECK_INSTANCE_TYPE ((object), OT_TYPE_DB_CONNECTION))
#define OT_IS_DB_CONNECTION_CLASS(klass)    (G_TYPE_CHECK_CLASS_TYPE ((klass), OT_TYPE_DB_CONNECTION))
#define OT_DB_CONNECTION_GET_CLASS(object)  (G_TYPE_INSTANCE_GET_CLASS ((object), OT_TYPE_DB_CONNECTION, OtDbConnectionClass))


typedef struct _OtDbConnection      OtDbConnection;
typedef struct _OtDbConnectionClass OtDbConnectionClass;

struct _OtDbConnection
{
    GInitiallyUnowned parent_instance;

    gchar   *url;
    gchar   *username;
    gchar   *password;
    gchar   *encoding;
};

struct _OtDbConnectionClass
{
    GInitiallyUnownedClass parent_class;

    OtDbConnection* (*new_clone)                (OtDbConnection *connection);
    gboolean        (*connect)                  (OtDbConnection *connection);
    gboolean        (*disconnect)               (OtDbConnection *connection);
    gboolean        (*commit)                   (OtDbConnection *connection);
    gboolean        (*rollback)                 (OtDbConnection *connection);
    gboolean        (*is_connected)             (OtDbConnection *connection);
    gboolean        (*test_connection)          (OtDbConnection *connection);
    gpointer        (*get_conn_handle)          (OtDbConnection *connection);
    gboolean        (*is_error)                 (OtDbConnection *connection);
    void            (*init_error)               (OtDbConnection *connection);
    gchar*          (*dup_current_err_msg)      (OtDbConnection *connection);
    GPtrArray*      (*dup_table_name_list)      (OtDbConnection *connection, OtDbTableType table_type);
    GPtrArray*      (*dup_index_name_list)      (OtDbConnection *connection);
    GPtrArray*      (*dup_procedure_name_list)  (OtDbConnection *connection);
    GPtrArray*      (*dup_column_list)          (OtDbConnection *connection, const gchar *table_name);
    GPtrArray*      (*dup_index_key_list)       (OtDbConnection *connection, const gchar *table_name);
    gchar*          (*dup_simple_sql_select)    (OtDbConnection *connection, const gchar *table_name,
                                                 GPtrArray *fields, GPtrArray *orders);
    gchar*          (*dup_simple_sql_count)     (OtDbConnection *connection, const gchar *table_name,
                                                 GPtrArray *where_fields);
    gchar*          (*dup_simple_sql_delete)    (OtDbConnection *connection, const gchar *table_name,
                                                 GPtrArray *where_fields);
    gpointer        (*new_dataset)              (OtDbConnection *connection);
    OtDbColumn*     (*new_column)               (OtDbConnection *connection);
};

GType           ot_db_connection_get_type                   (void);
void            ot_db_connection_set_url                    (OtDbConnection *connection,
                                                             const gchar    *url);
void            ot_db_connection_set_username               (OtDbConnection *connection,
                                                             const gchar    *username);
void            ot_db_connection_set_password               (OtDbConnection *connection,
                                                             const gchar    *password);
void            ot_db_connection_set_encoding               (OtDbConnection *connection,
                                                             const gchar    *encoding);
OtDbConnection* ot_db_connection_new_clone                  (OtDbConnection *connection);
gboolean        ot_db_connection_connect                    (OtDbConnection *connection);
gboolean        ot_db_connection_disconnect                 (OtDbConnection *connection);
gboolean        ot_db_connection_commit                     (OtDbConnection *connection);
gboolean        ot_db_connection_rollback                   (OtDbConnection *connection);
gboolean        ot_db_connection_is_connected               (OtDbConnection *connection);
gboolean        ot_db_connection_test_connection            (OtDbConnection *connection);
gpointer        ot_db_connection_get_conn_handle            (OtDbConnection *connection);
gboolean        ot_db_connection_is_error                   (OtDbConnection *connection);
void            ot_db_connection_init_error                 (OtDbConnection *connection);
gchar*          ot_db_connection_dup_current_err_msg        (OtDbConnection *connection);
GPtrArray*      ot_db_connection_dup_table_name_list        (OtDbConnection *connection,
                                                             OtDbTableType   table_type);
GPtrArray*      ot_db_connection_dup_index_name_list        (OtDbConnection *connection);
GPtrArray*      ot_db_connection_dup_procedure_name_list    (OtDbConnection *connection);
GPtrArray*      ot_db_connection_dup_column_list            (OtDbConnection *connection,
                                                             const gchar    *table_name);
GPtrArray*      ot_db_connection_dup_index_key_list         (OtDbConnection *connection,
                                                             const gchar    *table_name);
gchar*          ot_db_connection_dup_simple_sql_select      (OtDbConnection *connection,
                                                             const gchar    *table_name,
                                                             GPtrArray      *fields,
                                                             GPtrArray      *orders);
gchar*          ot_db_connection_dup_simple_sql_count       (OtDbConnection *connection,
                                                             const gchar    *table_name,
                                                             GPtrArray      *where_fields);
gchar*          ot_db_connection_dup_simple_sql_delete      (OtDbConnection *connection,
                                                             const gchar    *table_name,
                                                             GPtrArray      *where_fields);
gpointer        ot_db_connection_new_dataset                (OtDbConnection *connection);
OtDbColumn*     ot_db_connection_new_column                 (OtDbConnection *connection);
gchar*          ot_db_connection_dup_str_for_view           (OtDbConnection *connection,
                                                             const gchar    *str);
gchar*          ot_db_connection_dup_str_for_dbms           (OtDbConnection *connection,
                                                             const gchar    *str);

G_END_DECLS

#endif /*OTDBCONNECTION_H_*/
