/* 파일 설명 적는곳

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#include <string.h>
#include <stdlib.h>
#include <glib/gprintf.h>

#include "../otak-util/otsys.h"
#include "../otak-util/otstring.h"
#include "otdbdefine.h"
#include "otdbdatasetcub.h"
#include "otdbconnectioncub.h"
#include "otdbcolumncub.h"
#include "otdbindexkeycub.h"
#include "cas_cci.h"


static OtDbConnectionClass *ot_db_connection_cub_parent_class = NULL;



static void
ot_db_connection_cub_dispose (GObject *gobject)
{
    OtDbConnectionCub *connection_cub = OT_DB_CONNECTION_CUB (gobject);

    if (connection_cub->err_msg != NULL)
    {
        g_free (connection_cub->err_msg);
        connection_cub->err_msg = NULL;
    }

    if (connection_cub->host != NULL)
    {
        g_free (connection_cub->host);
        connection_cub->host = NULL;
    }

    if (connection_cub->database != NULL)
    {
        g_free (connection_cub->database);
        connection_cub->database = NULL;
    }

    G_OBJECT_CLASS (ot_db_connection_cub_parent_class)->dispose (gobject);
}

static OtDbConnection*
ot_db_connection_cub_new_clone (OtDbConnection *connection)
{
    return ot_db_connection_cub_new (connection->url, connection->username,
            connection->password, connection->encoding); 
}

static gboolean
ot_db_connection_cub_connect (OtDbConnection *connection)
{
    OtDbConnectionCub   *connection_cub = OT_DB_CONNECTION_CUB (connection);
    gint                 conn_handle    = 0;

    if (ot_db_connection_is_connected (connection))
    {
        // TODO : luna 국제화
        g_debug ("이미 연결 핸들이 만들어져 있습니다. 다시 만들지 않습니다.");
        return TRUE;
    }

    ot_db_connection_init_error (connection);

    conn_handle = cci_connect (connection_cub->host, connection_cub->port,
            connection_cub->database, connection->username, connection->password);
    if (conn_handle < 0)
    {
        switch (conn_handle)
        {
            // TODO : luna 국제화
            case CCI_ER_NO_MORE_MEMORY:
                connection_cub->err_msg = g_strdup ("메모리가 부족합니다.");
            break;

            case CCI_ER_HOSTNAME:
                connection_cub->err_msg = g_strdup ("해당 호스트에 연결할 수 없습니다.");
            break;

            default:
                connection_cub->err_msg = g_strdup ("알려지지 않은 오류가 발생하였습니다.");
            break;
        }

        connection_cub->err_code = conn_handle;

        return FALSE;
    }

    connection_cub->conn_handle = conn_handle;

    /* 접속 테스트 */
    if (!ot_db_connection_test_connection (connection))
    {
        // TODO : luna 국제화
        connection_cub->err_msg = g_strdup ("접속 실패. 접속정보를 확인하여 주십시요");
        connection_cub->conn_handle = 0;

        return FALSE;
    }

    return TRUE;
}

static gboolean
ot_db_connection_cub_disconnect (OtDbConnection *connection)
{
    OtDbConnectionCub   *connection_cub = OT_DB_CONNECTION_CUB (connection);
    T_CCI_ERROR          cci_error;
    gint                 res            = 0;

    ot_db_connection_init_error (connection);

    if (!ot_db_connection_test_connection (connection))
    {
        // TODO : luna 국제화
        connection_cub->err_msg = g_strdup ("connection test fail");

        return FALSE;
    }

    res = cci_end_tran (connection_cub->conn_handle, CCI_TRAN_ROLLBACK, &cci_error);
    if (res < 0)
    {
        goto label_error;
    }

    res = cci_disconnect (connection_cub->conn_handle, &cci_error);
    if (res < 0)
    {
        goto label_error;
    }

    connection_cub->conn_handle = 0;

    return TRUE;


label_error:
    connection_cub->err_msg = g_strdup (cci_error.err_msg);
    connection_cub->err_code = cci_error.err_code;

    return FALSE;
}

static gboolean
ot_db_connection_cub_commit (OtDbConnection *connection)
{
    OtDbConnectionCub   *connection_cub = OT_DB_CONNECTION_CUB (connection);
    T_CCI_ERROR          cci_error;
    gint                 res            = 0;

    ot_db_connection_init_error (connection);

    res = cci_end_tran (connection_cub->conn_handle, CCI_TRAN_COMMIT, &cci_error);
    if (res < 0)
    {
        connection_cub->err_msg = g_strdup (cci_error.err_msg);
        connection_cub->err_code = cci_error.err_code;

        return FALSE;
    }

    return TRUE;
}

static gboolean
ot_db_connection_cub_rollback (OtDbConnection *connection)
{
    OtDbConnectionCub   *connection_cub = OT_DB_CONNECTION_CUB (connection);
    T_CCI_ERROR          cci_error;
    gint                 res            = 0;

    ot_db_connection_init_error (connection);

    res = cci_end_tran (connection_cub->conn_handle, CCI_TRAN_ROLLBACK, &cci_error);
    if (res < 0)
    {
        connection_cub->err_msg = g_strdup (cci_error.err_msg);
        connection_cub->err_code = cci_error.err_code;

        return FALSE;
    }

    return TRUE;
}

static gboolean
ot_db_connection_cub_is_connected (OtDbConnection *connection)
{
    if (OT_DB_CONNECTION_CUB (connection)->conn_handle <= 0)
    {
        return FALSE;
    }

    return TRUE;
}

static gboolean
ot_db_connection_cub_test_connection (OtDbConnection *connection)
{
    gint res = 0;

    res = cci_get_db_version (OT_DB_CONNECTION_CUB (connection)->conn_handle, NULL, 0);
    if (res < 0)
    {
        switch (res)
        {
            case CCI_ER_CON_HANDLE:
            case CCI_ER_CONNECT:
                return FALSE;
            break;
        }
    }

    return TRUE;
}

static gpointer
ot_db_connection_cub_get_conn_handle (OtDbConnection *connection)
{
    return &(OT_DB_CONNECTION_CUB (connection)->conn_handle);
}

static gboolean
ot_db_connection_cub_is_error (OtDbConnection *connection)
{
    if (OT_DB_CONNECTION_CUB (connection)->err_code != 0)
    {
        return TRUE;
    }

    return FALSE;
}

static void
ot_db_connection_cub_init_error (OtDbConnection *connection)
{
    OtDbConnectionCub *connection_cub = OT_DB_CONNECTION_CUB (connection);

    connection_cub->err_code    = 0;
    if (connection_cub->err_msg != NULL)
    {
        g_free (connection_cub->err_msg);
        connection_cub->err_msg = NULL;
    }
}

static gchar*
ot_db_connection_cub_dup_current_err_msg (OtDbConnection *connection)
{
    OtDbConnectionCub *connection_cub = OT_DB_CONNECTION_CUB (connection);

    if (connection_cub->err_msg == NULL)
    {
        return NULL;
    }

    return g_strdup (connection_cub->err_msg);
}

static GPtrArray*
ot_db_connection_cub_dup_table_name_list (OtDbConnection    *connection,
                                          OtDbTableType      table_type)
{
    OtDbDataset *dataset    = ot_db_connection_new_dataset (connection);
    GPtrArray   *array      = NULL;
    gchar        qry[128]   = "SELECT class_name FROM db_class";

    switch (table_type)
    {
        case OT_DB_TABLE_TYPE_NORMAL:
            g_strlcat (qry, " WHERE is_system_class='NO'", sizeof (qry));
        break;

        case OT_DB_TABLE_TYPE_SYSTEM:
            g_strlcat (qry, " WHERE is_system_class='YES'", sizeof (qry));
        break;

        case OT_DB_TABLE_TYPE_BOTH:
        break;
    }

    if (!ot_db_dataset_prepare (dataset, qry))
    {
        return NULL;
    }

    if (!ot_db_dataset_execute (dataset))
    {
        return NULL;
    }

    array = g_ptr_array_new ();
    while (ot_db_dataset_result_next (dataset))
    {
        GPtrArray *data_array = ot_db_dataset_dup_result_data (dataset, FALSE);

        g_ptr_array_add (array, g_strdup (g_ptr_array_index (data_array, 0)));
        ot_db_dataset_free_result_data (dataset, data_array);
    }

    g_object_unref (dataset);
    dataset = NULL;

    return array;
}

static GPtrArray*
ot_db_connection_cub_dup_index_name_list (OtDbConnection *connection)
{
    OtDbDataset *dataset    = ot_db_connection_new_dataset (connection);
    GPtrArray   *array      = NULL;
    gchar        qry[320]   = "SELECT index_name FROM db_index";

    if (!ot_db_dataset_prepare (dataset, qry))
    {
        return NULL;
    }

    if (!ot_db_dataset_execute (dataset))
    {
        return NULL;
    }

    array = g_ptr_array_new ();
    while (ot_db_dataset_result_next (dataset))
    {
        GPtrArray *data_array = ot_db_dataset_dup_result_data (dataset, FALSE);

        g_ptr_array_add (array, g_strdup (g_ptr_array_index (data_array, 0)));
        ot_db_dataset_free_result_data (dataset, data_array);
    }

    g_object_unref (dataset);
    dataset = NULL;


    return array;
}

static GPtrArray*
ot_db_connection_cub_dup_procedure_name_list (OtDbConnection *connection)
{
    OtDbDataset *dataset    = ot_db_connection_new_dataset (connection);
    GPtrArray   *array      = NULL;
    gchar        qry[320]   = "SELECT sp_name FROM db_stored_procedure";

    if (!ot_db_dataset_prepare (dataset, qry))
    {
        return NULL;
    }

    if (!ot_db_dataset_execute (dataset))
    {
        return NULL;
    }

    array = g_ptr_array_new ();
    while (ot_db_dataset_result_next (dataset))
    {
        GPtrArray *data_array = ot_db_dataset_dup_result_data (dataset, FALSE);

        g_ptr_array_add (array, g_strdup (g_ptr_array_index (data_array, 0)));
        ot_db_dataset_free_result_data (dataset, data_array);
    }

    g_object_unref (dataset);
    dataset = NULL;

    return array;
}

static GPtrArray*
ot_db_connection_cub_dup_column_list (OtDbConnection    *connection,
                                      const gchar       *table_name)
{
    OtDbConnectionCub   *connection_cub = OT_DB_CONNECTION_CUB (connection);
    OtDbDataset         *dataset        = ot_db_connection_new_dataset (connection);
    OtDbDatasetCub      *dataset_cub    = OT_DB_DATASET_CUB (dataset);
    T_CCI_ERROR          cci_error;
    gint                 req            = 0;
    GPtrArray           *array          = NULL;
    gint                 i              = 0;
    gint                 j              = 0;

    ot_db_connection_init_error (connection);

    if (!ot_db_connection_is_connected (connection))
    {
        connection_cub->err_msg     = g_strdup ("커넥션 핸들을 먼저 생성하여주십시요.");
        connection_cub->err_code    = -1;

        return FALSE;
    }

    req = cci_schema_info (connection_cub->conn_handle, CCI_SCH_ATTRIBUTE,
            (char *) table_name, NULL, CCI_ATTR_NAME_PATTERN_MATCH, &cci_error);
    if (req < 0)
    {
        connection_cub->err_msg = g_strdup (cci_error.err_msg);
        connection_cub->err_code = cci_error.err_code;
        
        return FALSE;
    }

    dataset_cub->req_handle = req;
    dataset->req_columns    = ot_db_dataset_dup_result_columns (dataset);

    array = g_ptr_array_new ();
    while (ot_db_dataset_result_next (dataset))
    {
        GPtrArray       *data_array = ot_db_dataset_dup_result_data (dataset, FALSE);
        OtDbColumn      *column     = ot_db_connection_new_column (connection);
        OtDbColumnCub   *column_cub = OT_DB_COLUMN_CUB (column);
        gpointer         utype      = NULL;

        for (i = 0; i < dataset->req_columns->len; i++)
        {
            OtDbColumn      *column_sel     = g_ptr_array_index (dataset->req_columns, i);
            gpointer         schema_value   = g_ptr_array_index (data_array, i);
    
            if (ot_string_equals ("ATTR_NAME", column_sel->name))
            {
                column->name = ot_sys_g_free (column->name);

                if (schema_value != NULL)
                {
                    column->name = g_strdup (schema_value);
                }
            }
            else if (ot_string_equals ("DOMAIN", column_sel->name))
            {
                utype = schema_value;
            }
            else if (ot_string_equals ("SCALE", column_sel->name))
            {
                column->scale = *(gint *) schema_value;
            }
            else if (ot_string_equals ("PRECISION", column_sel->name))
            {
                column->size = *(gint *) schema_value;
            }
            else if (ot_string_equals ("INDEXED", column_sel->name))
            {
                column->index = *(gint *) schema_value == 1;
            }
            else if (ot_string_equals ("NON_NULL", column_sel->name))
            {
                column->not_null = *(gint *) schema_value == 1;
            }
            else if (ot_string_equals ("SHARED", column_sel->name))
            {
                column_cub->shared = *(gint *) schema_value == 1;
            }
            else if (ot_string_equals ("UNIQUE", column_sel->name))
            {
                column->unique = *(gint *) schema_value == 1;
            }
            else if (ot_string_equals ("DEFAULT", column_sel->name))
            {
                column->default_value = ot_sys_g_free (column->default_value);
    
                if (schema_value != NULL)
                {
                    column->default_value = g_strdup (schema_value);
                }
            }
            else if (ot_string_equals ("UNIQUE", column_sel->name))
            {
                column->unique = *(gint *) schema_value == 1;
            }
            else if (ot_string_equals ("ATTR_ORDER", column_sel->name))
            {
                column->order = *(gint *) schema_value;
            }
            else if (ot_string_equals ("SOURCE_CLASS", column_sel->name))
            {
                column_cub->inheritance = ot_sys_g_free (column_cub->inheritance);

                if (schema_value != NULL)
                {
                    if (!ot_string_equals (table_name, schema_value))
                    {
                        column_cub->inheritance = g_strdup (schema_value);
                    }
                }
            }
            else if (ot_string_equals ("IS_KEY", column_sel->name))
            {
                column->pkey_order = *(gint *) schema_value == 1 ? 1 : 0;
            }
        }

        ot_db_column_set_type (column, utype, column->size);

        g_ptr_array_add (array, column);

        ot_db_dataset_free_result_data (dataset, data_array);
    }

    g_object_unref (dataset);
    dataset = NULL;

    /*
     * 컬럼중 PK인것을 찾아 PK의 Order를 넣어준다.
     */
    GPtrArray *key_array = ot_db_connection_dup_index_key_list (connection, table_name);
    if (key_array == NULL)
    {
        array = ot_sys_g_ptr_array_free_with_element (array, g_object_unref);
        return NULL;
    }

    for (i = 0; i < array->len; i++)
    {
        OtDbColumn *column = g_ptr_array_index (array, i);

        for (j = 0; j < key_array->len; j++)
        {
            OtDbIndexKey *index_key = g_ptr_array_index (key_array, j);

            if (index_key->pkey
                    && ot_string_equals (column->name, index_key->field_name))
            {
                column->pkey_order = index_key->order;
            }
        }
    }

    key_array = ot_sys_g_ptr_array_free_with_element (key_array,
                    g_object_unref);

    return array;
}

static GPtrArray*
ot_db_connection_cub_dup_index_key_list (OtDbConnection *connection,
                                         const gchar    *table_name)
{
    OtDbDataset *dataset    = ot_db_connection_new_dataset (connection);
    GPtrArray   *array      = NULL;
    GString     *qry_tmp    = g_string_new (NULL);
    gchar        qry[2048]  = "";

    g_string_append (qry_tmp, "SELECT idk.index_name, idk.class_name, idk.key_attr_name, idk.key_order, ");
    g_string_append (qry_tmp, "       (SELECT is_primary_key FROM db_index WHERE class_name=idk.class_name AND index_name = idk.index_name) AS is_primary_key, ");
    g_string_append (qry_tmp, "       (SELECT is_unique FROM db_index WHERE class_name=idk.class_name AND index_name = idk.index_name) AS is_unique, ");
    g_string_append (qry_tmp, "       (SELECT is_reverse FROM db_index WHERE class_name=idk.class_name AND index_name = idk.index_name) AS is_reverse, ");
    g_string_append (qry_tmp, "       (SELECT def_order FROM db_attribute WHERE class_name=idk.class_name AND attr_name=idk.key_attr_name) AS field_order ");
    g_string_append (qry_tmp, "  FROM db_index_key idk WHERE class_name='%s'");

    g_sprintf (qry, qry_tmp->str, table_name);
    g_string_free (qry_tmp, TRUE);

    if (!ot_db_dataset_prepare (dataset, qry))
    {
        return NULL;
    }

    if (!ot_db_dataset_execute (dataset))
    {
        return NULL;
    }

    while (ot_db_dataset_result_next (dataset))
    {
        if (array == NULL)
        {
            array = g_ptr_array_new ();
        }

        OtDbIndexKey    *index_key  = ot_db_index_key_cub_new ();
        GPtrArray       *data_array = ot_db_dataset_dup_result_data (dataset, FALSE);

        index_key->index_name   = g_strdup (g_ptr_array_index (data_array, 0));
        index_key->class_name   = g_strdup (g_ptr_array_index (data_array, 1));
        index_key->field_name   = g_strdup (g_ptr_array_index (data_array, 2));
        index_key->order        = *((gint *) g_ptr_array_index (data_array, 3)) + 1;
        index_key->pkey         = ot_string_equals ("YES", g_ptr_array_index (data_array, 4));
        index_key->unique       = ot_string_equals ("YES", g_ptr_array_index (data_array, 5));
        index_key->reverse      = ot_string_equals ("YES", g_ptr_array_index (data_array, 6));
        index_key->field_order  = *((gint *) g_ptr_array_index (data_array, 7)) + 1;

        g_ptr_array_add (array, index_key);
        ot_db_dataset_free_result_data (dataset, data_array);
    }

    g_object_unref (dataset);
    dataset = NULL;

    return array;
}

static gchar*
ot_db_connection_cub_dup_simple_sql_select (OtDbConnection  *connection,
                                            const gchar     *table_name,
                                            GPtrArray       *fields,
                                            GPtrArray       *orders)
{
    gboolean     is_fields  = FALSE;
    GString     *sql        = g_string_new (NULL);
    gchar       *return_sql = NULL;

    g_string_append (sql, "SELECT ");

    if (fields != NULL)
    {
        if (fields->len > 0)
        {
            gint i = 0;
            for (i = 0; i < fields->len; i++)
            {
                if (i > 0)
                {
                    g_string_append (sql, ", ");
                }
                g_string_append (sql, g_ptr_array_index (fields, i));
            }

            is_fields = TRUE;
        }
    }

    if (!is_fields)
    {
        g_string_append (sql, "*");
    }

    g_string_append (sql, " FROM ");
    g_string_append (sql, table_name);

    if (orders != NULL)
    {
        if (orders->len > 0)
        {
            g_string_append (sql, " ORDER BY ");

            gint i = 0;
            for (i = 0; i < orders->len; i++)
            {
                if (i > 0)
                {
                    g_string_append (sql, ", ");
                }
                g_string_append (sql, g_ptr_array_index (orders, i));
            }
        }
    }

    return_sql = sql->str;
    g_string_free (sql, FALSE);

    return return_sql;
}

static gchar*
ot_db_connection_cub_dup_simple_sql_count (OtDbConnection   *connection,
                                           const gchar      *table_name,
                                           GPtrArray        *where_fields)
{
    GString     *sql        = g_string_new (NULL);
    gchar       *return_sql = NULL;

    g_string_append (sql, "SELECT COUNT(*) FROM ");
    g_string_append (sql, table_name);

    if (where_fields != NULL)
    {
        if (where_fields->len > 0)
        {
            g_string_append (sql, " WHERE ");

            gint i = 0;
            for (i = 0; i < where_fields->len; i++)
            {
                if (i > 0)
                {
                    g_string_append (sql, " AND ");
                }
                g_string_append (sql, g_ptr_array_index (where_fields, i));
                g_string_append (sql, "=?");
            }
        }
    }

    return_sql = sql->str;
    g_string_free (sql, FALSE);

    return return_sql;
}

static gchar*
ot_db_connection_cub_dup_simple_sql_delete (OtDbConnection  *connection,
                                            const gchar     *table_name,
                                            GPtrArray       *where_fields)
{
    GString     *sql        = g_string_new (NULL);
    gchar       *return_sql = NULL;

    g_string_append (sql, "DELETE FROM ");
    g_string_append (sql, table_name);

    if (where_fields != NULL)
    {
        if (where_fields->len > 0)
        {
            g_string_append (sql, " WHERE ");

            gint i = 0;
            for (i = 0; i < where_fields->len; i++)
            {
                if (i > 0)
                {
                    g_string_append (sql, " AND ");
                }
                g_string_append (sql, g_ptr_array_index (where_fields, i));
                g_string_append (sql, "=?");
            }
        }
    }

    return_sql = sql->str;
    g_string_free (sql, FALSE);

    return return_sql;
}

static gpointer
ot_db_connection_cub_new_dataset (OtDbConnection *connection)
{
    OtDbDataset *dataset = ot_db_dataset_cub_new ();

    ot_db_dataset_set_connection (dataset, connection);

    return dataset;
}

static OtDbColumn*
ot_db_connection_cub_new_column (OtDbConnection *connection)
{
    return ot_db_column_cub_new ();
}

static void
ot_db_connection_cub_class_init (OtDbConnectionCubClass *class)
{
    GObjectClass        *gobject_class  = G_OBJECT_CLASS (class);
    OtDbConnectionClass *cclass         = OT_DB_CONNECTION_CLASS (class);

    ot_db_connection_cub_parent_class = g_type_class_peek_parent (class);

    gobject_class->dispose = ot_db_connection_cub_dispose;

    cclass->new_clone               = ot_db_connection_cub_new_clone;
    cclass->connect                 = ot_db_connection_cub_connect;
    cclass->disconnect              = ot_db_connection_cub_disconnect;
    cclass->commit                  = ot_db_connection_cub_commit;
    cclass->rollback                = ot_db_connection_cub_rollback;
    cclass->is_connected            = ot_db_connection_cub_is_connected;
    cclass->test_connection         = ot_db_connection_cub_test_connection;
    cclass->get_conn_handle         = ot_db_connection_cub_get_conn_handle;
    cclass->is_error                = ot_db_connection_cub_is_error;
    cclass->init_error              = ot_db_connection_cub_init_error;
    cclass->dup_current_err_msg     = ot_db_connection_cub_dup_current_err_msg;
    cclass->dup_table_name_list     = ot_db_connection_cub_dup_table_name_list;
    cclass->dup_column_list         = ot_db_connection_cub_dup_column_list;
    cclass->dup_index_key_list      = ot_db_connection_cub_dup_index_key_list;
    cclass->dup_index_name_list     = ot_db_connection_cub_dup_index_name_list;
    cclass->dup_procedure_name_list = ot_db_connection_cub_dup_procedure_name_list;
    cclass->dup_simple_sql_select   = ot_db_connection_cub_dup_simple_sql_select;
    cclass->dup_simple_sql_count    = ot_db_connection_cub_dup_simple_sql_count;
    cclass->dup_simple_sql_delete   = ot_db_connection_cub_dup_simple_sql_delete;
    cclass->new_dataset             = ot_db_connection_cub_new_dataset;
    cclass->new_column              = ot_db_connection_cub_new_column;
}

static void
ot_db_connection_cub_init (OtDbConnectionCub *connection)
{
    connection->conn_handle = 0;
    connection->err_code    = 0;
    connection->err_msg     = NULL;

    connection->host        = NULL;
    connection->port        = -1;
    connection->database    = NULL;
}

GType
ot_db_connection_cub_get_type (void)
{
    static GType widget_type = 0;

    if (G_UNLIKELY (widget_type == 0))
    {
        const GTypeInfo widget_info =
        {
            sizeof (OtDbConnectionCubClass),
            NULL,       /* base_init */
            NULL,
            (GClassInitFunc) ot_db_connection_cub_class_init,
            NULL,       /* class_finalize */
            NULL,       /* class_data */
            sizeof (OtDbConnectionCub),
            0,          /* n_preallocs */
            (GInstanceInitFunc) ot_db_connection_cub_init,
            NULL,       /* value_table */
        };
        widget_type = g_type_register_static (OT_TYPE_DB_CONNECTION, "OtDbConnectionCub",
                &widget_info, 0);
    }

    return widget_type;
}



/*
 * public
 */

OtDbConnection*
ot_db_connection_cub_new (const gchar   *url,
                          const gchar   *username,
                          const gchar   *password,
                          const gchar   *encoding)
{
    OtDbConnectionCub *connection = OT_DB_CONNECTION_CUB (
            g_object_new (OT_TYPE_DB_CONNECTION_CUB, NULL));

    gboolean      parse_done    = FALSE;
    gchar       **str_arr       = NULL;

    str_arr = g_strsplit (url, OT_DB_CONNECTION_URL_PROTO_DELIMITER, 0);
    if (g_strv_length (str_arr) == 2)
    {
        if (ot_string_equals (str_arr[0], "cubrid"))
        {
            gchar *property = g_strdup (str_arr[1]);

            g_strfreev (str_arr);
            str_arr = g_strsplit (property, OT_DB_CONNECTION_URL_PROP_DELIMITER, 0);

            g_free (property);
            property = NULL;
        
            if (g_strv_length (str_arr) == 3)
            {
                connection->host = g_strdup (str_arr[0]);
                connection->port = atoi (str_arr[1]);
                connection->database = g_strdup (str_arr[2]);

                parse_done = TRUE;
            }
        }
    }

    if (str_arr != NULL)
    {
        g_strfreev (str_arr);
        str_arr = NULL;
    }

    if (!parse_done)
    {
        g_object_unref (connection);
        g_warning ("Cubrid url parse error. url : %s", url);

        return NULL;
    }


    ot_db_connection_set_url (OT_DB_CONNECTION (connection), url);
    ot_db_connection_set_username (OT_DB_CONNECTION (connection), username);
    ot_db_connection_set_password (OT_DB_CONNECTION (connection), password);
    ot_db_connection_set_encoding (OT_DB_CONNECTION (connection), encoding);

    return OT_DB_CONNECTION (connection);
}
