/* 파일 설명 적는곳

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#ifndef OTDBDATASET_H_
#define OTDBDATASET_H_

#include <glib-object.h>

#include "otdbdefine.h"
#include "otdbcolumn.h"
#include "otdbconnection.h"

G_BEGIN_DECLS

#define OT_TYPE_DB_DATASET               (ot_db_dataset_get_type ())
#define OT_DB_DATASET(object)            (G_TYPE_CHECK_INSTANCE_CAST ((object), OT_TYPE_DB_DATASET, OtDbDataset))
#define OT_DB_DATASET_CLASS(klass)       (G_TYPE_CHECK_CLASS_CAST ((klass), OT_TYPE_DB_DATASET, OtDbDatasetClass))
#define OT_IS_DB_DATASET(object)         (G_TYPE_CHECK_INSTANCE_TYPE ((object), OT_TYPE_DB_DATASET))
#define OT_IS_DB_DATASET_CLASS(klass)    (G_TYPE_CHECK_CLASS_TYPE ((klass), OT_TYPE_DB_DATASET))
#define OT_DB_DATASET_GET_CLASS(object)  (G_TYPE_INSTANCE_GET_CLASS ((object), OT_TYPE_DB_DATASET, OtDbDatasetClass))


#define OT_DB_DATASET_URL_PROTO_DELIMITER    "://"
#define OT_DB_DATASET_URL_PROP_DELIMITER     ":"


typedef struct _OtDbDataset         OtDbDataset;
typedef struct _OtDbDatasetClass    OtDbDatasetClass;

struct _OtDbDataset
{
    GInitiallyUnowned parent_instance;

    OtDbConnection *connection;

    gint fetch_size;

    GPtrArray       *req_columns;       // <OtDbColumn>
    OtDbStmtType     req_stmt_type;
    gint             req_modify_count;
    gint             req_cursor_index;
};

struct _OtDbDatasetClass
{
    GInitiallyUnownedClass parent_class;

    gboolean        (*prepare)              (OtDbDataset *dataset, const gchar *query);
    OtDbParamErr    (*set_param)            (OtDbDataset *dataset, gint index,
                                             OtDbCType ctype, gpointer dbtype,
                                             gpointer value, gint length);
    gboolean        (*execute)              (OtDbDataset *dataset);
    gboolean        (*close)                (OtDbDataset *dataset);
    GPtrArray*      (*dup_result_columns)   (OtDbDataset *dataset);
    gboolean        (*result_next)          (OtDbDataset *dataset);
    gboolean        (*result_move_cursor)   (OtDbDataset *dataset, gint row_index);
    GPtrArray*      (*dup_result_data)      (OtDbDataset *dataset, gboolean string_only);
    gboolean        (*update_result_datas)  (OtDbDataset *dataset, gint *row_indexes,
                                             GPtrArray *row_datas, gchar *table_name);
    gboolean        (*delete_result_datas)  (OtDbDataset *dataset, gint *row_indexes,
                                             gint row_count, gchar *table_name);
    gpointer        (*get_result_dbtype)    (OtDbDataset *dataset, gint column_index);
};

GType           ot_db_dataset_get_type              (void);
void            ot_db_dataset_set_connection        (OtDbDataset    *dataset,
                                                     OtDbConnection *connection);
void            ot_db_dataset_set_fetch_size        (OtDbDataset    *dataset,
                                                     gint            fetch_size);
gboolean        ot_db_dataset_prepare               (OtDbDataset *dataset,
                                                     const gchar *query);
OtDbParamErr    ot_db_dataset_set_param             (OtDbDataset    *dataset,
                                                     gint            index,
                                                     OtDbCType       ctype,
                                                     gpointer        dbtype,
                                                     gpointer        value,
                                                     gint            length);
gboolean        ot_db_dataset_execute               (OtDbDataset *dataset);
gboolean        ot_db_dataset_close                 (OtDbDataset *dataset);
GPtrArray*      ot_db_dataset_dup_result_columns    (OtDbDataset *dataset);
gboolean        ot_db_dataset_result_next           (OtDbDataset *dataset);
gboolean        ot_db_dataset_result_move_cursor    (OtDbDataset    *dataset,
                                                     gint            row_index);
GPtrArray*      ot_db_dataset_dup_result_data       (OtDbDataset    *dataset,
                                                     gboolean        string_only);
gboolean        ot_db_dataset_update_result_datas   (OtDbDataset    *dataset,
                                                     gint           *row_indexes,
                                                     GPtrArray      *row_datas,
                                                     gchar          *table_name);
gboolean        ot_db_dataset_delete_result_datas   (OtDbDataset    *dataset,
                                                     gint           *row_indexes,
                                                     gint            row_count,
                                                     gchar          *table_name);
gpointer        ot_db_dataset_get_result_dbtype     (OtDbDataset    *dataset,
                                                     gint            column_index);
void            ot_db_dataset_clear                 (OtDbDataset *dataset);
GPtrArray*      ot_db_dataset_free_result_data      (OtDbDataset    *dataset,
                                                     GPtrArray      *data_array);

G_END_DECLS

#endif /*OTDBDATASET_H_*/
