/* 파일 설명 적는곳

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#include <string.h>
#include <stdlib.h>

#include "../otak-util/otsys.h"
#include "../otak-util/otstring.h"
#include "otdbdefine.h"
#include "otdbdatasetcub.h"
#include "otdbconnectioncub.h"
#include "otdbcolumncub.h"
#include "otdbindexkey.h"
#include "cas_cci.h"


static OtDbDatasetClass *ot_db_dataset_cub_parent_class = NULL;



T_CCI_A_TYPE
ot_db_dataset_cub_get_atype_from_ctype (OtDbCType ctype)
{
    T_CCI_A_TYPE atype = 0;

    switch (ctype)
    {
        case OT_DB_C_TYPE_INT:
            atype = CCI_A_TYPE_INT;
        break;

        case OT_DB_C_TYPE_INT64:
            atype = CCI_A_TYPE_DOUBLE;
        break;

        case OT_DB_C_TYPE_FLOAT:
            atype = CCI_A_TYPE_FLOAT;
        break;

        case OT_DB_C_TYPE_DOUBLE:
            atype = CCI_A_TYPE_DOUBLE;
        break;

        case OT_DB_C_TYPE_STR:
            atype = CCI_A_TYPE_STR;
        break;

        case OT_DB_C_TYPE_STR_DATE:
        case OT_DB_C_TYPE_STR_DATETIME:
        case OT_DB_C_TYPE_STR_TIME:
            atype = CCI_A_TYPE_DATE;
        break;

        case OT_DB_C_TYPE_STR_BUF:
            atype = CCI_A_TYPE_BIT;
        break;

        default:
        break;
    }

    return atype;
}

static OtDbStmtType
ot_db_dataset_cub_get_stmt_type_from_cmd (T_CCI_SQLX_CMD cmd)
{
    OtDbStmtType stmt_type = OT_DB_STMT_TYPE_NONE;

    switch (cmd)
    {
        case SQLX_CMD_SELECT:
            stmt_type = OT_DB_STMT_TYPE_GET;
        break;

        case SQLX_CMD_INSERT:
            stmt_type = OT_DB_STMT_TYPE_PUT;
        break;

        case SQLX_CMD_UPDATE:
        case SQLX_CMD_DELETE:
            stmt_type = OT_DB_STMT_TYPE_MODIFY;
        break;

        default:
            stmt_type = OT_DB_STMT_TYPE_OTHER;
        break;
    }

    return stmt_type;
}

static void
ot_db_dataset_cub_dispose (GObject *gobject)
{
//    OtDbDatasetCub *dataset_cub = OT_DB_DATASET_CUB (gobject);

    G_OBJECT_CLASS (ot_db_dataset_cub_parent_class)->dispose (gobject);
}

gboolean
ot_db_dataset_cub_prepare (OtDbDataset  *dataset,
                           const gchar  *query)
{
    OtDbConnection      *connection     = dataset->connection;
    OtDbConnectionCub   *connection_cub = OT_DB_CONNECTION_CUB (dataset->connection);
    OtDbDatasetCub      *dataset_cub    = OT_DB_DATASET_CUB (dataset);
    T_CCI_ERROR          cci_error;
    gint                 req            = 0;

    ot_db_dataset_close (dataset);
    ot_db_connection_init_error (connection);

    if (!ot_db_connection_is_connected (connection))
    {
        connection_cub->err_msg     = g_strdup ("커넥션 핸들을 먼저 생성하여주십시요.");
        connection_cub->err_code    = -1;

        return FALSE;
    }

    req = cci_prepare (connection_cub->conn_handle, (char *) query,
            CCI_PREPARE_UPDATABLE, &cci_error);
    if (req < 0)
    {
        connection_cub->err_msg = g_strdup (cci_error.err_msg);
        connection_cub->err_code = cci_error.err_code;
        
        return FALSE;
    }

    dataset_cub->req_handle = req;

    return TRUE;
}

static OtDbParamErr
ot_db_dataset_cub_set_param (OtDbDataset    *dataset,
                             gint            index,
                             OtDbCType       ctype,
                             gpointer        dbtype,
                             gpointer        value,
                             gint            length)
{
    // TODO 구현 리턴값도
    OtDbDatasetCub  *dataset_cub    = OT_DB_DATASET_CUB (dataset);
    T_CCI_A_TYPE     cub_atype      = 0;
    T_CCI_U_TYPE     cub_utype      = *((T_CCI_U_TYPE *) dbtype);
    gint             final_index    = index;
    OtDbParamErr     return_err     = OT_DB_PARAM_ERR_NONE;

    switch (ctype)
    {
        case OT_DB_C_TYPE_FILENAME:
        case OT_DB_C_TYPE_FD:
            return OT_DB_PARAM_ERR_NOT_SUPPORT_TYPE;
        break;

        default:
            cub_atype = ot_db_dataset_cub_get_atype_from_ctype (ctype);
            if (cub_atype == 0)
            {
                return OT_DB_PARAM_ERR_UNKNOWN_TYPE;
            }
        break;
    }

    gint res = cci_bind_param (dataset_cub->req_handle, final_index,
            cub_atype, value, cub_utype, (char) CCI_BIND_PTR);
    switch (res)
    {
        case CCI_ER_REQ_HANDLE:
            return_err = OT_DB_PARAM_ERR_PREPARE;
        break;

        case CCI_ER_TYPE_CONVERSION:
            return_err = OT_DB_PARAM_ERR_MISMATCH_TYPE;
        break;

        case CCI_ER_BIND_INDEX:
            return_err = OT_DB_PARAM_ERR_OUT_BOUND;
        break;

        case CCI_ER_ATYPE:
            return_err = OT_DB_PARAM_ERR_UNKNOWN_TYPE;
        break;

        case CCI_ER_NO_MORE_MEMORY:
            return_err = OT_DB_PARAM_ERR_SYSTEM;
        break;
    }

    return return_err;
}

static gint
ot_db_dataset_cub_execute (OtDbDataset *dataset)
{
    OtDbDatasetCub      *dataset_cub        = OT_DB_DATASET_CUB (dataset);
    OtDbConnectionCub   *connection_cub     = OT_DB_CONNECTION_CUB (dataset->connection);
    T_CCI_ERROR          cci_error;
    T_CCI_QUERY_RESULT  *query_result       = NULL;
    T_CCI_SQLX_CMD       stmt_type;
    gint                 res                = 0;
    gint                 exec_result_res    = 0;

    ot_db_connection_init_error (dataset->connection);
    ot_db_dataset_clear (dataset);

    res = cci_execute (dataset_cub->req_handle, 0, 0, &cci_error);
    if (res < 0)
    {
        goto label_error;
    }

    exec_result_res = cci_execute_result (dataset_cub->req_handle, &query_result, &cci_error);
    if (exec_result_res < 0)
    {
        goto label_error;
    }

    stmt_type = CCI_QUERY_RESULT_STMT_TYPE (query_result, 1);
    if (stmt_type == SQLX_CMD_SELECT)
    {
        dataset->req_columns = ot_db_dataset_dup_result_columns (dataset);

        gint fetch_size = 0;
        if (dataset->fetch_size > 0)
        {
            fetch_size = dataset->fetch_size;
        }

        res = cci_fetch_size (dataset_cub->req_handle, fetch_size);
        if (res < 0)
        {
            strcpy (cci_error.err_msg, "fetch size 지정중 오류가 발생하였습니다.");
            cci_error.err_code = 0;

            goto label_error;
        }
    }
    else
    {
        dataset->req_modify_count = CCI_QUERY_RESULT_RESULT (query_result, 1);
    }

    dataset->req_stmt_type = ot_db_dataset_cub_get_stmt_type_from_cmd (stmt_type);
    cci_query_result_free (query_result, exec_result_res);

    return TRUE;

label_error:
    connection_cub->err_msg = g_strdup (cci_error.err_msg);
    connection_cub->err_code = cci_error.err_code;

    if (query_result != NULL)
    {
        cci_query_result_free (query_result, exec_result_res);
    }
    ot_db_dataset_close (dataset);

    return FALSE;
}

static gboolean
ot_db_dataset_cub_close (OtDbDataset *dataset)
{
    OtDbDatasetCub      *dataset_cub    = OT_DB_DATASET_CUB (dataset);
    OtDbConnectionCub   *connection_cub = OT_DB_CONNECTION_CUB (dataset->connection);
    gint                 res            = 0;

    ot_db_connection_init_error (dataset->connection);

    if (dataset_cub->req_handle > 0)
    {
        res = cci_close_req_handle (dataset_cub->req_handle);
        if (res < 0)
        {
            switch (res)
            {
                case CCI_ER_REQ_HANDLE:
                    connection_cub->err_msg = g_strdup ("Request Handle이 유효하지 않음");
                break;
    
                case CCI_ER_COMMUNICATION:
                    connection_cub->err_msg = g_strdup ("데이터를 전송/수신 할 수 없음");
                break;
            }
    
            connection_cub->err_code = res;
    
            return FALSE;
        }
        else
        {
            dataset_cub->req_handle = 0;
        }
    }

    return TRUE;
}

static GPtrArray*
ot_db_dataset_cub_dup_result_columns (OtDbDataset *dataset)
{
    OtDbDatasetCub  *dataset_cub    = OT_DB_DATASET_CUB (dataset);
    OtDbColumn      *column         = NULL;
    OtDbColumnCub   *column_cub     = NULL;
    T_CCI_COL_INFO  *col_info       = NULL;
    gint             col_count      = 0;
    GPtrArray       *array          = NULL;

    col_info = cci_get_result_info (dataset_cub->req_handle, NULL, &col_count);
    if (col_info == NULL)
    {
        return NULL;
    }

    array = g_ptr_array_new ();

    gint i = 0;
    for (i = 1; i <= col_count; i++)
    {
        T_CCI_U_TYPE utype = CCI_GET_RESULT_INFO_TYPE (col_info, i);

        column      = ot_db_column_cub_new ();
        column_cub  = OT_DB_COLUMN_CUB (column);

        ot_db_column_set_type (column, &utype, CCI_GET_RESULT_INFO_PRECISION (col_info, i));

        column->name        = g_strdup (CCI_GET_RESULT_INFO_NAME (col_info, i));
        column_cub->utype   = CCI_GET_RESULT_INFO_TYPE (col_info, i);

        g_ptr_array_add (array, column);
    }

    return array;
}

static gboolean
ot_db_dataset_cub_result_next (OtDbDataset *dataset)
{
    return ot_db_dataset_result_move_cursor (dataset, dataset->req_cursor_index + 1);
}

static gboolean
ot_db_dataset_cub_result_move_cursor (OtDbDataset   *dataset,
                                      gint           cursor_index)
{
    OtDbDatasetCub      *dataset_cub    = OT_DB_DATASET_CUB (dataset);
    OtDbConnectionCub   *connection_cub = OT_DB_CONNECTION_CUB (dataset->connection);
    T_CCI_ERROR          cci_error;
    gint                 res            = 0;

    ot_db_connection_init_error (dataset->connection);

    res = cci_cursor (dataset_cub->req_handle, cursor_index + 1,
            CCI_CURSOR_FIRST, &cci_error);
    if (res == CCI_ER_NO_MORE_DATA)
    {
        return FALSE;
    }
    else if (res < 0)
    {
        connection_cub->err_msg = g_strdup (cci_error.err_msg);
        connection_cub->err_code = cci_error.err_code;

        return FALSE;
    }

    return TRUE;
}

static GPtrArray*
ot_db_dataset_cub_dup_result_data (OtDbDataset  *dataset,
                                   gboolean      string_only)
{
    // TODO 구현
    OtDbDatasetCub      *dataset_cub    = OT_DB_DATASET_CUB (dataset);
    OtDbConnection      *connection     = OT_DB_CONNECTION (dataset->connection);
    OtDbConnectionCub   *connection_cub = OT_DB_CONNECTION_CUB (dataset->connection);
    T_CCI_ERROR          cci_error;
    GPtrArray           *req_columns    = dataset->req_columns;
    GPtrArray           *return_array   = g_ptr_array_new ();
    gint                 res            = 0;
    gboolean             str_convert    = !ot_string_equals (connection->encoding, OT_DB_DEFAULT_ENCODING);;

    gchar               *str_value      = NULL;
    gint                 int_value      = 0;
    gfloat               float_value    = 0;
    gdouble              double_value   = 0;
    T_CCI_BIT            bit_value;
    T_CCI_SET            set_value;
    T_CCI_DATE           date_value;

    ot_db_connection_init_error (connection);

    res = cci_fetch (dataset_cub->req_handle, &cci_error);
    if (res < 0)
    {
        goto label_error;
    }

    gint i = 0;
    // cci_get_data index from 1
    for (i = 1; i <= req_columns->len; i++)
    {
        void            *buffer;
        gpointer         return_value   = NULL;
        gint             indicator      = -1;
        OtDbColumn      *column = NULL;
        T_CCI_A_TYPE     atype;

        column = g_ptr_array_index (req_columns, i - 1);

        if (string_only)
        {
            atype = CCI_A_TYPE_STR;
        }
        else
        {
            atype = ot_db_dataset_cub_get_atype_from_ctype (column->ctype);
        }
 
        switch (atype)
        {
            case CCI_A_TYPE_STR:
                buffer = &str_value;
            break;

            case CCI_A_TYPE_INT:
                buffer = &int_value;
            break;

            case CCI_A_TYPE_FLOAT:
                buffer = &float_value;
            break;

            case CCI_A_TYPE_DOUBLE:
                buffer = &double_value;
            break;

            case CCI_A_TYPE_BIT:
                buffer = &bit_value;
            break;

            case CCI_A_TYPE_SET:
                buffer = &set_value;
            break;

            case CCI_A_TYPE_DATE:
                buffer = &date_value;
            break;
        }

        res = cci_get_data (dataset_cub->req_handle, i, atype, buffer, &indicator);
        if (res < 0)
        {
            goto label_error;
        }

        if (indicator < 0)
        {
            return_value = NULL;
        }
        else
        {
            switch (atype)
            {
                case CCI_A_TYPE_STR:
                    if (str_convert)
                    {
                        return_value = (gpointer) ot_db_connection_dup_str_for_view (
                                connection, str_value);
                    }
                    else
                    {
                        return_value = (gpointer) g_strdup (str_value);
                    }
                break;
    
                case CCI_A_TYPE_INT:
                    return_value                = g_malloc (sizeof (gint));
                    *((gint *) return_value)    = int_value;
                break;
    
                case CCI_A_TYPE_FLOAT:
                    return_value                = g_malloc (sizeof (gfloat));
                    *((gfloat *) return_value)   = float_value;
                break;
    
                case CCI_A_TYPE_DOUBLE:
                    return_value                = g_malloc (sizeof (gdouble));
                    *((gdouble *) return_value) = double_value;
                break;
    
                case CCI_A_TYPE_BIT:
                    {
                        OtDbStrBuf *str_buf = g_malloc (sizeof (OtDbStrBuf));
    
                        str_buf->size   = bit_value.size;
                        str_buf->data   = g_memdup (bit_value.buf, bit_value.size);
                        
                        return_value = str_buf;
                    }
                break;
    
                case CCI_A_TYPE_SET:
                    cci_set_free (set_value);
                break;
    
                case CCI_A_TYPE_DATE:
                    switch (column->ctype)
                    {
                        case OT_DB_C_TYPE_STR_DATE:
                            return_value = g_strdup_printf ("%04d%02d%02d",
                                    date_value.yr, date_value.mon, date_value.day);
                        break;
    
                        case OT_DB_C_TYPE_STR_DATETIME:
                            return_value = g_strdup_printf ("%04d%02d%02d%02d%02d%02d",
                                    date_value.yr, date_value.mon, date_value.day,
                                    date_value.hh, date_value.mm, date_value.ss);
                        break;
    
                        case OT_DB_C_TYPE_STR_TIME:
                            return_value = g_strdup_printf ("%02d%02d%02d",
                                    date_value.hh, date_value.mm, date_value.ss);
                        break;
    
                        default:
                        break;
                    }
                break;
            }
        }

        g_ptr_array_add (return_array, return_value);
    }

    return return_array;

label_error:
    connection_cub->err_msg = g_strdup (cci_error.err_msg);
    connection_cub->err_code = cci_error.err_code;

    ot_db_dataset_free_result_data (dataset, return_array);
    g_ptr_array_free (return_array, TRUE);

    return NULL;
}

static gboolean
ot_db_dataset_cub_update_result_datas (OtDbDataset  *dataset,
                                       gint         *row_indexes,
                                       GPtrArray    *row_datas,
                                       gchar        *table_name)
{
    OtDbDatasetCub      *dataset_cub    = OT_DB_DATASET_CUB (dataset);
    OtDbConnectionCub   *connection_cub = OT_DB_CONNECTION_CUB (dataset->connection);

    gint i = 0;
    for (i = 0; i < row_datas->len; i++)
    {
        gint         row_index      = row_indexes[i];
        GPtrArray   *column_datas   = g_ptr_array_index (row_datas, i);

        gint j = 0;
        for (j = 0; j < column_datas->len; j++)
        {
            OtDbColumn      *column         = NULL;
            OtDbColumnData  *column_data    = g_ptr_array_index (column_datas, j);
            T_CCI_ERROR      cci_error;
            T_CCI_BIT        bit_value;
            T_CCI_A_TYPE     atype;
            gpointer         data_final     = NULL;
            gint             res            = 0;

            column  = g_ptr_array_index (dataset->req_columns, column_data->column_index);
            atype   = ot_db_dataset_cub_get_atype_from_ctype (column->ctype);

            if (atype == CCI_A_TYPE_BIT)
            {
                bit_value.buf   = column_data->data;
                bit_value.size  = column_data->size;
                data_final      = &bit_value;
            }
            else if (column_data->data != NULL && atype == CCI_A_TYPE_STR)
            {
                data_final = ot_db_connection_dup_str_for_dbms (dataset->connection,
                                column_data->data);
            }
            else
            {
                data_final = column_data->data;
            }

            res = cci_cursor_update (dataset_cub->req_handle, row_index + 1,
                    column_data->column_index + 1, atype, data_final, &cci_error);
            if (atype == CCI_A_TYPE_STR)
            {
                if (data_final != NULL)
                {
                    g_free (data_final);
                    data_final = NULL;
                }
            }

            if (res < 0)
            {
                connection_cub->err_msg = g_strdup (cci_error.err_msg);
                connection_cub->err_code = cci_error.err_code;
            
                return FALSE;
            }
        }
    }

    return TRUE;
}

static gboolean
ot_db_dataset_cub_delete_result_datas (OtDbDataset  *dataset,
                                       gint         *row_indexes,
                                       gint          row_count,
                                       gchar        *table_name)
{
    g_assert (table_name != NULL && row_count > 0);

    OtDbConnection      *conn       = dataset->connection;
    OtDbConnectionCub   *conn_cub   = OT_DB_CONNECTION_CUB (conn);
    GPtrArray           *keys       = NULL;
    GPtrArray           *columns    = dataset->req_columns;
    GPtrArray           *fields     = g_ptr_array_new();
    GPtrArray           *data_array = NULL;
    GPtrArray           *cnt_arr    = NULL;
    OtDbDataset         *cnt_dset   = NULL;
    OtDbDataset         *del_dset   = NULL;
    gchar               *sql        = NULL;
    gboolean             result     = FALSE;
    gint                *fld_ords   = g_malloc0 (sizeof (gint) * columns->len);
    gint                 i          = 0;
    gint                 j          = 0;

    keys = ot_db_connection_dup_index_key_list (conn, table_name);
    if (keys == NULL)
    {
        goto label_end;
    }

    if (keys->len > 0)
    {
        for (i = 0; i < keys->len; i++)
        {
            OtDbIndexKey    *key    = g_ptr_array_index (keys, i);
            OtDbColumn      *column = g_ptr_array_index (columns, i);

            if (ot_db_column_is_using_where (column))
            {
                g_ptr_array_add (fields, g_strdup (key->field_name));
                fld_ords[i] = key->field_order;
            }
        }
    }
    else
    {
        for (i = 0; i < columns->len; i++)
        {
            OtDbColumn *column = g_ptr_array_index (columns, i);

            if (ot_db_column_is_using_where (column))
            {
                g_ptr_array_add (fields, g_strdup (column->name));
                fld_ords[i] = column->order;
            }
        }
    }

    if (fields->len <= 0)
    {
        goto label_end;
    }

    cnt_dset = ot_db_connection_new_dataset (conn);
    del_dset = ot_db_connection_new_dataset (conn);

    /* 삭제 자료 Counting prepare */
    sql = ot_db_connection_dup_simple_sql_count (conn, table_name, fields);
    if (!ot_db_dataset_prepare (cnt_dset, sql))
    {
        goto label_end;
    }
    sql = ot_sys_g_free (sql);

    /* 삭제 prepare */
    sql = ot_db_connection_dup_simple_sql_delete (conn, table_name, fields);
    if (!ot_db_dataset_prepare (del_dset, sql))
    {
        goto label_end;
    }
    sql = ot_sys_g_free (sql);

    for (i = 0; i < row_count; i++)
    {
        if (!ot_db_dataset_result_move_cursor (dataset, row_indexes[i]))
        {
            goto label_end;
        }

        data_array = ot_db_dataset_dup_result_data (dataset, FALSE);
        if (data_array == NULL)
        {
            goto label_end;
        }

        /* 위에서 준비한 where절의 파라미터들을 count, delete에 바인딩 */
        for (j = 0; j < fields->len; j++)
        {
            gint         index  = fld_ords[j] - 1;
            OtDbColumn  *column = g_ptr_array_index (columns, index);
            gpointer     dbtype = ot_db_dataset_get_result_dbtype (dataset,
                                    index);

            if (ot_db_dataset_set_param (cnt_dset, j + 1, column->ctype, dbtype,
                    g_ptr_array_index (data_array, index), -1) != OT_DB_PARAM_ERR_NONE)
            {
                goto label_end;
            }

            if (ot_db_dataset_set_param (del_dset, j + 1, column->ctype, dbtype,
                    g_ptr_array_index (data_array, index), -1) != OT_DB_PARAM_ERR_NONE)
            {
                goto label_end;
            }
        }

        gboolean is_del = FALSE;

        /* 삭제 자료 건수 */
        if (ot_db_dataset_execute (cnt_dset))
        {
            if (ot_db_dataset_result_next (cnt_dset))
            {
                cnt_arr = ot_db_dataset_dup_result_data (cnt_dset, FALSE);
                if (cnt_arr != NULL)
                {
                    gint *cnt = g_ptr_array_index (cnt_arr, 0);

                    if (*cnt == 1)
                    {
                        if (ot_db_dataset_execute (del_dset))
                        {
                            is_del = TRUE;
                        }
                    }
                    else
                    {
                        conn_cub->err_msg   = g_strdup ("삭제하려는 조건의 자료가 1개가 아닙니다.");
                        conn_cub->err_code  = -1;
                    }

                    cnt_arr = ot_db_dataset_free_result_data (cnt_dset,
                                cnt_arr);
                }
            }
        }

        data_array = ot_db_dataset_free_result_data (dataset, data_array);

        if (!is_del)
        {
            goto label_end;
        }
    }

    result = TRUE;


label_end:

    if (cnt_dset != NULL)
    {
        g_object_unref (cnt_dset);
        cnt_dset = NULL;
    }

    if (del_dset != NULL)
    {
        g_object_unref (del_dset);
        del_dset = NULL;
    }

    keys = ot_sys_g_ptr_array_free_with_element (keys, g_object_unref);
    fields = ot_sys_g_ptr_array_free_with_element (fields, g_free);
    fld_ords = ot_sys_g_free (fld_ords);
    sql = ot_sys_g_free (sql);

    data_array = ot_db_dataset_free_result_data (dataset, data_array);

    return result;
}

static gpointer
ot_db_dataset_cub_get_result_dbtype (OtDbDataset    *dataset,
                                     gint            column_index)
{
    OtDbColumnCub *column_cub = g_ptr_array_index (dataset->req_columns, column_index);

    return &column_cub->utype;
}

static void
ot_db_dataset_cub_class_init (OtDbDatasetCubClass *class)
{
    GObjectClass        *gobject_class  = G_OBJECT_CLASS (class);
    OtDbDatasetClass    *cclass         = OT_DB_DATASET_CLASS (class);

    ot_db_dataset_cub_parent_class = g_type_class_peek_parent (class);

    gobject_class->dispose = ot_db_dataset_cub_dispose;

    cclass->prepare             = ot_db_dataset_cub_prepare;
    cclass->set_param           = ot_db_dataset_cub_set_param;
    cclass->execute             = ot_db_dataset_cub_execute;
    cclass->close               = ot_db_dataset_cub_close;
    cclass->dup_result_columns  = ot_db_dataset_cub_dup_result_columns;
    cclass->result_next         = ot_db_dataset_cub_result_next;
    cclass->result_move_cursor  = ot_db_dataset_cub_result_move_cursor;
    cclass->dup_result_data     = ot_db_dataset_cub_dup_result_data;
    cclass->update_result_datas = ot_db_dataset_cub_update_result_datas;
    cclass->delete_result_datas = ot_db_dataset_cub_delete_result_datas;
    cclass->get_result_dbtype   = ot_db_dataset_cub_get_result_dbtype;
}

static void
ot_db_dataset_cub_init (OtDbDatasetCub *dataset)
{
    dataset->req_handle = 0;
}

GType
ot_db_dataset_cub_get_type (void)
{
    static GType widget_type = 0;

    if (G_UNLIKELY (widget_type == 0))
    {
        const GTypeInfo widget_info =
        {
            sizeof (OtDbDatasetCubClass),
            NULL,       /* base_init */
            NULL,
            (GClassInitFunc) ot_db_dataset_cub_class_init,
            NULL,       /* class_finalize */
            NULL,       /* class_data */
            sizeof (OtDbDatasetCub),
            0,          /* n_preallocs */
            (GInstanceInitFunc) ot_db_dataset_cub_init,
            NULL,       /* value_table */
        };
        widget_type = g_type_register_static (OT_TYPE_DB_DATASET, "OtDbDatasetCub",
                &widget_info, 0);
    }

    return widget_type;
}



/*
 * public
 */

OtDbDataset*
ot_db_dataset_cub_new (void)
{
    OtDbDatasetCub *dataset = OT_DB_DATASET_CUB (g_object_new (OT_TYPE_DB_DATASET_CUB, NULL));

    return OT_DB_DATASET (dataset);
}
