/* 파일 설명 적는곳

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#include <stdlib.h>
#include <gtk/gtk.h>

#include "otstring.h"
#include "otgtk.h"
#include "otsys.h"

GtkWidget*
ot_gtk_widget_free (GtkWidget *widget)
{
    g_object_ref_sink (widget);
    gtk_widget_destroy (widget);
    g_object_unref (widget);

    return NULL;
}

void
ot_gtk_widget_move_container (GtkWidget     *widget,
                              GtkContainer  *container)
{
    GtkWidget *parent = gtk_widget_get_parent (widget);

    g_object_ref_sink (widget);
    if (parent != NULL)
    {
        gtk_container_remove (GTK_CONTAINER (parent), widget);
    }
    gtk_container_add (GTK_CONTAINER (container), widget);
    g_object_unref (widget);
}

gboolean
ot_gtk_list_store_compare_value (GtkListStore   *store,
                                 GtkTreeIter    *iter,
                                 GType           type,
                                 gint            index,
                                 gpointer        value)
{
    GtkTreeModel    *model          = GTK_TREE_MODEL (store);
    gchar           *str_value      = NULL;
    gint             int_value      = 0;
    gint64           int64_value    = 0;
    gfloat           float_value    = 0.0;
    gdouble          double_value   = 0.0;
    gboolean         return_value   = TRUE;

    switch (type)
    {
        case G_TYPE_STRING:
            gtk_tree_model_get (model, iter, index, &str_value, -1);
            return_value = ot_string_equals (str_value, value);
            str_value = ot_sys_g_free (str_value);
        break;

        case G_TYPE_INT:
            gtk_tree_model_get (model, iter, index, &int_value, -1);
            return_value = (int_value == *(gint *) value);
        break;

        case G_TYPE_INT64:
            gtk_tree_model_get (model, iter, index, &int64_value, -1);
            return_value = (int64_value == *(gint64 *) value);
        break;

        case G_TYPE_FLOAT:
            gtk_tree_model_get (model, iter, index, &float_value, -1);
            return_value = (float_value == *(gfloat *) value);
        break;

        case G_TYPE_DOUBLE:
            gtk_tree_model_get (model, iter, index, &double_value, -1);
            return_value = (double_value == *(gdouble *) value);
        break;

        default:
            return FALSE;
        break;
    }

    return return_value;
}

gboolean
ot_gtk_list_store_set_value (GtkListStore   *store,
                             GtkTreeIter    *iter,
                             GType           type,
                             gint            index,
                             gpointer        value,
                             gboolean        compare)
{
    if (compare)
    {
        if (ot_gtk_list_store_compare_value (store, iter, type, index, value))
        {
            return FALSE;
        }
    }

    if (value == NULL)
    {
        gtk_list_store_set (store, iter, index, value, -1);

        return TRUE;
    }

    switch (type)
    {
        case G_TYPE_STRING:
            gtk_list_store_set (store, iter, index, value, -1);
        break;

        case G_TYPE_INT:
            gtk_list_store_set (store, iter, index, *(gint *) value, -1);
        break;

        case G_TYPE_INT64:
            gtk_list_store_set (store, iter, index, *(gint64 *) value, -1);
        break;

        case G_TYPE_FLOAT:
            gtk_list_store_set (store, iter, index, *(gfloat *) value, -1);
        break;

        case G_TYPE_DOUBLE:
            gtk_list_store_set (store, iter, index, *(gdouble *) value, -1);
        break;

        case G_TYPE_POINTER:
            gtk_list_store_set (store, iter, index, value, -1);
        break;

        default:
            return FALSE;
        break;
    }

    return TRUE;
}

gboolean
ot_gtk_list_store_valid_from_string (GtkListStore   *store,
                                     GType           type,
                                     const gchar    *value)
{

    switch (type)
    {
        case G_TYPE_STRING:
            return TRUE;
        break;

        case G_TYPE_INT:
        case G_TYPE_INT64:
            if (ot_string_is_empty (value) || !ot_string_is_numeric (value))
            {
                return FALSE;
            }
        break;

            if (ot_string_is_empty (value) || !ot_string_is_numeric (value))
            {
                return FALSE;
            }
        break;

        case G_TYPE_FLOAT:
        case G_TYPE_DOUBLE:
            if (ot_string_is_empty (value) || !ot_string_is_float (value))
            {
                return FALSE;
            }
        break;

        default:
            return FALSE;
        break;
    }

    return FALSE;
}

gboolean
ot_gtk_list_store_set_value_from_string (GtkListStore   *store,
                                         GtkTreeIter    *iter,
                                         GType           type,
                                         gint            index,
                                         const gchar    *value,
                                         gboolean        compare)
{
    gint        int_value       = 0;
    gint64      int64_value     = 0;
    gfloat      float_value     = 0.0;
    gdouble     double_value    = 0.0;
    gpointer    value_pointer   = NULL;

    if (!ot_gtk_list_store_valid_from_string (store, type, value))
    {
        return FALSE;
    }

    switch (type)
    {
        case G_TYPE_STRING:
            value_pointer = (gpointer) value;
        break;

        case G_TYPE_INT:
            int_value = atoi (value);
            value_pointer = &int_value;
        break;

        case G_TYPE_INT64:
            int64_value = atol (value);
            value_pointer = &int64_value;
        break;

        case G_TYPE_FLOAT:
            float_value = atof (value);
            value_pointer = &float_value;
        break;

        case G_TYPE_DOUBLE:
            double_value = atof (value);
            value_pointer = &double_value;
        break;

        default:
            return FALSE;
        break;
    }

    return ot_gtk_list_store_set_value (store, iter, type, index, value_pointer, compare);
}
