/* 파일 설명 적는곳

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#include <string.h>

#include "otstring.h"

gchar*
ot_string_dup_ftrim (const gchar *str)
{
    gchar *return_str = NULL;

    if (str == NULL)
    {
        return NULL;
    }
    if (strlen (str) == 0)
    {
        return g_strdup (str);
    }

    // 위의 검증으로 아래는 무조건 1이상의 길이이다.
    gint i = 0;
    for (i = 0; i > strlen (str); i++)
    {
        if (str[i] != ' ')
        {
            break;
        }
    }

    return_str = g_strdup (&(str[i]));

    return return_str;
}

gchar*
ot_string_dup_rtrim (const gchar *str)
{
    gchar *return_str = NULL;

    if (str == NULL)
    {
        return NULL;
    }
    if (strlen (str) == 0)
    {
        return g_strdup (str);
    }

    // 위의 검증으로 아래는 무조건 1이상의 길이이다.
    gint i = 0;
    for (i = strlen (str) - 1; i >= 0; i--)
    {
        if (str[i] != ' ')
        {
            break;
        }
    }

    return_str = g_strndup (str, i + 1);

    return return_str;
}

gchar*
ot_string_dup_trim (const gchar *str)
{
    gchar   *return_str = NULL;
    gchar   *ftrim_str  = NULL;

    ftrim_str   = ot_string_dup_ftrim (str);
    return_str  = ot_string_dup_rtrim (ftrim_str);

    if (ftrim_str != NULL)
    {
        g_free (ftrim_str);
        ftrim_str = NULL;
    }

    return return_str;
}

gboolean
ot_string_is_numeric (const gchar *str)
{
    gint        i;
    gboolean    numeric = TRUE;

    for (i = 0; i < strlen (str); i++)
    {
        if (!g_ascii_isdigit (str[i]))
        {
            numeric = FALSE;
            break;
        }
    }

    return numeric;
}

gboolean
ot_string_is_float (const gchar *str)
{
    gint        i;
    gboolean    numeric = TRUE;

    for (i = 0; i < strlen (str); i++)
    {
        if (!g_ascii_isdigit (str[i]) && str[i] != '.')
        {
            numeric = FALSE;
            break;
        }
    }

    return numeric;
}

gboolean
ot_string_is_alpha (const gchar *str)
{
    gint        i;
    gboolean    alpha = TRUE;

    for (i = 0; i < strlen (str); i++)
    {
        if (!g_ascii_isalpha (str[i]))
        {
            alpha = FALSE;
            break;
        }
    }

    return alpha;
}

gboolean
ot_string_is_alphanum (const gchar *str)
{
    gint        i;
    gboolean    alphanum = TRUE;

    for (i = 0; i < strlen (str); i++)
    {
        if (!g_ascii_isalnum (str[i]))
        {
            alphanum = FALSE;
            break;
        }
    }

    return alphanum;
}

gboolean
ot_string_is_onebyte (const gchar *str)
{
    gint            i;
    gboolean        onebyte = TRUE;
    unsigned char   asc     = 0;

    for (i = 0; i < strlen (str); i++)
    {
        asc = str[i];
        if (asc > 127)
        {
            onebyte = FALSE;
            break;
        }
    }

    return onebyte;
}

gboolean
ot_string_equals (const gchar   *str1,
                  const gchar   *str2)
{
    if (str1 == str2)
    {
        return TRUE;
    }

    if (str1 == NULL || str2 == NULL)
    {
        return FALSE;
    }

    if (strlen (str1) != strlen (str2))
    {
        return FALSE;
    }

    return (g_ascii_strncasecmp (str1, str2, strlen (str1)) == 0);
}

gboolean
ot_string_is_empty (const gchar *str)
{
    if (str == NULL)
    {
        return FALSE;
    }

    return strlen (str) == 0;
}
