/* 파일 설명 적는곳

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#include <stdlib.h>
#include <string.h>
#include <glib.h>



/*
 * private
 */

static gint
ot_sys_path_get_before_last (const gchar *path)
{
    gint last_slash = -1;

    gint i;
    for (i = 0; i < strlen (path); i++)
    {
        if (path[i] == '/')
        {
            last_slash = i;
        }
    }

    return last_slash;
}



/*
 * public
 */

gpointer
ot_sys_g_free (gpointer value)
{
    if (value != NULL)
    {
        g_free (value);
    }

    return NULL;
}

gpointer
ot_sys_g_ptr_array_free_with_element (GPtrArray         *array,
                                      GDestroyNotify     element_destroy_notify)
{
    g_assert (element_destroy_notify != NULL);

    if (array == NULL)
    {
        return NULL;
    }

    gint i = 0;
    for (i = 0; i < array->len; i++)
    {
        gpointer value = g_ptr_array_index (array, i);
        if (value == NULL)
        {
            continue;
        }

        element_destroy_notify (value);
        value = NULL;
    }
    g_ptr_array_free (array, TRUE);

    return NULL;
}

gchar
*ot_sys_exec_abs_path_new (const gchar *exec_path)
{
    gint     position       = -1;
    gchar   *pwd            = NULL;
    gchar   *exec_dir_path  = NULL;
    gchar   *return_path    = NULL;

    // 실행명령을 내린 경로 (PWD) 가져오기
    if (exec_path[0] == '/')
    {
        pwd = g_strdup ("");
    }
    else
    {
        pwd = g_strdup_printf ("%s", getenv ("PWD"));
    }

    // 경로의 마지막 1칸전 위치 가져오기
    position = ot_sys_path_get_before_last (exec_path);
    if (position > -1)
    {
        exec_dir_path = g_strndup (exec_path, position);

        return_path = g_strdup_printf ("%s/%s", pwd, exec_dir_path);
    }
    else
    {
        // PATH 가 정의된 실행파일을 실행한 경우 (유닉스 계열에서는 PATH 정의 없이는 ./라도 붙은 채로 실행되어야하므로 현재 구간은 수행되어지지 않는다)
        return_path = NULL;
    }

    // free
    pwd = ot_sys_g_free (pwd);
    exec_dir_path = ot_sys_g_free (exec_dir_path);

    return (return_path);
}
