/* 파일 설명 적는곳

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#include <glib.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

#include "otxml.h"



static xmlNodePtr
ot_xml_get_node_by_name (xmlNodePtr      xml_node,
                         const gchar    *node_name,
                         gint            index)
{
    xmlNodePtr  curr_node   = xml_node;
    gint        curr_index  = 0;

    while (curr_node != NULL)
    {
        if (!xmlStrcmp (curr_node->name, (const xmlChar *) node_name))
        {
            if (curr_index == index)
            {
                break;
            }
            curr_index++;
        }

        curr_node = curr_node->next;
    }

    return curr_node;
}



xmlDocPtr
ot_xml_new (const gchar *filename)
{
    xmlDocPtr xml_doc = NULL;

    xml_doc = xmlParseFile (filename);

    return xml_doc;
}

xmlDocPtr
ot_xml_free (xmlDocPtr xml_doc)
{
    xmlFreeDoc (xml_doc);

    return NULL;
}

xmlNodePtr
ot_xml_get_child_node (xmlNodePtr node,
                       ...)
{
    xmlNodePtr   curr_node  = node;
    va_list      arg_list   = NULL;

    va_start (arg_list, node);
    while (TRUE)
    {
        const gchar *node_name  = va_arg (arg_list, const gchar *);
        gint         node_index = 0;

        if (node_name == NULL)
        {
            break;
        }

        node_index = va_arg (arg_list, gint);

        curr_node = curr_node->xmlChildrenNode;

        curr_node = ot_xml_get_node_by_name (curr_node, node_name, node_index);
        if (curr_node == NULL)
        {
            break;
        }
    }
    va_end(arg_list);
    
    return curr_node;
}

xmlNodePtr
ot_xml_add_child_node (xmlNodePtr    parent,
                       const gchar  *name,
                       const gchar  *value)
{
    return xmlNewChild (parent, NULL, (const xmlChar *) name,
            (const xmlChar *) value);
}

xmlNodePtr
ot_xml_remove_node (xmlNodePtr node)
{
    xmlUnlinkNode (node);
    xmlFreeNode (node);

    return NULL;
}

gboolean
ot_xml_remove_child_node (xmlNodePtr     parent,
                          const gchar   *name,
                          gint           index)
{
    xmlNodePtr c_nd = NULL;

    c_nd = ot_xml_get_child_node (parent, name, index, NULL);
    if (c_nd != NULL)
    {
        ot_xml_remove_node (c_nd);

        return TRUE;
    }

    return FALSE;
}


gchar*
ot_xml_dup_value (xmlDocPtr     xml_doc,
                  xmlNodePtr    node)
{
    gchar   *return_value   = NULL;
    xmlChar *value          = xmlNodeListGetString (xml_doc, node->xmlChildrenNode, 1);

    if (value != NULL)
    {
        return_value = g_strdup ((gchar *) value);
    }

    xmlFree (value);
    value = NULL;

    return return_value;
}

void
ot_xml_set_value (xmlNodePtr    node,
                  const gchar   *value)
{
    xmlNodeSetContent (node->xmlChildrenNode, (const xmlChar *) value);
}

gchar*
ot_xml_dup_attribute (xmlNodePtr     node,
                      const gchar   *attr_name)
{
    gchar   *return_value   = NULL;
    xmlChar *value          = xmlGetProp (node, (xmlChar *) attr_name);

    if (value != NULL)
    {
        return_value = g_strdup ((gchar *) value);
    }

    xmlFree (value);
    value = NULL;

    return return_value;
}

void
ot_xml_set_attribute (xmlNodePtr     node,
                      const gchar   *attr_name,
                      const gchar   *value)
{
    xmlSetProp (node, (const xmlChar *) attr_name, (const xmlChar *) value);
}
