/* 파일 설명 적는곳

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#include <gtk/gtk.h>

#include "../otak-util/otgtk.h"
#include "otgridrenderertext.h"
#include "otgrid.h"

static GtkTreeViewClass *ot_grid_parent_class = NULL;



/*
 * signal
 */

void
ot_grid_renderer_edited_cb (GtkCellRendererText *renderer,
                            gchar               *path,
                            gchar               *new_text,
                            gpointer             user_data)
{
    OtGridRendererText  *renderer_text  = OT_GRID_RENDERER_TEXT (renderer);
    OtGrid              *grid           = OT_GRID (user_data);
    OtGridClass         *class          = OT_GRID_GET_CLASS (grid);
    GtkTreeModel        *model          = gtk_tree_view_get_model (GTK_TREE_VIEW (grid));
    GtkListStore        *store          = GTK_LIST_STORE (model);
    GtkTreeIter          iter;
    GType                type           = G_TYPE_NONE;
    gboolean             is_valid       = FALSE;
    gboolean             is_changed     = TRUE;

    gtk_tree_model_get_iter_from_string (model, &iter, path);

    type        = gtk_tree_model_get_column_type (model,
                    renderer_text->renderer_index);
    is_valid    = ot_gtk_list_store_valid_from_string (store, type, new_text);

    if (is_valid && class->changed != NULL)
    {
        is_changed = class->changed (grid, OT_GRID_RENDERER_TEXT (renderer),
                        &iter, new_text);
    }

    if (is_changed)
    {
        ot_gtk_list_store_set_value_from_string (store, &iter,
                gtk_tree_model_get_column_type (model, renderer_text->renderer_index),
                renderer_text->renderer_index, new_text, TRUE);
    }
}



/*
 * private
 */

static void
ot_grid_destroy (GtkObject *object)
{
//    OtGrid *grid = OT_GRID (object);

    (* GTK_OBJECT_CLASS (ot_grid_parent_class)->destroy) (object);
}

static void
ot_grid_cursor_changed (GtkTreeView *treeview)
{
    OtGrid              *grid       = OT_GRID (treeview);
    GtkTreePath         *path       = NULL;
    GtkTreeViewColumn   *column     = NULL;
    GList               *renderers  = NULL;

    gtk_tree_view_get_cursor (treeview, &path, &column);

    if (path != NULL)
    {
        grid->cursor_row_index = (gtk_tree_path_get_indices (path))[0];
        gtk_tree_path_free (path);
    }

    if (column != NULL)
    {
        renderers = gtk_tree_view_column_get_cell_renderers (column);
        grid->cursor_renderer = renderers->data;
    
        g_list_free (renderers);
    }

    if (ot_grid_parent_class->cursor_changed != NULL)
    {
        ot_grid_parent_class->cursor_changed (treeview);
    }
}

/* class init */
static void
ot_grid_class_init (OtGridClass *klass)
{
    GtkObjectClass      *object_class   = GTK_OBJECT_CLASS (klass);
    GtkTreeViewClass    *treeview_class = GTK_TREE_VIEW_CLASS (klass);

    ot_grid_parent_class = g_type_class_peek_parent (klass);

    object_class->destroy = ot_grid_destroy;

    treeview_class->cursor_changed = ot_grid_cursor_changed;

    klass->changed = NULL;
}

/* instance init */
static void
ot_grid_init (OtGrid *grid)
{
    grid->cursor_row_index  = -1;
    grid->cursor_renderer   = NULL;
}


/* get type */
GType
ot_grid_get_type (void)
{
    static GType widget_type = 0;

    if (G_UNLIKELY (widget_type == 0))
    {
        const GTypeInfo widget_info =
        {
            sizeof (OtGridClass),
            NULL,       /* base_init */
            NULL,
            (GClassInitFunc) ot_grid_class_init,
            NULL,       /* class_finalize */
            NULL,       /* class_data */
            sizeof (OtGrid),
            0,          /* n_preallocs */
            (GInstanceInitFunc) ot_grid_init,
            NULL,       /* value_table */
        };
        widget_type = g_type_register_static (GTK_TYPE_TREE_VIEW, "OtGrid",
                &widget_info, 0);
    }

    return widget_type;
}



/*
 * public
 */

OtGrid*
ot_grid_new (void)
{
    OtGrid *grid = OT_GRID (g_object_new (OT_TYPE_GRID, NULL));

    return grid;
}

OtGridRendererText*
ot_grid_get_cursor_renderer (OtGrid *grid)
{
    return grid->cursor_renderer;
}

void
ot_grid_construct_columns (OtGrid        *grid,
                           gint           num_cols,
                           GType         *types,
                           const gchar  **column_titles)
{
    GtkTreeView         *treeview       = GTK_TREE_VIEW (grid);
    GtkListStore        *store          = NULL;
    GtkTreeViewColumn   *column         = NULL;
    GtkCellRenderer     *renderer       = NULL;
    const gchar         *column_title   = NULL;

    store = GTK_LIST_STORE (gtk_tree_view_get_model (treeview));
    if (store != NULL)
    {
        gtk_list_store_clear (store);
    }

    store = gtk_list_store_newv (num_cols, types);
    gtk_tree_view_set_model (treeview, GTK_TREE_MODEL (store));
    g_object_unref (store);

    GList *columns          = gtk_tree_view_get_columns (treeview);
    GList *columns_start    = columns;
    while (columns != NULL)
    {
        column = GTK_TREE_VIEW_COLUMN (columns->data);

        gtk_tree_view_column_clear (column);
        gtk_tree_view_remove_column (treeview, column);

        columns = columns->next;
    }
    g_list_free (columns_start);

    gint i = 0;
    for (i = 0; i < num_cols; i++)
    {
        if (types[i] == G_TYPE_POINTER)
        {
            continue;
        }

        if (column_titles != NULL)
        {
            column_title = column_titles[i];
        }
        else
        {
            column_title = NULL;
        }

        renderer = GTK_CELL_RENDERER (ot_grid_renderer_text_new ());
        ot_grid_renderer_text_set_column_index (OT_GRID_RENDERER_TEXT (renderer), i);
        ot_grid_renderer_text_set_renderer_index (OT_GRID_RENDERER_TEXT (renderer), i);
        g_signal_connect ((gpointer) renderer, "edited",
                G_CALLBACK (ot_grid_renderer_edited_cb), (gpointer) grid);

        column = gtk_tree_view_column_new_with_attributes (column_title, renderer, "text", i, NULL);
        gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
    }

}

void
ot_grid_append_row_from_array (OtGrid       *grid,
                               GPtrArray    *array)
{
    GtkTreeIter      iter;
    GtkTreeModel    *model  = gtk_tree_view_get_model (GTK_TREE_VIEW (grid));
    GtkListStore    *store  = GTK_LIST_STORE (model);

    gtk_list_store_append (store, &iter);

    ot_grid_set_row_from_array (grid, &iter, array);
}

void
ot_grid_delete_row (OtGrid      *grid,
                    GtkTreeIter *iter)
{
    GtkTreeView         *tview  = GTK_TREE_VIEW (grid);
    GtkTreeModel        *model  = gtk_tree_view_get_model (tview);
    GtkTreeSelection    *sel    = gtk_tree_view_get_selection (tview);
    GtkTreeIter          s_iter;

    if (gtk_tree_selection_get_selected (sel, NULL, &s_iter))
    {
        GtkTreePath *path   = gtk_tree_model_get_path (model, iter);
        GtkTreePath *s_path = gtk_tree_model_get_path (model, &s_iter);

        if (gtk_tree_path_compare (path, s_path) == 0)
        {
            if (!ot_grid_select_prev_row (grid))
            {
                ot_grid_select_next_row (grid);
            }
        }

        gtk_tree_path_free (path);
        path = NULL;
        gtk_tree_path_free (s_path);
        s_path = NULL;
    }

    gtk_list_store_remove (GTK_LIST_STORE (model), iter);
}

void
ot_grid_set_row_from_array (OtGrid      *grid,
                            GtkTreeIter *iter,
                            GPtrArray   *array)
{
    GtkTreeModel    *model  = gtk_tree_view_get_model (GTK_TREE_VIEW (grid));
    GtkListStore    *store  = GTK_LIST_STORE (model);

    gint i = 0;
    for (i = 0; i < array->len; i++)
    {
        gpointer value = g_ptr_array_index (array, i);
        ot_gtk_list_store_set_value (store, iter,
                gtk_tree_model_get_column_type (GTK_TREE_MODEL (store), i),
                i, value, FALSE);
    }
}

void
ot_grid_set_resizable (OtGrid   *grid,
                       gboolean  resizable)
{
    GList   *columns        = gtk_tree_view_get_columns (GTK_TREE_VIEW (grid));
    GList   *columns_start  = columns;

    while (columns != NULL)
    {
        gtk_tree_view_column_set_resizable (GTK_TREE_VIEW_COLUMN (columns->data), resizable);
        columns = columns->next;
    }
    g_list_free (columns_start);
}

void
ot_grid_set_clickable (OtGrid   *grid,
                       gboolean  clickable)
{
    GList   *columns        = gtk_tree_view_get_columns (GTK_TREE_VIEW (grid));
    GList   *columns_start  = columns;

    while (columns != NULL)
    {
        gtk_tree_view_column_set_clickable (GTK_TREE_VIEW_COLUMN (columns->data), clickable);
        columns = columns->next;
    }
    g_list_free (columns_start);
}

void
ot_grid_set_editable (OtGrid    *grid,
                      gboolean   editable)
{
    GList   *columns        = gtk_tree_view_get_columns (GTK_TREE_VIEW (grid));
    GList   *columns_start  = columns;

    while (columns != NULL)
    {
        GList *renderers = gtk_tree_view_column_get_cell_renderers (
                            GTK_TREE_VIEW_COLUMN (columns->data));

        g_object_set (GTK_CELL_RENDERER (renderers->data), "editable", editable, NULL);
        g_list_free (renderers);

        columns = columns->next;
    }
    g_list_free (columns_start);
}

void
ot_grid_set_start_editing_aft (OtGrid   *grid,
                               gpointer  start_editing_aft)
{
    GList   *columns        = gtk_tree_view_get_columns (GTK_TREE_VIEW (grid));
    GList   *columns_start  = columns;

    while (columns != NULL)
    {
        GList *renderers = gtk_tree_view_column_get_cell_renderers (
                            GTK_TREE_VIEW_COLUMN (columns->data));

        ot_grid_renderer_text_set_start_editing_aft (renderers->data,
                start_editing_aft);
        g_list_free (renderers);

        columns = columns->next;
    }
    g_list_free (columns_start);
}

void
ot_grid_clear_datas (OtGrid *grid)
{
    GtkListStore *store = NULL;

    store = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (grid)));
    if (store != NULL)
    {
        gtk_list_store_clear (store);
    }
}

void
ot_grid_unselect_all (OtGrid *grid)
{
    GtkTreeSelection *sel = NULL;

    sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (grid));
    gtk_tree_selection_unselect_all (sel);
}

GtkTreePath*
ot_grid_dup_first_row_path (OtGrid *grid)
{
    GtkTreeModel    *model  = gtk_tree_view_get_model (GTK_TREE_VIEW (grid));
    gint             cnt    = gtk_tree_model_iter_n_children (model, NULL);

    if (cnt < 1)
    {
        return NULL;
    }

    return gtk_tree_path_new_first ();
}

GtkTreePath*
ot_grid_dup_last_row_path (OtGrid *grid)
{
    GtkTreeModel    *model  = gtk_tree_view_get_model (GTK_TREE_VIEW (grid));
    gint             cnt    = gtk_tree_model_iter_n_children (model, NULL);

    if (cnt < 1)
    {
        return NULL;
    }

    return gtk_tree_path_new_from_indices (cnt - 1, -1);
}

void
ot_grid_select_row_with_scroll (OtGrid      *grid,
                                GtkTreePath *path)
{
    GtkTreeView         *tview  = GTK_TREE_VIEW (grid);
    GtkTreeSelection    *sel    = gtk_tree_view_get_selection (tview);

    gtk_tree_selection_select_path (sel, path);
    gtk_tree_view_scroll_to_cell (tview, path, NULL, FALSE, 0, 0);
}

gboolean
ot_grid_select_first_row (OtGrid *grid)
{
    GtkTreePath *path = ot_grid_dup_first_row_path (grid);

    if (path != NULL)
    {
        ot_grid_select_row_with_scroll (grid, path);
        gtk_tree_path_free (path);
        path = NULL;

        return TRUE;
    }

    return FALSE;
}

gboolean
ot_grid_select_last_row (OtGrid *grid)
{
    GtkTreePath *path = ot_grid_dup_last_row_path (grid);

    if (path != NULL)
    {
        ot_grid_select_row_with_scroll (grid, path);
        gtk_tree_path_free (path);
        path = NULL;

        return TRUE;
    }

    return FALSE;
}

gboolean
ot_grid_select_prev_row (OtGrid *grid)
{
    GtkTreeView         *tview  = GTK_TREE_VIEW (grid);
    GtkTreeModel        *model  = gtk_tree_view_get_model (tview);
    GtkTreeSelection    *sel    = gtk_tree_view_get_selection (tview);
    GtkTreePath         *path   = NULL;
    GtkTreeIter          iter;
    gboolean             is_prv = FALSE;

    if (!gtk_tree_selection_get_selected (sel, &model, &iter))
    {
        return ot_grid_select_first_row (grid);
    }

    path    = gtk_tree_model_get_path (model, &iter);
    is_prv  = gtk_tree_path_prev (path);

    if (is_prv)
    {
        ot_grid_select_row_with_scroll (grid, path);
    }

    gtk_tree_path_free (path);
    path = NULL;

    return is_prv;
}

gboolean
ot_grid_select_next_row (OtGrid *grid)
{
    GtkTreeView         *tview  = GTK_TREE_VIEW (grid);
    GtkTreeModel        *model  = gtk_tree_view_get_model (tview);
    GtkTreeSelection    *sel    = gtk_tree_view_get_selection (tview);
    GtkTreePath         *path   = NULL;
    GtkTreeIter          iter;

    if (!gtk_tree_selection_get_selected (sel, &model, &iter))
    {
        return ot_grid_select_first_row (grid);
    }

    if (gtk_tree_model_iter_next (model, &iter))
    {
        path = gtk_tree_model_get_path (model, &iter);
        ot_grid_select_row_with_scroll (grid, path);
        gtk_tree_path_free (path);
        path = NULL;

        return TRUE;
    }

    return FALSE;
}
