/* 파일 설명 적는곳

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#ifndef OTGRID_H_
#define OTGRID_H_

#include <gtk/gtktreeview.h>

#include "otgridrenderertext.h"

G_BEGIN_DECLS

#define OT_TYPE_GRID            (ot_grid_get_type ())
#define OT_GRID(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), OT_TYPE_GRID, OtGrid))
#define OT_GRID_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), OT_TYPE_GRID, OtGridClass))
#define OT_IS_GRID(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), OT_TYPE_GRID))
#define OT_IS_GRID_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), OT_TYPE_GRID))
#define OT_GRID_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), OT_TYPE_GRID, OtGridClass))


typedef struct _OtGrid      OtGrid;
typedef struct _OtGridClass OtGridClass;

struct _OtGrid
{
    GtkTreeView treeview;

    gint                 cursor_row_index;
    OtGridRendererText  *cursor_renderer;
};

struct _OtGridClass
{
    GtkTreeViewClass parent_class;

    gboolean (* changed) (OtGrid *grid, OtGridRendererText *renderer,
            GtkTreeIter *iter, const gchar *new_text);
};

GType               ot_grid_get_type                (void);
OtGrid*             ot_grid_new                     (void);
OtGridRendererText* ot_grid_get_cursor_renderer     (OtGrid *grid);
void                ot_grid_construct_columns       (OtGrid          *grid,
                                                     gint             num_cols,
                                                     GType           *types,
                                                     const gchar    **column_titles);
void                ot_grid_append_row_from_array   (OtGrid     *grid,
                                                     GPtrArray  *array);
void                ot_grid_delete_row              (OtGrid         *grid,
                                                     GtkTreeIter    *iter);
void                ot_grid_set_row_from_array      (OtGrid         *grid,
                                                     GtkTreeIter    *iter,
                                                     GPtrArray      *array);
void                ot_grid_set_resizable           (OtGrid     *grid,
                                                     gboolean    resizable);
void                ot_grid_set_clickable           (OtGrid     *grid,
                                                     gboolean    clickable);
void                ot_grid_set_editable            (OtGrid     *grid,
                                                     gboolean    editable);
void                ot_grid_set_start_editing_aft   (OtGrid     *grid,
                                                     gpointer    start_editing_aft);
void                ot_grid_clear_datas             (OtGrid *grid);
void                ot_grid_unselect_all            (OtGrid *grid);

GtkTreePath*        ot_grid_dup_first_row_path      (OtGrid *grid);
GtkTreePath*        ot_grid_dup_last_row_path       (OtGrid *grid);
void                ot_grid_select_row_with_scroll  (OtGrid         *grid,
                                                     GtkTreePath    *path);
gboolean            ot_grid_select_first_row        (OtGrid *grid);
gboolean            ot_grid_select_last_row         (OtGrid *grid);
gboolean            ot_grid_select_prev_row         (OtGrid *grid);
gboolean            ot_grid_select_next_row         (OtGrid *grid);

G_END_DECLS

#endif /*OTGRID_H_*/
