/* 파일 설명 적는곳

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#include <gtk/gtk.h>

#include "otgridrenderertext.h"

static GtkCellRendererTextClass *ot_grid_renderer_text_parent_class = NULL;



/*
 * signal
 */



/*
 * private
 */

static void
ot_grid_renderer_text_render (GtkCellRenderer       *cell,
                              GdkWindow             *window,
                              GtkWidget             *widget,
                              GdkRectangle          *background_area,
                              GdkRectangle          *cell_area,
                              GdkRectangle          *expose_area,
                              GtkCellRendererState   flags)
{
    GdkGC           *gc     = gdk_gc_new (window);
    GtkStateType     state  = GTK_STATE_NORMAL;

    if (flags & GTK_CELL_RENDERER_SELECTED)
    {
        // state = GTK_STATE_SELECTED;
    }

    gdk_gc_set_rgb_fg_color (gc, & (widget->style->bg[state]));

    gdk_draw_rectangle (window, gc, FALSE, background_area->x - 1, background_area->y,
            background_area->width, background_area->height);

    g_object_unref (G_OBJECT (gc));

    (* GTK_CELL_RENDERER_CLASS (ot_grid_renderer_text_parent_class)->render) (
            cell, window, widget, background_area, cell_area, expose_area, flags);
}

static GtkCellEditable*
ot_grid_renderer_text_start_editing (GtkCellRenderer        *cell,
                                     GdkEvent               *event,
                                     GtkWidget              *widget,
                                     const gchar            *path,
                                     GdkRectangle           *background_area,
                                     GdkRectangle           *cell_area,
                                     GtkCellRendererState    flags)
{
    OtGridRendererText  *renderer   = OT_GRID_RENDERER_TEXT (cell);
    GtkCellEditable     *editable   = NULL;

    editable = (* GTK_CELL_RENDERER_CLASS (ot_grid_renderer_text_parent_class)->start_editing) (
                cell, event, widget, path, background_area, cell_area, flags);

    if (renderer->start_editing_aft != NULL)
    {
        renderer->start_editing_aft (cell, event, widget, path, background_area,
                cell_area, flags, editable);
    }

    return editable;
}

/* class init */
static void
ot_grid_renderer_text_class_init (OtGridRendererTextClass *klass)
{
    GtkCellRendererClass *renderer_class = GTK_CELL_RENDERER_CLASS (klass);

    ot_grid_renderer_text_parent_class = g_type_class_peek_parent (klass);

    renderer_class->render = ot_grid_renderer_text_render;
    renderer_class->start_editing = ot_grid_renderer_text_start_editing;
}

/* instance init */
static void
ot_grid_renderer_text_init (OtGridRendererText *renderer)
{
    renderer->column_index      = -1;
    renderer->renderer_index    = -1;
}


/* get type */
GType
ot_grid_renderer_text_get_type (void)
{
    static GType widget_type = 0;

    if (G_UNLIKELY (widget_type == 0))
    {
        const GTypeInfo widget_info =
        {
            sizeof (OtGridRendererTextClass),
            NULL,       /* base_init */
            NULL,
            (GClassInitFunc) ot_grid_renderer_text_class_init,
            NULL,       /* class_finalize */
            NULL,       /* class_data */
            sizeof (OtGridRendererText),
            0,          /* n_preallocs */
            (GInstanceInitFunc) ot_grid_renderer_text_init,
            NULL,       /* value_table */
        };
        widget_type = g_type_register_static (GTK_TYPE_CELL_RENDERER_TEXT, "OtGridRendererText",
                &widget_info, 0);
    }

    return widget_type;
}



/*
 * public
 */

OtGridRendererText*
ot_grid_renderer_text_new (void)
{
    OtGridRendererText *renderer = OT_GRID_RENDERER_TEXT (g_object_new (OT_TYPE_GRID_RENDERER_TEXT, NULL));

    return renderer;
}

void
ot_grid_renderer_text_set_column_index (OtGridRendererText  *renderer,
                                        gint                 column_index)
{
    renderer->column_index = column_index;
}

void
ot_grid_renderer_text_set_renderer_index (OtGridRendererText    *renderer,
                                          gint                   renderer_index)
{
    renderer->renderer_index = renderer_index;
}

void
ot_grid_renderer_text_set_start_editing_aft (OtGridRendererText *renderer,
                                             gpointer            start_editing_aft)
{
    renderer->start_editing_aft = start_editing_aft;
}
