/* 파일 설명 적는곳

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#include <gtk/gtk.h>

#include "otnotebookpage.h"

static GtkTableClass *ot_notebook_page_parent_class = NULL;



/*
 * signal
 */
static void
ot_notebook_page_destroy_button_clicked_cb (GtkButton   *button,
                                            gpointer     user_data)
{
    ot_notebook_page_remove (OT_NOTEBOOK_PAGE (user_data));
}



/*
 * private
 */

static void
ot_notebook_page_destroy (GtkObject *object)
{
    OtNotebookPage *notebook_page = OT_NOTEBOOK_PAGE (object);

    /* 사용자가 지정한 destroy callback 을 먼저 실행한다. 이때까진 모든 정보가 살아있다. */
    if (notebook_page->destroy_callback != NULL)
    {
        notebook_page->destroy_callback (notebook_page);
        notebook_page->destroy_callback = NULL;
    }

    if (notebook_page->tab_image != NULL)
    {
        gtk_widget_destroy (notebook_page->tab_image);
        notebook_page->tab_image = NULL;
    }

    if (notebook_page->tab_label != NULL)
    {
        gtk_widget_destroy (notebook_page->tab_label);
        notebook_page->tab_label = NULL;
    }

    if (notebook_page->tab_destroy_button != NULL)
    {
        gtk_widget_destroy (notebook_page->tab_destroy_button);
        notebook_page->tab_destroy_button = NULL;
    }

    if (notebook_page->child != NULL)
    {
        gtk_widget_destroy (notebook_page->child);
        notebook_page->child = NULL;
    }

    notebook_page->notebook = NULL;
    notebook_page->ref_data = NULL;

    (* GTK_OBJECT_CLASS (ot_notebook_page_parent_class)->destroy) (object);
}

/* class init */
static void
ot_notebook_page_class_init (OtNotebookPageClass *klass)
{

    GtkObjectClass *object_class = GTK_OBJECT_CLASS (klass);

    ot_notebook_page_parent_class = g_type_class_peek_parent (klass);

    // 이곳에 연결하면 gtk_widget_destroy시 함께 실행된다.
    object_class->destroy = ot_notebook_page_destroy;
}

/* instance init */
static void
ot_notebook_page_init (OtNotebookPage *notebook_page)
{
    notebook_page->tab_image            = gtk_image_new ();
    notebook_page->tab_label            = gtk_label_new (NULL);
    notebook_page->tab_destroy_button   = gtk_button_new ();
    notebook_page->child                = NULL;
    notebook_page->destroy_callback     = NULL;
    notebook_page->ref_data             = NULL;

    GtkWidget *image = gtk_image_new_from_stock (GTK_STOCK_CLOSE,
            GTK_ICON_SIZE_MENU);
    gtk_container_add (GTK_CONTAINER (notebook_page->tab_destroy_button), image);
    gtk_button_set_relief (GTK_BUTTON (notebook_page->tab_destroy_button),
            GTK_RELIEF_NONE);

    gtk_table_set_col_spacings (GTK_TABLE (notebook_page), 5);
    gtk_table_resize (GTK_TABLE (notebook_page), 1, 3);

    gtk_table_attach (GTK_TABLE (notebook_page),
            notebook_page->tab_image,
            0, 1, 0, 1,
            0, 0, 0, 0);
    gtk_table_attach (GTK_TABLE (notebook_page),
            notebook_page->tab_label,
            1, 2, 0, 1,
            0, 0, 0, 0);
    gtk_table_attach (GTK_TABLE (notebook_page),
            notebook_page->tab_destroy_button,
            2, 3, 0, 1,
            0, 0, 0, 0);

    gtk_widget_show_all (GTK_WIDGET (notebook_page));

    g_signal_connect ((gpointer) notebook_page->tab_destroy_button, "clicked",
            G_CALLBACK (ot_notebook_page_destroy_button_clicked_cb),
            (gpointer) notebook_page);
}


/* get type */
GType
ot_notebook_page_get_type (void)
{
    static GType widget_type = 0;

    if (G_UNLIKELY (widget_type == 0))
    {
        const GTypeInfo widget_info =
        {
            sizeof (OtNotebookPageClass),
            NULL,       /* base_init */
            NULL,
            (GClassInitFunc) ot_notebook_page_class_init,
            NULL,       /* class_finalize */
            NULL,       /* class_data */
            sizeof (OtNotebookPage),
            0,          /* n_preallocs */
            (GInstanceInitFunc) ot_notebook_page_init,
            NULL,       /* value_table */
        };
        widget_type = g_type_register_static (GTK_TYPE_TABLE, "OtNotebookPage",
                &widget_info, 0);
    }

    return widget_type;
}



/*
 * public
 */

OtNotebookPage*
ot_notebook_page_new (GtkNotebook   *notebook,
                      GtkWidget     *child,
                      gboolean       show_destroy_button,
                      gpointer       ref_data)
{
    OtNotebookPage *notebook_page = OT_NOTEBOOK_PAGE (g_object_new (OT_TYPE_NOTEBOOK_PAGE, NULL));

    notebook_page->notebook = GTK_WIDGET (notebook);
    notebook_page->child = child;

    gtk_notebook_append_page (notebook, child, GTK_WIDGET (notebook_page));
    gtk_widget_show (child);

    if (show_destroy_button)
    {
        gtk_widget_show (notebook_page->tab_destroy_button);
    }
    else
    {
        gtk_widget_hide (notebook_page->tab_destroy_button);
    }

    notebook_page->ref_data = ref_data;

    return notebook_page;
}

void
ot_notebook_page_remove (OtNotebookPage *notebook_page)
{
    gint page_num = ot_notebook_page_get_page_num (notebook_page);
    gtk_notebook_remove_page (GTK_NOTEBOOK (notebook_page->notebook), page_num);
}

gint
ot_notebook_page_get_page_num (OtNotebookPage *notebook_page)
{
    return gtk_notebook_page_num (GTK_NOTEBOOK (notebook_page->notebook),
            notebook_page->child);
}

void
ot_notebook_page_set_tab_image_from_stock (OtNotebookPage   *notebook_page,
                                           const gchar      *stock_id)
{
    gtk_image_set_from_stock (GTK_IMAGE (notebook_page->tab_image),
            stock_id, GTK_ICON_SIZE_MENU);
    gtk_widget_show_all (notebook_page->tab_image);
}

void
ot_notebook_page_set_tab_image_from_pixbuf (OtNotebookPage  *notebook_page,
                                            const gchar     *filename)
{
    GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file (filename, NULL);

    gtk_image_set_from_pixbuf (GTK_IMAGE (notebook_page->tab_image), pixbuf);
    gtk_widget_show (notebook_page->tab_image);

    g_object_unref (pixbuf);
    pixbuf = NULL;
}

void
ot_notebook_page_set_tab_label (OtNotebookPage  *notebook_page,
                                const gchar     *label_text)
{
    gtk_label_set_text (GTK_LABEL (notebook_page->tab_label), label_text);
}
