/* 파일 설명 적는곳

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#include <glib-object.h>

#include "odmconninfo.h"


static gpointer parent_class = NULL;



static void
odm_conn_info_dispose (GObject *gobject)
{
    OdmConnInfo *conn_info = ODM_CONN_INFO (gobject);

    if (conn_info->name != NULL)
    {
        g_free (conn_info->name);
        conn_info->name = NULL;
    }

    conn_info->ref_data = NULL;

    G_OBJECT_CLASS (parent_class)->dispose (gobject);
}

static void
odm_conn_info_base_class_init (OdmConnInfoClass *class)
{
}

static void
odm_conn_info_base_class_finalize (OdmConnInfoClass *class)
{
}

static void
odm_conn_info_class_init (OdmConnInfoClass *class)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (class);

    parent_class = g_type_class_ref (G_TYPE_OBJECT);

    gobject_class->dispose = odm_conn_info_dispose;

    class->dup_url_for_ot_db    = NULL;
    class->new_ot_db_connection = NULL;
}

static void
odm_conn_info_init (OdmConnInfo         *conn_info,
                    OdmConnInfoClass    *class)
{
    conn_info->name     = NULL;
    conn_info->ref_data = NULL;
}

GType
odm_conn_info_get_type (void)
{
    static GType object_type = 0;

    if (!object_type)
    {
        const GTypeInfo object_info =
        {
            sizeof (OdmConnInfoClass),
            (GBaseInitFunc) odm_conn_info_base_class_init,
            (GBaseFinalizeFunc) odm_conn_info_base_class_finalize,
            (GClassInitFunc) odm_conn_info_class_init,
            NULL,       /* class_finalize */
            NULL,       /* class_data */
            sizeof (OdmConnInfo),
            16,     /* n_preallocs */
            (GInstanceInitFunc) odm_conn_info_init,
            NULL,       /* value_table */
        };

        object_type = g_type_register_static (G_TYPE_INITIALLY_UNOWNED, "OdmConnInfo", 
                &object_info, G_TYPE_FLAG_ABSTRACT);
    }

    return object_type;
}



/*
 * public
 */

void
odm_conn_info_set_ref_data (OdmConnInfo *conn_info,
                            gpointer     ref_data)
{
    conn_info->ref_data = ref_data;
}

gchar*
odm_conn_info_dup_url_for_ot_db (OdmConnInfo *conn_info)
{
    OdmConnInfoClass *class = ODM_CONN_INFO_GET_CLASS (conn_info);

    return class->dup_url_for_ot_db (conn_info);
}

OtDbConnection*
odm_conn_info_new_ot_db_connection (OdmConnInfo *conn_info)
{
    OdmConnInfoClass *class = ODM_CONN_INFO_GET_CLASS (conn_info);

    return class->new_ot_db_connection (conn_info);
}
