/* 파일 설명 적는곳

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#include <string.h>
#include <stdlib.h>

#include "../otak-lib/otak-db/otdbconnectioncub.h"
#include "odmconninfocub.h"


static OdmConnInfoClass *odm_conn_info_cub_parent_class = NULL;



static void
odm_conn_info_cub_dispose (GObject *gobject)
{
    OdmConnInfoCub *conn_info_cub = ODM_CONN_INFO_CUB (gobject);

    if (conn_info_cub->host != NULL)
    {
        g_free (conn_info_cub->host);
        conn_info_cub->host = NULL;
    }

    if (conn_info_cub->port != NULL)
    {
        g_free (conn_info_cub->port);
        conn_info_cub->port = NULL;
    }

    if (conn_info_cub->database != NULL)
    {
        g_free (conn_info_cub->database);
        conn_info_cub->database = NULL;
    }

    if (conn_info_cub->username != NULL)
    {
        g_free (conn_info_cub->username);
        conn_info_cub->username = NULL;
    }

    if (conn_info_cub->password != NULL)
    {
        g_free (conn_info_cub->password);
        conn_info_cub->password = NULL;
    }

    if (conn_info_cub->encoding != NULL)
    {
        g_free (conn_info_cub->encoding);
        conn_info_cub->encoding = NULL;
    }

    G_OBJECT_CLASS (odm_conn_info_cub_parent_class)->dispose (gobject);
}

static gchar*
odm_conn_info_cub_dup_url_for_ot_db (OdmConnInfo *conn_info)
{
    OdmConnInfoCub *conn_info_cub = ODM_CONN_INFO_CUB (conn_info);

    return g_strdup_printf ("cubrid://%s:%s:%s",
            conn_info_cub->host, conn_info_cub->port, conn_info_cub->database);
}

static OtDbConnection*
odm_conn_info_cub_new_ot_db_connection (OdmConnInfo *conn_info)
{
    OdmConnInfoCub  *conn_info_cub  = ODM_CONN_INFO_CUB (conn_info);
    gchar           *url            = odm_conn_info_cub_dup_url_for_ot_db (conn_info);
    OtDbConnection  *ot_db_conn     = NULL;

    ot_db_conn = ot_db_connection_cub_new (url,
                conn_info_cub->username, conn_info_cub->password,
                conn_info_cub->encoding);

    g_free (url);
    url = NULL;

    return ot_db_conn;
}

static void
odm_conn_info_cub_class_init (OdmConnInfoCubClass *class)
{
    GObjectClass        *gobject_class  = G_OBJECT_CLASS (class);
    OdmConnInfoClass    *cclass         = ODM_CONN_INFO_CLASS (class);

    odm_conn_info_cub_parent_class = g_type_class_peek_parent (class);

    gobject_class->dispose = odm_conn_info_cub_dispose;

    cclass->dup_url_for_ot_db       = odm_conn_info_cub_dup_url_for_ot_db;
    cclass->new_ot_db_connection    = odm_conn_info_cub_new_ot_db_connection;
}

static void
odm_conn_info_cub_init (OdmConnInfoCub *conn_info)
{
    conn_info->host      = NULL;
    conn_info->port      = NULL;
    conn_info->database  = NULL;
    conn_info->username  = NULL;
    conn_info->password  = NULL;
    conn_info->encoding  = NULL;
}

GType
odm_conn_info_cub_get_type (void)
{
    static GType widget_type = 0;

    if (G_UNLIKELY (widget_type == 0))
    {
        const GTypeInfo widget_info =
        {
            sizeof (OdmConnInfoCubClass),
            NULL,       /* base_init */
            NULL,
            (GClassInitFunc) odm_conn_info_cub_class_init,
            NULL,       /* class_finalize */
            NULL,       /* class_data */
            sizeof (OdmConnInfoCub),
            0,          /* n_preallocs */
            (GInstanceInitFunc) odm_conn_info_cub_init,
            NULL,       /* value_table */
        };
        widget_type = g_type_register_static (ODM_TYPE_CONN_INFO, "OdmConnInfoCub",
                &widget_info, 0);
    }

    return widget_type;
}



/*
 * public
 */

OdmConnInfo*
odm_conn_info_cub_new (const gchar  *name,
                       const gchar  *host,
                       const gchar  *port,
                       const gchar  *database,
                       const gchar  *username,
                       const gchar  *password,
                       const gchar  *encoding)
{
    OdmConnInfoCub *conn_info = ODM_CONN_INFO_CUB (
            g_object_new (ODM_TYPE_CONN_INFO_CUB, NULL));

    conn_info->host     = g_strdup (host);
    conn_info->port     = g_strdup (port);
    conn_info->database = g_strdup (database);
    conn_info->username = g_strdup (username);
    conn_info->password = g_strdup (password);
    conn_info->encoding = g_strdup (encoding);

    ODM_CONN_INFO (conn_info)->name = g_strdup (name);

    return ODM_CONN_INFO (conn_info);
}
