/* 파일 설명 적는곳

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#include <unistd.h>
#include <glib/gstdio.h>
#include <gtk/gtk.h>

#include "../otak-lib/otak-util/otsys.h"
#include "../otak-lib/otak-util/otgtk.h"
#include "../otak-lib/otak-db/otdbcolumncub.h"
#include "../otak-lib/otak-db/otdbutil.h"
#include "odmcolumnbrowser.h"
#include "../../config-path.h"

#define NOT_NULL_YES    "Y"
#define NOT_NULL_NO     ""

#ifndef _OTAK_DBMANAGER_PATH_H_
#define INST_PREFIX "/usr"
#endif


static
const gchar *ACT_BTN_IMAGE_FILENAME[ODM_COLUMN_BROWSER_ACT_BTN_CNT] =
{
    "add_16.png",
    "delete_16.png",
    "print_16.png"
};

static
const gchar *COLUMN_TITLE[ODM_COLUMN_BROWSER_COLUMN_CNT] =
{
    "Name",
    "No",
    "Type",
    "Size",
    "Scale",
    "Not Null",
    "PKey",
    "Unique",
    "Index",
    "Shared",
    "Default",
    "Inheritance",
    "Comment"
};


static GtkTableClass *odm_column_browser_parent_class = NULL;



/*
 * signal
 */



/*
 * private
 */

static void
odm_column_browser_construct_columns (OdmColumnBrowser *browser)
{
    GType types[ODM_COLUMN_BROWSER_COLUMN_CNT];

    gint i = 0;
    for (i = 0; i < ODM_COLUMN_BROWSER_COLUMN_CNT; i++)
    {
        types[i] = G_TYPE_STRING;        
    }

    ot_grid_construct_columns (OT_GRID (browser->grid),
            ODM_COLUMN_BROWSER_COLUMN_CNT, types, COLUMN_TITLE);

    ot_grid_set_resizable (OT_GRID (browser->grid), TRUE);
    ot_grid_set_clickable (OT_GRID (browser->grid), TRUE);
}

static void
odm_column_browser_destroy (GtkObject *object)
{
    OdmColumnBrowser *browser = ODM_COLUMN_BROWSER (object);

    if (browser->grid != NULL)
    {
        browser->grid = OT_GRID (ot_gtk_widget_free (GTK_WIDGET (browser->grid)));
    }

    if (browser->scrolled != NULL)
    {
        browser->scrolled = ot_gtk_widget_free (browser->scrolled);
    }

    gint i = 0;
    for (i = 0; i < ODM_COLUMN_BROWSER_ACT_BTN_CNT; i++)
    {
        if (browser->action_buttons[i] != NULL)
        {
            gtk_widget_destroy (gtk_button_get_image (
                    GTK_BUTTON (browser->action_buttons[i])));
            gtk_widget_destroy (browser->action_buttons[i]);
            browser->action_buttons[i] = NULL;
        }
    }

    if (browser->action_table != NULL)
    {
        gtk_widget_destroy (browser->action_table);
        browser->action_table = NULL;
    }

    if (browser->table_name != NULL)
    {
        g_free (browser->table_name);
        browser->table_name = NULL;
    }

    browser->shared_db_conn = NULL;

    (* GTK_OBJECT_CLASS (odm_column_browser_parent_class)->destroy) (object);
}

/* class init */
static void
odm_column_browser_class_init (OdmColumnBrowserClass *klass)
{

    GtkObjectClass *object_class = GTK_OBJECT_CLASS (klass);

    odm_column_browser_parent_class = g_type_class_peek_parent (klass);

    // 이곳에 연결하면 gtk_widget_destroy시 함께 실행된다.
    object_class->destroy = odm_column_browser_destroy;
}

/* instance init */
static void
odm_column_browser_init (OdmColumnBrowser *browser)
{
    browser->scrolled       = gtk_scrolled_window_new (NULL, NULL);
    browser->grid           = ot_grid_new ();
    browser->action_table   = gtk_table_new (1, 1, FALSE);
    browser->shared_db_conn = NULL;
    browser->table_name     = NULL;

    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (browser->scrolled),
            GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_container_add (GTK_CONTAINER (browser->scrolled),
            GTK_WIDGET (browser->grid));

    gint i = 0;
    for (i = 0; i < ODM_COLUMN_BROWSER_ACT_BTN_CNT; i++)
    {
        GtkWidget   *image  = gtk_image_new ();
        GtkWidget   *button = gtk_button_new ();

        gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
        gtk_button_set_image (GTK_BUTTON (button), image);

        gtk_table_attach (GTK_TABLE (browser->action_table), button,
                i, i + 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);

        browser->action_buttons[i] = button;
    }

    /* column_browser (table) */
    gtk_table_resize (GTK_TABLE (browser), 2, 2);

    gtk_table_attach (GTK_TABLE (browser), browser->action_table,
            0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
    gtk_table_attach (GTK_TABLE (browser), GTK_WIDGET (browser->scrolled),
            0, 2, 1, 2, GTK_EXPAND | GTK_SHRINK | GTK_FILL,
            GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);

    odm_column_browser_construct_columns (browser);

    gtk_widget_show_all (GTK_WIDGET (browser));
}


/* get type */
GType
odm_column_browser_get_type (void)
{
    static GType widget_type = 0;

    if (G_UNLIKELY (widget_type == 0))
    {
        const GTypeInfo widget_info =
        {
            sizeof (OdmColumnBrowserClass),
            NULL,       /* base_init */
            NULL,
            (GClassInitFunc) odm_column_browser_class_init,
            NULL,       /* class_finalize */
            NULL,       /* class_data */
            sizeof (OdmColumnBrowser),
            0,          /* n_preallocs */
            (GInstanceInitFunc) odm_column_browser_init,
            NULL,       /* value_table */
        };
        widget_type = g_type_register_static (GTK_TYPE_TABLE, "OdmColumnBrowser",
                &widget_info, 0);
    }

    return widget_type;
}



/*
 * public
 */

OdmColumnBrowser*
odm_column_browser_new (void)
{
    OdmColumnBrowser *browser = ODM_COLUMN_BROWSER (g_object_new (ODM_TYPE_COLUMN_BROWSER, NULL));

    return browser;
}

void
odm_column_browser_dupset_db (OdmColumnBrowser  *browser,
                              OtDbConnection    *shared_db_conn,
                              const gchar       *table_name)
{
    browser->table_name = ot_sys_g_free (browser->table_name);

    browser->shared_db_conn = shared_db_conn;
    browser->table_name     = g_strdup (table_name);
}

void
odm_column_browser_show_action_table (OdmColumnBrowser  *browser,
                                      gboolean           show)
{
    if (show)
    {
        gtk_widget_show (browser->action_table);
    }
    else
    {
        gtk_widget_hide (browser->action_table);
    }
}

void
odm_column_browser_button_loading (OdmColumnBrowser *browser,
                                   const gchar      *image_dir)
{
    gint i = 0;
    for (i = 0; i < ODM_COLUMN_BROWSER_ACT_BTN_CNT; i++)
    {
        GtkWidget   *image      = gtk_button_get_image (GTK_BUTTON (browser->action_buttons[i]));
        gchar       *image_path = g_strdup_printf ("%s/%s", image_dir, ACT_BTN_IMAGE_FILENAME[i]);

        if (0 != g_access ((const gchar *)image_path, R_OK))
        {
            image_path = ot_sys_g_free (image_path);
            image_path = g_strdup_printf (INST_PREFIX"/share/otak-dbmanager/%s", ACT_BTN_IMAGE_FILENAME[i]);
            if (0 != g_access ((const gchar *)image_path, R_OK))
            {
                image_path = ot_sys_g_free (image_path);
                image_path = g_strdup_printf ("%s/%s", image_dir, ACT_BTN_IMAGE_FILENAME[i]);
            }
        }

        gtk_image_set_from_file (GTK_IMAGE (image), image_path);

        g_free (image_path);
        image_path = NULL;
    }
}

gboolean
odm_column_browser_refresh (OdmColumnBrowser *browser)
{
    GPtrArray   *columns    = NULL;
    GPtrArray   *arr        = NULL;

    columns = ot_db_connection_dup_column_list (browser->shared_db_conn,
                browser->table_name);

    gint i = 0;
    for (i = 0; i < columns->len; i++)
    {
        OtDbColumn      *column     = g_ptr_array_index (columns, i);
        OtDbColumnCub   *column_cub = OT_DB_COLUMN_CUB (column);

        arr = g_ptr_array_new ();

        g_ptr_array_add (arr, g_strdup (column->name));
        g_ptr_array_add (arr, ot_db_util_dup_str_value (OT_DB_C_TYPE_INT, &column->order));
        g_ptr_array_add (arr, ot_db_column_dup_type_name (column));
        g_ptr_array_add (arr, ot_db_util_dup_str_value (OT_DB_C_TYPE_INT, &column->size));
        g_ptr_array_add (arr, ot_db_util_dup_str_value (OT_DB_C_TYPE_INT, &column->scale));
        g_ptr_array_add (arr, g_strdup (column->not_null ? NOT_NULL_YES : NOT_NULL_NO));
        g_ptr_array_add (arr, ot_db_util_dup_str_value (OT_DB_C_TYPE_INT, &column->pkey_order));
        g_ptr_array_add (arr, g_strdup (column->unique ? NOT_NULL_YES : NOT_NULL_NO));
        g_ptr_array_add (arr, g_strdup (column->index ? NOT_NULL_YES : NOT_NULL_NO));
        g_ptr_array_add (arr, g_strdup (column_cub->shared ? NOT_NULL_YES : NOT_NULL_NO));
        g_ptr_array_add (arr, g_strdup (column->default_value));
        g_ptr_array_add (arr, g_strdup (column_cub->inheritance));
        g_ptr_array_add (arr, g_strdup (column->comment));
        
        ot_grid_append_row_from_array (OT_GRID (browser->grid), arr);

        ot_sys_g_ptr_array_free_with_element (arr, g_free);
    }

    ot_sys_g_ptr_array_free_with_element (columns, g_object_unref);

    return TRUE;
}
