/* 파일 설명 적는곳

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#include <unistd.h>
#include <glib/gstdio.h>
#include <gtk/gtk.h>

#include "../otak-lib/otak-util/otsys.h"
#include "../otak-lib/otak-util/otgtk.h"
#include "odmdatabrowser.h"
#include "../../config-path.h"

#ifndef _OTAK_DBMANAGER_PATH_H_
#define INST_PREFIX "/usr"
#endif

static void odm_data_browser_navi_btn_first_clicked_cb (GtkButton *button, gpointer user_data);
static void odm_data_browser_navi_btn_last_clicked_cb (GtkButton *button, gpointer user_data);
static void odm_data_browser_navi_btn_prev_clicked_cb (GtkButton *button, gpointer user_data);
static void odm_data_browser_navi_btn_next_clicked_cb (GtkButton *button, gpointer user_data);
static void odm_data_browser_navi_btn_delete_clicked_cb (GtkButton *button, gpointer user_data);
static void odm_data_browser_navi_btn_apply_clicked_cb (GtkButton *button, gpointer user_data);
static void odm_data_browser_navi_btn_cancel_clicked_cb (GtkButton *button, gpointer user_data);
static void odm_data_browser_navi_btn_refresh_clicked_cb (GtkButton *button, gpointer user_data);
static void odm_data_browser_edit_btn_set_null_clicked_cb (GtkButton *button, gpointer user_data);


static
const gchar *NAVI_BTN_IMAGE_FILENAME[ODM_DATA_BROWSER_NAVI_BTN_CNT] =
{
    "navi_first_16.png",
    "navi_prev_16.png",
    "navi_next_16.png",
    "navi_last_16.png",
    "navi_add_16.png",
    "navi_delete_16.png",
    "navi_apply_16.png",
    "navi_cancel_16.png",
    "navi_refresh_16.png"
};

static
const gchar *EDIT_BTN_IMAGE_FILENAME[ODM_DATA_BROWSER_EDIT_BTN_CNT] =
{
    "null_value_16.png"
};

static
const GCallback NAVI_BTN_CALLBACK[ODM_DATA_BROWSER_NAVI_BTN_CNT] =
{
    G_CALLBACK (odm_data_browser_navi_btn_first_clicked_cb),
    G_CALLBACK (odm_data_browser_navi_btn_prev_clicked_cb),
    G_CALLBACK (odm_data_browser_navi_btn_next_clicked_cb),
    G_CALLBACK (odm_data_browser_navi_btn_last_clicked_cb),
    NULL,
    G_CALLBACK (odm_data_browser_navi_btn_delete_clicked_cb),
    G_CALLBACK (odm_data_browser_navi_btn_apply_clicked_cb),
    G_CALLBACK (odm_data_browser_navi_btn_cancel_clicked_cb),
    G_CALLBACK (odm_data_browser_navi_btn_refresh_clicked_cb)
};

static
const GCallback EDIT_BTN_CALLBACK[ODM_DATA_BROWSER_EDIT_BTN_CNT] =
{
    G_CALLBACK (odm_data_browser_edit_btn_set_null_clicked_cb)
};

static
const gchar *NAVI_BTN_TOOLTIP[ODM_DATA_BROWSER_NAVI_BTN_CNT] =
{
    "First",
    "Prev",
    "Next",
    "Last",
    "Add",
    "Delete",
    "Apply",
    "Cancel",
    "Refresh"
};

static
const gchar *EDIT_BTN_TOOLTIP[ODM_DATA_BROWSER_EDIT_BTN_CNT] =
{
    "Set Null Value"
};

static
const gint NAVI_BTN_WRITE_IDX[4] =
{
    4, 5, 6, 7
};

static
const gint EDIT_BTN_WRITE_IDX[1] =
{
    0
};


static GtkTableClass *odm_data_browser_parent_class = NULL;



/*
 * signal
 */

static void
odm_data_browser_navi_btn_first_clicked_cb (GtkButton   *button,
                                            gpointer     user_data)
{
    ot_grid_select_first_row (
            OT_GRID (ODM_DATA_BROWSER (user_data)->db_grid));
}

static void
odm_data_browser_navi_btn_last_clicked_cb (GtkButton    *button,
                                           gpointer      user_data)
{
    while (odm_data_browser_next_block (ODM_DATA_BROWSER (user_data)) > 0);

    ot_grid_select_last_row (
            OT_GRID (ODM_DATA_BROWSER (user_data)->db_grid));
}

static void
odm_data_browser_navi_btn_prev_clicked_cb (GtkButton    *button,
                                           gpointer      user_data)
{
    ot_grid_select_prev_row (
            OT_GRID (ODM_DATA_BROWSER (user_data)->db_grid));
}

static void
odm_data_browser_navi_btn_next_clicked_cb (GtkButton    *button,
                                           gpointer      user_data)
{
    if (!ot_grid_select_next_row (
            OT_GRID (ODM_DATA_BROWSER (user_data)->db_grid)))
    {
        if (odm_data_browser_next_block (ODM_DATA_BROWSER (user_data)) > 0)
        {
            ot_grid_select_next_row (
                        OT_GRID (ODM_DATA_BROWSER (user_data)->db_grid));
        }
    }
}

static void
odm_data_browser_navi_btn_delete_clicked_cb (GtkButton  *button,
                                             gpointer    user_data)
{
    ot_db_grid_delete (ODM_DATA_BROWSER (user_data)->db_grid);
}

static void
odm_data_browser_navi_btn_apply_clicked_cb (GtkButton   *button,
                                            gpointer     user_data)
{
    ot_db_grid_apply_changed (ODM_DATA_BROWSER (user_data)->db_grid, NULL);
}

static void
odm_data_browser_navi_btn_cancel_clicked_cb (GtkButton  *button,
                                             gpointer    user_data)
{
    ot_db_grid_cancel_changed (ODM_DATA_BROWSER (user_data)->db_grid);
}

static void
odm_data_browser_navi_btn_refresh_clicked_cb (GtkButton *button,
                                              gpointer   user_data)
{
    odm_data_browser_refresh (ODM_DATA_BROWSER (user_data));
}

static void
odm_data_browser_edit_btn_set_null_clicked_cb (GtkButton    *button,
                                               gpointer      user_data)
{
    ot_db_grid_set_null_value (ODM_DATA_BROWSER (user_data)->db_grid);
}

static void
odm_data_browser_vadjustment_value_changed_cb (GtkAdjustment    *adjustment,
                                               gpointer          user_data)
{
    if (adjustment->page_size + adjustment->value == adjustment->upper)
    {
        odm_data_browser_next_block (ODM_DATA_BROWSER (user_data));
    }
}



/*
 * private
 */

static void
odm_data_browser_buttons_loading (OdmDataBrowser     *browser,
                                  const gchar        *image_dir,
                                  GtkWidget         **buttons,
                                  const gchar       **filenames,
                                  gint                count)
{
    gint i = 0;
    for (i = 0; i < count; i++)
    {
        GtkWidget   *image      = NULL;
        gchar       *image_path = NULL;

        image       = gtk_button_get_image (GTK_BUTTON (buttons[i]));
        image_path  = g_strdup_printf ("%s/%s", image_dir, filenames[i]);

        if (0 != g_access ((const gchar *)image_path, R_OK))
        {
            image_path = ot_sys_g_free (image_path);
            image_path = g_strdup_printf (INST_PREFIX"/share/otak-dbmanager/%s", filenames[i]);
            if (0 != g_access ((const gchar *)image_path, R_OK))
            {
                image_path = ot_sys_g_free (image_path);
                image_path = g_strdup_printf ("%s/%s", image_dir, filenames[i]);
            }
        }

        gtk_image_set_from_file (GTK_IMAGE (image), image_path);

        g_free (image_path);
        image_path = NULL;
    }
}

static void
odm_data_browser_buttons_set_read_only (OdmDataBrowser   *browser,
                                        GtkWidget       **buttons,
                                        const gint       *write_idxs,
                                        gint              count,
                                        gboolean          readonly)
{
    gint i = 0;
    for (i = 0; i < count; i++)
    {
        gint idx = write_idxs[i];

        gtk_widget_set_sensitive (buttons[idx], !readonly);
    }
}

static void
odm_data_browser_destroy (GtkObject *object)
{
    OdmDataBrowser *browser = ODM_DATA_BROWSER (object);

    if (browser->db_grid != NULL)
    {
        browser->db_grid = OT_DB_GRID (ot_gtk_widget_free (GTK_WIDGET (browser->db_grid)));
    }

    if (browser->scrolled != NULL)
    {
        browser->scrolled = ot_gtk_widget_free (browser->scrolled);
    }

    if (browser->statusbar != NULL)
    {
        gtk_widget_destroy (browser->statusbar);
        browser->statusbar = NULL;
    }

    if (browser->action_table != NULL)
    {
        gtk_widget_destroy (browser->action_table);
        browser->action_table = NULL;
    }

    gint i = 0;
    for (i = 0; i < ODM_DATA_BROWSER_NAVI_BTN_CNT; i++)
    {
        if (browser->navi_buttons[i] != NULL)
        {
            gtk_widget_destroy (gtk_button_get_image (
                    GTK_BUTTON (browser->navi_buttons[i])));
            gtk_widget_destroy (browser->navi_buttons[i]);
            browser->navi_buttons[i] = NULL;
        }
    }

    if (browser->navi_table != NULL)
    {
        gtk_widget_destroy (browser->navi_table);
        browser->navi_table = NULL;
    }

    for (i = 0; i < ODM_DATA_BROWSER_EDIT_BTN_CNT; i++)
    {
        if (browser->edit_buttons[i] != NULL)
        {
            gtk_widget_destroy (gtk_button_get_image (
                    GTK_BUTTON (browser->edit_buttons[i])));
            gtk_widget_destroy (browser->edit_buttons[i]);
            browser->edit_buttons[i] = NULL;
        }
    }

    if (browser->edit_table != NULL)
    {
        gtk_widget_destroy (browser->edit_table);
        browser->edit_table = NULL;
    }

    (* GTK_OBJECT_CLASS (odm_data_browser_parent_class)->destroy) (object);
}

/* class init */
static void
odm_data_browser_class_init (OdmDataBrowserClass *klass)
{

    GtkObjectClass *object_class = GTK_OBJECT_CLASS (klass);

    odm_data_browser_parent_class = g_type_class_peek_parent (klass);

    // 이곳에 연결하면 gtk_widget_destroy시 함께 실행된다.
    object_class->destroy = odm_data_browser_destroy;
}

static void
odm_data_browser_buttons_init (OdmDataBrowser    *browser,
                               GtkWidget         *table,
                               GtkWidget        **buttons,
                               const GCallback   *callbacks,
                               const gchar      **tooltips,
                               gint               count)
{
    gint i = 0;
    for (i = 0; i < count; i++)
    {
        GtkWidget   *image  = gtk_image_new ();
        GtkWidget   *button = gtk_button_new ();

        gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
        gtk_button_set_image (GTK_BUTTON (button), image);

        gtk_table_attach (GTK_TABLE (table), button, i, i + 1, 0, 1,
                GTK_FILL, GTK_FILL, 0, 0);

        if (tooltips != NULL)
        {
            if (tooltips[i] != NULL)
            {
                gtk_widget_set_tooltip_text (button, tooltips[i]);
            }
        }

        if (callbacks != NULL)
        {
            if (callbacks[i] != NULL)
            {
                g_signal_connect ((gpointer) button, "clicked", callbacks[i],
                        (gpointer) browser);
            }
        }

        buttons[i] = button;
    }
}

static void
odm_data_browser_scrolled_init (OdmDataBrowser *browser)
{
    GtkScrolledWindow   *scrl   = GTK_SCROLLED_WINDOW (browser->scrolled);
    GtkAdjustment       *v_adj  = NULL;

    v_adj = gtk_scrolled_window_get_vadjustment (scrl);

    g_signal_connect ((gpointer) v_adj, "value-changed",
            G_CALLBACK (odm_data_browser_vadjustment_value_changed_cb),
            (gpointer) browser);
}

/* instance init */
static void
odm_data_browser_init (OdmDataBrowser *browser)
{
    browser->scrolled       = gtk_scrolled_window_new (NULL, NULL);
    browser->db_grid        = ot_db_grid_new ();
    browser->statusbar      = gtk_statusbar_new ();
    browser->action_table   = gtk_table_new (1, 1, FALSE);
    browser->navi_table     = gtk_table_new (1, ODM_DATA_BROWSER_NAVI_BTN_CNT, FALSE);
    browser->edit_table     = gtk_table_new (1, ODM_DATA_BROWSER_EDIT_BTN_CNT, FALSE);

    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (browser->scrolled),
            GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_container_add (GTK_CONTAINER (browser->scrolled),
            GTK_WIDGET (browser->db_grid));

    odm_data_browser_scrolled_init (browser);

    odm_data_browser_buttons_init (browser, browser->navi_table,
            browser->navi_buttons, NAVI_BTN_CALLBACK, NAVI_BTN_TOOLTIP,
            ODM_DATA_BROWSER_NAVI_BTN_CNT);
    odm_data_browser_buttons_init (browser, browser->edit_table,
            browser->edit_buttons, EDIT_BTN_CALLBACK, EDIT_BTN_TOOLTIP,
            ODM_DATA_BROWSER_EDIT_BTN_CNT);

    /* statusbar */
    gtk_widget_set_size_request (browser->statusbar, -1, 20);

    /* data_browser (table) */
    gtk_table_resize (GTK_TABLE (browser), 4, 3);

    gtk_table_attach (GTK_TABLE (browser), browser->action_table,
            0, 1, 0, 1, GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
    gtk_table_attach (GTK_TABLE (browser), browser->navi_table,
            1, 2, 0, 1, GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
    gtk_table_attach (GTK_TABLE (browser), browser->edit_table,
            2, 3, 0, 1, GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
    gtk_table_attach (GTK_TABLE (browser), GTK_WIDGET (browser->scrolled),
            0, 4, 1, 2, GTK_EXPAND | GTK_SHRINK | GTK_FILL,
            GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
    gtk_table_attach (GTK_TABLE (browser), browser->statusbar,
            0, 4, 2, 3,
            GTK_EXPAND | GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 2);

    gtk_widget_show_all (GTK_WIDGET (browser));
}


/* get type */
GType
odm_data_browser_get_type (void)
{
    static GType widget_type = 0;

    if (G_UNLIKELY (widget_type == 0))
    {
        const GTypeInfo widget_info =
        {
            sizeof (OdmDataBrowserClass),
            NULL,       /* base_init */
            NULL,
            (GClassInitFunc) odm_data_browser_class_init,
            NULL,       /* class_finalize */
            NULL,       /* class_data */
            sizeof (OdmDataBrowser),
            0,          /* n_preallocs */
            (GInstanceInitFunc) odm_data_browser_init,
            NULL,       /* value_table */
        };
        widget_type = g_type_register_static (GTK_TYPE_TABLE, "OdmDataBrowser",
                &widget_info, 0);
    }

    return widget_type;
}



/*
 * public
 */

OdmDataBrowser*
odm_data_browser_new (void)
{
    OdmDataBrowser *browser = ODM_DATA_BROWSER (g_object_new (ODM_TYPE_DATA_BROWSER, NULL));

    return browser;
}

OtDbGrid*
odm_data_browser_get_db_grid (OdmDataBrowser *browser)
{
    return browser->db_grid;
}

void
odm_data_browser_show_action_table (OdmDataBrowser  *browser,
                                    gboolean         show)
{
    if (show)
    {
        gtk_widget_show (browser->action_table);
    }
    else
    {
        gtk_widget_hide (browser->action_table);
    }
}

void
odm_data_browser_button_loading (OdmDataBrowser *browser,
                                 const gchar    *image_dir)
{
    odm_data_browser_buttons_loading (browser, image_dir,
            browser->navi_buttons, NAVI_BTN_IMAGE_FILENAME,
            ODM_DATA_BROWSER_NAVI_BTN_CNT);
    odm_data_browser_buttons_loading (browser, image_dir,
            browser->edit_buttons, EDIT_BTN_IMAGE_FILENAME,
            ODM_DATA_BROWSER_EDIT_BTN_CNT);
}

gint
odm_data_browser_next_block (OdmDataBrowser *browser)
{
    return ot_db_grid_next_block (browser->db_grid);
}

gboolean
odm_data_browser_refresh (OdmDataBrowser *browser)
{
    GtkAdjustment *v_adj = NULL;

    v_adj = gtk_scrolled_window_get_vadjustment (
                GTK_SCROLLED_WINDOW (browser->scrolled));
    gtk_adjustment_set_value (v_adj, 0.0);

    return ot_db_grid_refresh (browser->db_grid);
}

void
odm_data_browser_set_readonly (OdmDataBrowser   *browser,
                               gboolean          readonly)
{
    ot_db_grid_set_editable (browser->db_grid, !readonly);

    odm_data_browser_buttons_set_read_only (browser, browser->navi_buttons,
            NAVI_BTN_WRITE_IDX, G_N_ELEMENTS (NAVI_BTN_WRITE_IDX), readonly);
    odm_data_browser_buttons_set_read_only (browser, browser->edit_buttons,
            EDIT_BTN_WRITE_IDX, G_N_ELEMENTS (EDIT_BTN_WRITE_IDX), readonly);
}
