/* 파일 설명 적는곳

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#include <gtk/gtk.h>

#include "../otak-lib/otak-util/otgtk.h"
#include "../otak-lib/otak-widget/otnotebookpage.h"
#include "odmcolumnbrowser.h"
#include "odmschemabrowser.h"


static GtkNotebookClass *odm_schema_browser_parent_class = NULL;



/*
 * signal
 */



/*
 * private
 */

static void
odm_schema_browser_destroy (GtkObject *object)
{
    OdmSchemaBrowser *browser = ODM_SCHEMA_BROWSER (object);

    gint    i       = 0;
    gint    count   = gtk_notebook_get_n_pages (GTK_NOTEBOOK (browser));
    for (i = count - 1; i >= 0; i--)
    {
        gtk_notebook_remove_page (GTK_NOTEBOOK (browser), i);
    }

    browser->column_browser = NULL;
    browser->data_browser   = NULL;

    if (browser->err_msg != NULL)
    {
        g_free (browser->err_msg);
        browser->err_msg = NULL;
    }

    (* GTK_OBJECT_CLASS (odm_schema_browser_parent_class)->destroy) (object);
}

/* class init */
static void
odm_schema_browser_class_init (OdmSchemaBrowserClass *klass)
{

    GtkObjectClass *object_class = GTK_OBJECT_CLASS (klass);

    odm_schema_browser_parent_class = g_type_class_peek_parent (klass);

    // 이곳에 연결하면 gtk_widget_destroy시 함께 실행된다.
    object_class->destroy = odm_schema_browser_destroy;
}

/* instance init */
static void
odm_schema_browser_init (OdmSchemaBrowser *browser)
{
    OtNotebookPage *page = NULL;

    browser->column_browser = odm_column_browser_new ();
    browser->data_browser   = odm_data_browser_new ();
    browser->err_msg        = NULL;

    page = ot_notebook_page_new (GTK_NOTEBOOK (browser),
            GTK_WIDGET (browser->column_browser), FALSE, NULL);
    ot_notebook_page_set_tab_label (page, "Column Browser");
    page = ot_notebook_page_new (GTK_NOTEBOOK (browser),
            GTK_WIDGET (browser->data_browser), FALSE, NULL);
    ot_notebook_page_set_tab_label (page, "Data Browser");
}


/* get type */
GType
odm_schema_browser_get_type (void)
{
    static GType widget_type = 0;

    if (G_UNLIKELY (widget_type == 0))
    {
        const GTypeInfo widget_info =
        {
            sizeof (OdmSchemaBrowserClass),
            NULL,       /* base_init */
            NULL,
            (GClassInitFunc) odm_schema_browser_class_init,
            NULL,       /* class_finalize */
            NULL,       /* class_data */
            sizeof (OdmSchemaBrowser),
            0,          /* n_preallocs */
            (GInstanceInitFunc) odm_schema_browser_init,
            NULL,       /* value_table */
        };
        widget_type = g_type_register_static (GTK_TYPE_NOTEBOOK, "OdmSchemaBrowser",
                &widget_info, 0);
    }

    return widget_type;
}



/*
 * public
 */

OdmSchemaBrowser*
odm_schema_browser_new (OtDbConnection   *shared_db_conn,
                        const gchar      *table_name,
                        const gchar      *image_dir,
                        gchar           **error)
{
    OdmSchemaBrowser    *browser    = ODM_SCHEMA_BROWSER (g_object_new (ODM_TYPE_SCHEMA_BROWSER, NULL));
    OtDbDataset         *dset       = NULL;
    OtDbGrid            *grid       = NULL;
    gchar               *err_msg    = NULL;

    odm_column_browser_dupset_db (browser->column_browser, shared_db_conn,
                table_name);
    odm_column_browser_button_loading (browser->column_browser, image_dir);
    odm_column_browser_refresh (browser->column_browser);

    /*
     * data browser의 db_grid에 dataset관련된 작업을 한다.
     */

    dset = ot_db_connection_new_dataset (shared_db_conn);
    if (dset == NULL)
    {
        err_msg = g_strdup ("Data Browser. 데이타베이스 커넥터 생성 실패.");

        goto label_error;
    }
    ot_db_dataset_set_fetch_size (dset, 64);

    grid = odm_data_browser_get_db_grid (browser->data_browser);
    ot_db_grid_set_dataset (grid, dset);
    ot_db_grid_dupset_table (grid, table_name);

    odm_data_browser_button_loading (browser->data_browser, image_dir);

    if (!odm_data_browser_refresh (browser->data_browser))
    {
        OtDbConnection *conn = dset->connection;

        if (ot_db_connection_is_error (conn))
        {
            err_msg = ot_db_connection_dup_current_err_msg (conn);
        }

        goto label_error;
    }

    return browser;


label_error:

    g_object_unref (dset);

    browser = ODM_SCHEMA_BROWSER (ot_gtk_widget_free (GTK_WIDGET (browser)));

    if (err_msg != NULL)
    {
        if (error != NULL)
        {
            *error = g_strdup (err_msg);
        }

        g_free (err_msg);
        err_msg = NULL;
    }

    return NULL;
}
