/* 파일 설명 적는곳

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#include <unistd.h>
#include <glib/gstdio.h>
#include <gtk/gtk.h>

#include "../otak-lib/otak-util/otsys.h"
#include "../otak-lib/otak-util/otgtk.h"
#include "../odmgbwdg.h"
#include "odmdialog.h"
#include "odmsqleditor.h"
#include "../../config-path.h"

#ifndef _OTAK_DBMANAGER_PATH_H_
#define INST_PREFIX "/usr"
#endif


static void odm_sql_editor_act_btn_exec_clicked_cb (GtkButton *button, gpointer user_data);

static
const gchar *ACT_BTN_IMAGE_FILENAME[ODM_SQL_EDITOR_ACT_BTN_CNT] =
{
    "navi_first_16.png"
};

static
const GCallback ACT_BTN_CALLBACK[ODM_SQL_EDITOR_ACT_BTN_CNT] =
{
    G_CALLBACK (odm_sql_editor_act_btn_exec_clicked_cb)
};


static GtkVPanedClass *odm_sql_editor_parent_class = NULL;



/*
 * signal
 */

static void
odm_sql_editor_size_allocate_cb (GtkWidget      *widget,
                                 GtkAllocation  *allocation,
                                 gpointer        user_data)
{
    OdmSqlEditor *editor = ODM_SQL_EDITOR (widget);

    if (!editor->is_size_allocate)
    {
        gtk_paned_set_position (GTK_PANED (editor), allocation->height - 150);

        editor->is_size_allocate = TRUE;
    }
}

static void
odm_sql_editor_act_btn_exec_clicked_cb (GtkButton   *button,
                                        gpointer     user_data)
{
    OdmSqlEditor    *edt        = ODM_SQL_EDITOR (user_data);
    OtDbGrid        *grid       = NULL;
    gchar           *full_txt   = NULL;

    grid = odm_data_browser_get_db_grid (edt->data_browser);
    full_txt = odm_sql_editor_dup_full_text (edt);

    ot_db_grid_dupset_sql (grid, full_txt);
    if (!ot_db_grid_refresh (grid))
    {
        OtDbDataset *dset   = ot_db_grid_get_dataset (grid);
        gchar       *err    = ot_db_connection_dup_current_err_msg (dset->connection);

        odm_dialog_message_modal (GTK_WINDOW (odm_gb_wdg_get_main_window ()),
                err, GTK_MESSAGE_ERROR);

        err = ot_sys_g_free (err);
    }

    full_txt = ot_sys_g_free (full_txt);
}




/*
 * private
 */

static void
odm_sql_editor_destroy (GtkObject *object)
{
    OdmSqlEditor *editor = ODM_SQL_EDITOR (object);

    
    if (editor->edt_textview != NULL)
    {
        gtk_widget_destroy (editor->edt_textview);
        editor->edt_textview = NULL;
    }

    if (editor->edt_scrolled != NULL)
    {
        editor->edt_scrolled = ot_gtk_widget_free (editor->edt_scrolled);
    }

    gint i = 0;
    for (i = 0; i < ODM_DATA_BROWSER_NAVI_BTN_CNT; i++)
    {
        if (editor->act_buttons[i] != NULL)
        {
            gtk_widget_destroy (gtk_button_get_image (
                    GTK_BUTTON (editor->act_buttons[i])));
            gtk_widget_destroy (editor->act_buttons[i]);
            editor->act_buttons[i] = NULL;
        }
    }

    if (editor->act_table != NULL)
    {
        gtk_widget_destroy (editor->act_table);
        editor->act_table = NULL;
    }

    if (editor->edt_table != NULL)
    {
        gtk_widget_destroy (editor->edt_table);
        editor->edt_table = NULL;
    }

    if (editor->data_browser != NULL)
    {
        ot_gtk_widget_free (GTK_WIDGET (editor->data_browser));
        editor->data_browser = NULL;
    }

    (* GTK_OBJECT_CLASS (odm_sql_editor_parent_class)->destroy) (object);
}

/* class init */
static void
odm_sql_editor_class_init (OdmSqlEditorClass *klass)
{

    GtkObjectClass *object_class = GTK_OBJECT_CLASS (klass);

    odm_sql_editor_parent_class = g_type_class_peek_parent (klass);

    // 이곳에 연결하면 gtk_widget_destroy시 함께 실행된다.
    object_class->destroy = odm_sql_editor_destroy;
}

/* instance init */
static void
odm_sql_editor_init (OdmSqlEditor *editor)
{
    editor->edt_table           = gtk_table_new (2, 1, FALSE);
    editor->edt_scrolled        = gtk_scrolled_window_new (NULL, NULL);
    editor->edt_textview        = gtk_text_view_new();
    editor->data_browser        = odm_data_browser_new ();
    editor->act_table           = gtk_table_new (1, ODM_SQL_EDITOR_ACT_BTN_CNT, FALSE);
    editor->is_size_allocate    = FALSE;

    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (editor->edt_scrolled),
            GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_container_add (GTK_CONTAINER (editor->edt_scrolled),
            GTK_WIDGET (editor->edt_textview));

    gint i = 0;
    for (i = 0; i < ODM_SQL_EDITOR_ACT_BTN_CNT; i++)
    {
        GtkWidget   *image  = gtk_image_new ();
        GtkWidget   *button = gtk_button_new ();

        gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
        gtk_button_set_image (GTK_BUTTON (button), image);

        gtk_table_attach (GTK_TABLE (editor->act_table), button,
                i, i + 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);

        if (ACT_BTN_CALLBACK[i] != NULL)
        {
            g_signal_connect ((gpointer) button, "clicked", ACT_BTN_CALLBACK[i],
                    (gpointer) editor);
        }

        editor->act_buttons[i] = button;
    }

    gtk_table_attach (GTK_TABLE (editor->edt_table), editor->act_table,
            0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
    gtk_table_attach (GTK_TABLE (editor->edt_table), editor->edt_scrolled,
            0, 1, 1, 2, GTK_EXPAND | GTK_SHRINK | GTK_FILL,
            GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);

    /* paned에 기본 위젯을 넣는다 */
    gtk_paned_add1 (GTK_PANED (editor), editor->edt_table);
    gtk_paned_add2 (GTK_PANED (editor), GTK_WIDGET (editor->data_browser));

    g_signal_connect ((gpointer) editor, "size_allocate",
            G_CALLBACK (odm_sql_editor_size_allocate_cb), NULL);

    gtk_widget_show_all (GTK_WIDGET (editor));
}


/* get type */
GType
odm_sql_editor_get_type (void)
{
    static GType widget_type = 0;

    if (G_UNLIKELY (widget_type == 0))
    {
        const GTypeInfo widget_info =
        {
            sizeof (OdmSqlEditorClass),
            NULL,       /* base_init */
            NULL,
            (GClassInitFunc) odm_sql_editor_class_init,
            NULL,       /* class_finalize */
            NULL,       /* class_data */
            sizeof (OdmSqlEditor),
            0,          /* n_preallocs */
            (GInstanceInitFunc) odm_sql_editor_init,
            NULL,       /* value_table */
        };
        widget_type = g_type_register_static (GTK_TYPE_VPANED, "OdmSqlEditor",
                &widget_info, 0);
    }

    return widget_type;
}



/*
 * public
 */

OdmSqlEditor*
odm_sql_editor_new (OtDbConnection   *shared_db_conn,
                    const gchar      *image_dir,
                    gchar           **error)
{
    OdmSqlEditor    *editor = ODM_SQL_EDITOR (g_object_new (ODM_TYPE_SQL_EDITOR, NULL));
    OtDbDataset     *dset   = NULL;
    OtDbGrid        *grid   = NULL;

    dset = ot_db_connection_new_dataset (shared_db_conn);
    if (dset == NULL)
    {
        if (error != NULL)
        {
            *error = g_strdup ("Data Browser. 데이타베이스 커넥터 생성 실패.");
        }

        editor = ODM_SQL_EDITOR (ot_gtk_widget_free (GTK_WIDGET (editor)));
    }
    ot_db_dataset_set_fetch_size (dset, 20);

    grid = odm_data_browser_get_db_grid (editor->data_browser);
    ot_db_grid_set_dataset (grid, dset);

    odm_data_browser_button_loading (editor->data_browser, image_dir);
    odm_data_browser_set_readonly (editor->data_browser, TRUE);

    odm_sql_editor_button_loading (editor, image_dir);

    g_object_unref (dset);

    return editor;
}

void
odm_sql_editor_button_loading (OdmSqlEditor *editor,
                               const gchar  *image_dir)
{
    gint i = 0;
    for (i = 0; i < ODM_SQL_EDITOR_ACT_BTN_CNT; i++)
    {
        GtkWidget   *image      = gtk_button_get_image (GTK_BUTTON (editor->act_buttons[i]));
        gchar       *image_path = g_strdup_printf ("%s/%s", image_dir, ACT_BTN_IMAGE_FILENAME[i]);

		if (0 != g_access ((const gchar *)image_path, R_OK))
        {
            image_path = ot_sys_g_free (image_path);
            image_path = g_strdup_printf (INST_PREFIX"/share/otak-dbmanager/%s", ACT_BTN_IMAGE_FILENAME[i]);
            if (0 != g_access ((const gchar *)image_path, R_OK))
            {
                image_path = ot_sys_g_free (image_path);
                image_path = g_strdup_printf ("%s/%s", image_dir, ACT_BTN_IMAGE_FILENAME[i]);
            }
        }

        gtk_image_set_from_file (GTK_IMAGE (image), image_path);

        g_free (image_path);
        image_path = NULL;
    }
}

gchar*
odm_sql_editor_dup_full_text (OdmSqlEditor *editor)
{
    GtkTextBuffer   *buf        = gtk_text_view_get_buffer (GTK_TEXT_VIEW (editor->edt_textview));
    GtkTextIter      start_iter;
    GtkTextIter      end_iter;

    gtk_text_buffer_get_start_iter (buf, &start_iter);
    gtk_text_buffer_get_end_iter (buf, &end_iter);

    return gtk_text_buffer_get_text (buf, &start_iter, &end_iter, FALSE);
}
